Package: tinyarray
Type: Package
Title: Expression Data Analysis and Visualization
Version: 2.4.3
Authors@R: person("Xiaojie", "Sun", email = "18763899370@163.com", role = c("aut", "cre"))
Maintainer: Xiaojie Sun <18763899370@163.com>
Description: The Gene Expression Omnibus (<https://www.ncbi.nlm.nih.gov/geo/>) and The Cancer Genome Atlas (<https://portal.gdc.cancer.gov/>) are widely used medical public databases. Our platform integrates routine analysis and visualization tools for expression data to provide concise and intuitive data analysis and presentation.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: BiocManager, clusterProfiler, dplyr, limma, stringr, tibble,
        pheatmap, ggplot2, survival, Hmisc, survminer, patchwork
Suggests: testthat, AnnoProbe, GEOquery, Biobase, VennDiagram,
        FactoMineR, factoextra, knitr, rmarkdown, cowplot, ggpubr,
        ggplotify, tidyr, labeling, Rtsne, scatterplot3d,
        ComplexHeatmap, circlize,org.Rn.eg.db,org.Mm.eg.db,org.Hs.eg.db
URL: https://github.com/xjsun1221/tinyarray
BugReports: https://github.com/xjsun1221/tinyarray/issues
Depends: R (>= 4.1.0)
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-03-05 09:01:31 UTC; xiaojie
Author: Xiaojie Sun [aut, cre]
Repository: CRAN
Date/Publication: 2025-03-05 13:20:02 UTC
Built: R 4.6.0; ; 2026-05-01 06:16:02 UTC; unix

Package: tinyarray
Type: Package
Title: Expression Data Analysis and Visualization
Version: 3.0.0
Authors@R: person("Xiaojie", "Sun", email = "18763899370@163.com", role = c("aut", "cre"))
Maintainer: Xiaojie Sun <18763899370@163.com>
Description: A toolkit for microarray and RNA-seq data analysis, including annotation conversion, differential expression, enrichment, survival analysis, and visualization.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: Hmisc, dplyr, ggplot2, limma, patchwork, pheatmap, stringr,
        curl, httr2, survival, survminer, tibble
Suggests: testthat, AnnoProbe, GEOquery, Biobase, VennDiagram,
        FactoMineR, factoextra, knitr, rmarkdown, cowplot, ggpubr,
        ggplotify, tidyr, labeling, Rtsne, scatterplot3d,
        ComplexHeatmap, circlize, AnnotationDbi, BiocManager,
        clusterProfiler, org.Rn.eg.db,org.Mm.eg.db,org.Hs.eg.db
Depends: R (>= 4.3.0)
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-08-02 06:04:21 UTC; 18763
Author: Xiaojie Sun [aut, cre]
Repository: CRAN
Date/Publication: 2026-08-02 06:20:02 UTC

# Generated by roxygen2: do not edit by hand

export(box_surv)
export(cor.full)
export(cor.one)
export(corheatmap)
export(corscatterplot)
export(double_enrich)
export(draw_KM)
export(draw_boxplot)
export(draw_heatmap)
export(draw_heatmap2)
export(draw_pca)
export(draw_tsne)
export(draw_venn)
export(draw_volcano)
export(draw_volcano2)
export(dumd)
export(edges_to_nodes)
export(exp_boxplot)
export(exp_surv)
export(find_anno)
export(geo_download)
export(geo_parser)
export(get_cgs)
export(get_count_txt)
export(get_deg)
export(get_deg_all)
export(get_gpl_txt)
export(ggheat)
export(hypertest)
export(interaction_to_edges)
export(intersect_all)
export(make_tcga_group)
export(match_exp_cl)
export(multi_deg)
export(multi_deg_all)
export(plcortest)
export(plot_deg)
export(point_cut)
export(quick_enrich)
export(risk_plot)
export(sam_filter)
export(surv_KM)
export(surv_cox)
export(t_choose)
export(trans_array)
export(trans_entrezexp)
export(trans_exp)
export(trans_exp_new)
export(union_all)
importFrom(BiocManager,install)
importFrom(Hmisc,rcorr)
importFrom(clusterProfiler,bitr)
importFrom(clusterProfiler,dotplot)
importFrom(clusterProfiler,enrichGO)
importFrom(clusterProfiler,enrichKEGG)
importFrom(clusterProfiler,setReadable)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(dplyr,union_all)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_boxplot)
importFrom(ggplot2,stat_ellipse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_light)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,ylim)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,topTable)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(patchwork,wrap_plots)
importFrom(pheatmap,pheatmap)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_split)
importFrom(stringr,str_starts)
importFrom(stringr,str_sub)
importFrom(stringr,str_to_lower)
importFrom(stringr,str_to_upper)
importFrom(stringr,str_wrap)
importFrom(survival,Surv)
importFrom(survival,coxph)
importFrom(survival,survdiff)
importFrom(survival,survfit)
importFrom(survminer,ggsurvplot)
importFrom(survminer,surv_cutpoint)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(utils,packageDescription)

README
================

- [1 0. Install the package](#1-0-install-the-package)
- [2 1. Simplified GEO microarray
  workflow](#2-1-simplified-geo-microarray-workflow)
- [3 2. Common small utilities](#3-2-common-small-utilities)
- [4 3. Download GEO-provided RNA-seq count
  data](#4-3-download-geo-provided-rna-seq-count-data)
- [5 4. Expression matrix conversion](#5-4-expression-matrix-conversion)
  - [5.1 4.1 trans_array](#51-41-trans_array)
  - [5.2 4.2 trans_exp_new](#52-42-trans_exp_new)
  - [5.3 4.3 trans_ensembl_exp](#53-43-trans_ensembl_exp)
  - [5.4 4.4 trans_entrezexp](#54-44-trans_entrezexp)
  - [5.5 4.5 TCGA grouping](#55-45-tcga-grouping)
- [6 5. Common plots](#6-5-common-plots)
  - [6.1 5.1 Volcano plot](#61-51-volcano-plot)
  - [6.2 5.2 PCA, tSNE, heatmap, and
    Venn](#62-52-pca-tsne-heatmap-and-venn)
  - [6.3 5.3 Boxplots and correlation
    plots](#63-53-boxplots-and-correlation-plots)
  - [6.4 5.4 Combined plots](#64-54-combined-plots)
- [7 6. Survival analysis](#7-6-survival-analysis)
- [8 7. Network and statistics
  utilities](#8-7-network-and-statistics-utilities)
- [9 8. Enrichment analysis](#9-8-enrichment-analysis)
- [10 9. Updates](#10-9-updates)
  - [10.1 9.1 New functions](#101-91-new-functions)
  - [10.2 9.2 Function updates](#102-92-function-updates)

✍️ Author: Xiaojie Sun

🌟 Role: Full-time lecturer at “Bioinformatics Skill Tree” for eight
years, co-founder of the WeChat public account “Bioinformatics Planet”,
and co-author of an SCI-indexed paper with an impact factor of
approximately 8. Co-editor of the second edition of *Decoding Life*;
upcoming book: *Practical Bioinformatics Data Analysis with R*.

🎓 Focused on bioinformatics data analysis training for beginners and
hands-on teaching.

tinyarray is an R package designed to simplify downstream GEO/TCGA
analysis. It integrates data download, ID conversion, differential
expression analysis, survival analysis, and commonly used bioinformatics
visualization. Many of the functions in this package were originally
written to meet my own daily analysis, teaching, and Q&A needs, and to
solve practical pain points.

# 1 0. Install the package

tinyarray depends on several Bioconductor packages, and
`install.packages()` cannot install all of them directly. You can
install those dependencies first and then install tinyarray.

``` r
if(!requireNamespace("BiocManager", quietly = TRUE))install.packages("BiocManager")
pkg_list <- c("limma", "GEOquery", "Biobase", "ComplexHeatmap",  "AnnotationDbi", "clusterProfiler", "org.Rn.eg.db", "org.Mm.eg.db",  "org.Hs.eg.db")
BiocManager::install(pkg_list,update = F,ask = F)
```

``` r
if(!require(tinyarray))install.packages("tinyarray")
```

# 2 1. Simplified GEO microarray workflow

Here we use `GSE7305` as an example and walk through downloading,
grouping, annotating, and differential expression analysis in order.

``` r
library(tinyarray)
library(stringr)
```

``` r
# Download data; use the local cached example when available so the tutorial
# can still render in offline environments.
if (file.exists("GSE7305_eSet.rds")) {
  geo_cached <- readRDS("GSE7305_eSet.rds")
  if (is.list(geo_cached) && length(geo_cached) == 1L && inherits(geo_cached[[1]], "ExpressionSet")) {
    geo_eset <- geo_cached[[1]]
    geo1 <- list(
      exp = Biobase::exprs(geo_eset),
      pd = Biobase::pData(geo_eset),
      gpl = Biobase::annotation(geo_eset)
    )
  } else {
    geo1 <- geo_cached
  }
} else {
  geo1 <- geo_download("GSE7305", destdir = tempdir())
}
```

    ## you can also use getGEO from GEOquery, by 
    ## getGEO("GSE7305", destdir=".", AnnotGPL = F, getGPL = F)

    ## 54675 probes, 20 samples from 5.020951 to 22011.934

``` r
if (is.null(geo1$group_candidates)) {
  geo1$group_candidates <- auto_geo_group(geo1$pd)
}
```

`geo_download()` returns a list with the following elements:

- `exp`: expression matrix
- `pd`: phenotype data
- `gpl`: platform accession
- `group_candidates`: automatic grouping candidates

This is also the most common entry point for downstream differential
analysis, annotation conversion, and automatic grouping.

``` r
# Extract expression matrix
exp1 = geo1$exp
# Decide whether to log-transform based on the data range; values between 0 and 20 are usually already logged
if(max(exp1) >50)exp1 <- log2(exp1 + 1)
# Extract clinical information
pd1 <- geo1$pd
# Get grouping information; geo1 contains a group_candidates subset that stores candidate groups
geo1$group_candidates
```

    ## $`title_Disease-Normal`
    ##  [1] Disease Disease Disease Disease Disease Disease Disease Disease Disease
    ## [10] Disease Normal  Normal  Normal  Normal  Normal  Normal  Normal  Normal 
    ## [19] Normal  Normal 
    ## Levels: Disease Normal
    ## 
    ## $`title_Ovary-Normal`
    ##  [1] Ovary  Ovary  Ovary  Ovary  Ovary  Ovary  Ovary  Ovary  Ovary  Ovary 
    ## [11] Normal Normal Normal Normal Normal Normal Normal Normal Normal Normal
    ## Levels: Ovary Normal
    ## 
    ## $`source_name_ch1_G-E`
    ##  [1] E E E E E G G G G G E E E E E G G G G G
    ## Levels: G E
    ## 
    ## $`source_name_ch1_M-E`
    ##  [1] M M M M M E E E E E M M M M M E E E E E
    ## Levels: M E
    ## 
    ## $`source_name_ch1_N-E`
    ##  [1] E E E E E E E E E E N N N N N N N N N N
    ## Levels: N E
    ## 
    ## $`description_Luteal-Follicular`
    ##  [1] Follicular Follicular Luteal     Luteal     Follicular Follicular
    ##  [7] Follicular Follicular Follicular Follicular Follicular Follicular
    ## [13] Luteal     Luteal     Follicular Follicular Follicular Follicular
    ## [19] Follicular Follicular
    ## Levels: Luteal Follicular

``` r
# For example, select the first candidate as the grouping code
Group1 = geo1$group_candidates[[1]]

# If there is no suitable candidate group, create one manually
k = str_detect(pd1$title, "Disease");table(k)
```

    ## k
    ## FALSE  TRUE 
    ##    10    10

``` r
Group1 = ifelse(k, "Disease", "Normal")
Group1 = factor(Group1, levels = c("Normal", "Disease"))
table(Group1)
```

    ## Group1
    ##  Normal Disease 
    ##      10      10

``` r
# Get probe annotation; for some platforms this cannot be obtained automatically, and the GPL page needs to be downloaded and parsed manually
ids1 <- get_ids(geo1$gpl)
```

    ## 

    ## The following code was used to obtain the annotation table:

    ## library(hgu133plus2.db); ids <- toTable(hgu133plus2SYMBOL)

``` r
# Check whether the probe annotation is correct; if most values below are TRUE, the annotation is usually correct
table(ids1$probe_id %in% rownames(exp1))
```

    ## 
    ##  TRUE 
    ## 43086

``` r
head(ids1)
```

    ##    probe_id symbol
    ## 1 1007_s_at   DDR1
    ## 2   1053_at   RFC2
    ## 3    117_at  HSPA6
    ## 4    121_at   PAX8
    ## 5 1255_g_at GUCA1A
    ## 6   1294_at   UBA7

`get_ids()` returns a cleaned two-column table:

- first column: `probe_id`
- second column: `symbol`

It automatically does several things:

- Prefer an available Bioconductor annotation package;
- If none is available, try `AnnoProbe::idmap()`;
- Remove probes whose symbol is `NA` or an empty string `""`;
- Drop extra columns and keep only `probe_id` and `symbol`;
- Convert `probe_id` to character;
- Reset row names.

If `get_ids()` gives a warning like:

No annotation table is available for GPL5555. Please extract it manually
from the GPL platform table. You can use `get_gpl_txt("GPL5555")` to get
the download link for the GPL platform table.

it means that neither a Bioconductor package nor `AnnoProbe::idmap()`
could provide a valid annotation, and you need to extract it manually
from the GPL platform table.

You can use `get_gpl_txt()` to get the platform-table download link, for
example:

``` r
get_gpl_txt(geo1$gpl)
```

    ## https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GPL570&targ=self&form=text&view=data

``` r
get_gpl_txt("GPL6244")
```

    ## https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GPL6244&targ=self&form=text&view=data

Download and read that file, then extract the probe_id and symbol
columns and rename them to `"probe_id"` and `"symbol"`, and they can be
used in downstream analysis.

``` r
# Differential expression analysis
deg1 <- get_deg(exp1, Group1, ids1, logFC_cutoff = 1, entriz = FALSE)
head(deg1)
```

    ##      logFC  AveExpr        t      P.Value    adj.P.Val        B    probe_id
    ## 1 6.270309 8.436140 45.39552 9.106509e-24 2.489492e-19 41.58809   202992_at
    ## 2 3.943359 7.351799 35.25755 2.600407e-21 4.739242e-17 37.34483   204971_at
    ## 3 2.318498 6.631187 32.33367 1.785551e-20 1.952500e-16 35.77275   228564_at
    ## 4 4.905540 8.140399 30.78154 5.320673e-20 4.155826e-16 34.85700 208131_s_at
    ## 5 4.878195 6.815838 29.02740 1.950627e-19 1.333131e-15 33.74579   210002_at
    ## 6 4.106051 9.045949 28.82714 2.273193e-19 1.380965e-15 33.61341   212190_at
    ##      symbol change
    ## 1        C7     up
    ## 2      CSTA     up
    ## 3 LINC01116     up
    ## 4     PTGIS     up
    ## 5     GATA6     up
    ## 6  SERPINE2     up

``` r
# Check the number of DE genes
table(deg1$change)
```

    ## 
    ##   down stable     up 
    ##    576  19613    623

``` r
# Differential expression analysis + plots
dcp1 <- get_deg_all(exp1, Group1, ids1, logFC_cutoff = 2, entriz = TRUE)
```

    ## 

    ## 

    ## 'select()' returned 1:many mapping between keys and columns

    ## 138 down genes,130 up genes

``` r
dcp1$plots
```

# 3 2. Common small utilities

This group of functions mainly solves the most basic data-cleaning
problems. If the input is handled correctly first, the downstream
analysis becomes much easier.

- `dumd()` gives a quick look at an object’s structure.
- `intersect_all()` and `union_all()` handle intersections and unions of
  multiple sets.
- `interaction_to_edges()` and `edges_to_nodes()` convert between a
  “relation table” and a “node table”.
- `cor.full()` and `cor.one()` are used for correlation analysis.

``` r
dumd(iris)
```

    ##        colname count
    ## 3 Petal.Length    43
    ## 1 Sepal.Length    35
    ## 2  Sepal.Width    23
    ## 4  Petal.Width    22
    ## 5      Species     3

``` r
intersect_all(1:5, 3:8, 4:10)
```

    ## [1] 4 5

``` r
union_all(1:5, 3:8, 4:10)
```

    ##  [1]  1  2  3  4  5  6  7  8  9 10

``` r
df <- data.frame(
  a = c("gene1", "gene2", "gene3"),
  b = c("d,f,a,b", "c,e,g", "a,b,d")
)
df
```

    ##       a       b
    ## 1 gene1 d,f,a,b
    ## 2 gene2   c,e,g
    ## 3 gene3   a,b,d

``` r
interaction_to_edges(df)
```

    ##       a1 a2
    ## 1  gene1  d
    ## 2  gene1  f
    ## 3  gene1  a
    ## 4  gene1  b
    ## 5  gene2  c
    ## 6  gene2  e
    ## 7  gene2  g
    ## 8  gene3  a
    ## 9  gene3  b
    ## 10 gene3  d

``` r
edges_to_nodes(interaction_to_edges(df))
```

    ##     gene type
    ## 1  gene1   a1
    ## 2  gene2   a1
    ## 3  gene3   a1
    ## 4      d   a2
    ## 5      f   a2
    ## 6      a   a2
    ## 7      b   a2
    ## 8      c   a2
    ## 9      e   a2
    ## 10     g   a2

``` r
x <- iris[, -5]
cor.full(x)
```

    ##                                p.value        cor obsnumber
    ## Sepal.Length:Sepal.Width  1.518983e-01 -0.1175698       150
    ## Sepal.Length:Petal.Length 1.038667e-47  0.8717538       150
    ## Sepal.Length:Petal.Width  2.325498e-37  0.8179411       150
    ## Sepal.Width:Petal.Length  4.513314e-08 -0.4284401       150
    ## Sepal.Width:Petal.Width   4.073229e-06 -0.3661259       150
    ## Petal.Length:Petal.Width  4.675004e-86  0.9628654       150

``` r
cor.one(x, "Sepal.Width")
```

    ##                   p.value        cor obsnumber
    ## Sepal.Length 1.518983e-01 -0.1175698       150
    ## Petal.Length 4.513314e-08 -0.4284401       150
    ## Petal.Width  4.073229e-06 -0.3661259       150

The outputs are straightforward:

- `dumd()` gives a quick summary of the object so you can confirm the
  data type and dimensions first.
- `intersect_all()` and `union_all()` return the set-operation results.
- `interaction_to_edges()` outputs an edge table, and `edges_to_nodes()`
  extracts nodes from the edge table.
- `cor.full()` returns the overall correlation result, and `cor.one()`
  returns the correlation between one variable and the others.

# 4 3. Download GEO-provided RNA-seq count data

For selected human RNA-seq datasets, GEO provides reprocessed count
matrices with Entrez IDs as row names. These matrices can be used
directly for transcriptome-wide differential expression analysis.

``` r
get_count_txt("GSE162550")
```

    ## https://www.ncbi.nlm.nih.gov/geo/download/?type=rnaseq_counts&acc=GSE162550&format=file&file=GSE162550_raw_counts_GRCh38.p13_NCBI.tsv.gz

By default, this function only returns the download URL for the count
matrix provided by GEO. You can copy the link into a browser to download
it. If you want to save it locally, set `download = TRUE`. If the
network is unstable, the download may fail. If the given GSE accession
does not have an official count file, the URL is invalid and the
download will fail.

# 5 4. Expression matrix conversion

This section turns an “expression matrix with original IDs” into a
matrix that can be used directly in downstream analysis.

- `trans_array()` converts a probe-level expression matrix into a
  gene-level matrix, and it can also rename rows based on a custom
  old-to-new ID mapping table.
- `trans_exp_new()` can convert matrices with Ensembl or Entrez row
  names.
- `trans_entrezexp()` specifically converts Entrez ID matrices to
  symbols.
- `trans_ensembl_exp()` specifically converts Ensembl ID matrices to
  gene symbols.
- `make_tcga_group()` is used for TCGA sample grouping.

We first create a few example matrices.

``` r
# Example array expression matrix
exp_array <- matrix(1:20, nrow = 5)
rownames(exp_array) <- paste0("probe", 1:5)
colnames(exp_array) <- paste0("sample", 1:4)
show_mat(exp_array)
```

    ##        sample1 sample2 sample3 sample4
    ## probe1       1       6      11      16
    ## probe2       2       7      12      17
    ## probe3       3       8      13      18
    ## probe4       4       9      14      19

``` r
ids_array <- data.frame(
  probe_id = c("probe1", "probe2", "probe3", "probe4", "probe5"),
  symbol = c("A", "B", "B", "C", "C")
)
head(ids_array)
```

    ##   probe_id symbol
    ## 1   probe1      A
    ## 2   probe2      B
    ## 3   probe3      B
    ## 4   probe4      C
    ## 5   probe5      C

``` r
# Example Ensembl expression matrix
exp_ensembl <- matrix(rnorm(200), nrow = 20)
rownames(exp_ensembl) <- c(
  sample(mRNA_anno$gene_id, 10, replace = TRUE),
  sample(lnc_anno$gene_id, 10, replace = TRUE)
)
colnames(exp_ensembl) <- paste0("s", 1:10)
show_mat(exp_ensembl)
```

    ##                             s1         s2        s3          s4
    ## ENSG00000105519.12 -0.76670821 -0.6530894 2.1291767  2.64591554
    ## ENSG00000075043.17  0.89973925 -1.0277558 0.1924601 -0.48781338
    ## ENSG00000121289.17  0.31039150 -0.2447213 0.1996285  0.35487582
    ## ENSG00000235631.1   0.00426799  0.5441694 2.3501687  0.08757415

``` r
# Example Entrez expression matrix
gene_demo <- sample(genes, 20, replace = TRUE)
exp_entrez <- matrix(rnorm(length(gene_demo) * 6), ncol = 6)
rownames(exp_entrez) <- gene_demo
colnames(exp_entrez) <- paste0("s", 1:6)
show_mat(exp_entrez)
```

    ##                s1         s2         s3         s4
    ## 140679  0.3831703 -0.1183658 0.24566685 -0.5400305
    ## 51046   0.9888129  1.0435844 1.07239869  0.1997114
    ## 56853  -0.1362858  0.5830176 1.19305547  0.2940977
    ## 166752 -1.3732949 -0.2314533 0.07126605 -0.2882327

## 5.1 4.1 trans_array

`trans_array()` is suitable when you already have a `probe_id -> symbol`
mapping table. It rewrites the row names of the expression matrix
according to that table and returns a new matrix. If one symbol
corresponds to multiple probes, only one row is kept by default; if you
set `average = TRUE`, those duplicated entries are merged by mean.

``` r
trans_array(exp_array, ids_array)
```

    ## 3 rownames transformed after duplicate rows removed

    ##   sample1 sample2 sample3 sample4
    ## A       1       6      11      16
    ## B       2       7      12      17
    ## C       4       9      14      19

``` r
trans_array(exp_array, ids_array, average = TRUE)
```

    ## 3 rownames transformed after averaging duplicate rows

    ##   sample1 sample2 sample3 sample4
    ## A     1.0     6.0    11.0    16.0
    ## B     2.5     7.5    12.5    17.5
    ## C     4.5     9.5    14.5    19.5

The first result is a matrix that keeps only unique symbols, and the
second result is a matrix that averages duplicate mappings. Both can be
used for downstream differential expression analysis or plotting; they only differ
in how repeated information is handled. The other `trans_*()` functions
described below also support the `average` argument.

## 5.2 4.2 trans_exp_new

``` r
s1 = trans_exp_new(exp_ensembl)
```

    ## 'select()' returned 1:many mapping between keys and columns

    ## 16 rownames transformed after duplicate rows removed

``` r
show_mat(s1)
```

    ##                 s1         s2        s3          s4
    ## CAPS   -0.76670821 -0.6530894 2.1291767  2.64591554
    ## KCNQ2   0.89973925 -1.0277558 0.1924601 -0.48781338
    ## CEP89   0.31039150 -0.2447213 0.1996285  0.35487582
    ## RNF148  0.00426799  0.5441694 2.3501687  0.08757415

It also supports the `species` argument, which can be set to human,
mouse, or rat.

It also supports the `type` argument, which can handle Ensembl (default,
equivalent to `trans_ensembl_exp`) and Entrez (equivalent to
`trans_entrezexp`) IDs.

- type = “entrez”

``` r
s2 = trans_exp_new(exp_entrez,type = "entrez")
```

    ## 'select()' returned 1:1 mapping between keys and columns

    ## 19 rownames transformed after duplicate rows removed

``` r
show_mat(s2)
```

    ##                 s1         s2         s3         s4
    ## SLC32A1  0.3831703 -0.1183658 0.24566685 -0.5400305
    ## ST8SIA3  0.9888129  1.0435844 1.07239869  0.1997114
    ## CELF4   -0.1362858  0.5830176 1.19305547  0.2940977
    ## FREM3   -1.3732949 -0.2314533 0.07126605 -0.2882327

- type = “array” with an explicit `ids` table, which converts IDs just
  like `trans_array()`.

``` r
s3 = trans_exp_new(exp_array,type = "array",ids = ids_array)
```

    ## 3 rownames transformed after duplicate rows removed

``` r
show_mat(s3)
```

    ##   sample1 sample2 sample3 sample4
    ## A       1       6      11      16
    ## B       2       7      12      17
    ## C       4       9      14      19

## 5.3 4.3 trans_ensembl_exp

``` r
s4 = trans_ensembl_exp(exp_ensembl)
```

    ## 'select()' returned 1:many mapping between keys and columns

    ## 16 rownames transformed after duplicate rows removed

``` r
show_mat(s4)
```

    ##                 s1         s2        s3          s4
    ## CAPS   -0.76670821 -0.6530894 2.1291767  2.64591554
    ## KCNQ2   0.89973925 -1.0277558 0.1924601 -0.48781338
    ## CEP89   0.31039150 -0.2447213 0.1996285  0.35487582
    ## RNF148  0.00426799  0.5441694 2.3501687  0.08757415

## 5.4 4.4 trans_entrezexp

``` r
s5 = trans_entrezexp(exp_entrez)
```

    ## 'select()' returned 1:1 mapping between keys and columns

    ## 19 rownames transformed after duplicate rows removed

``` r
show_mat(s5)
```

    ##                 s1         s2         s3         s4
    ## SLC32A1  0.3831703 -0.1183658 0.24566685 -0.5400305
    ## ST8SIA3  0.9888129  1.0435844 1.07239869  0.1997114
    ## CELF4   -0.1362858  0.5830176 1.19305547  0.2940977
    ## FREM3   -1.3732949 -0.2314533 0.07126605 -0.2882327

## 5.5 4.5 TCGA grouping

`make_tcga_group()` can accept either an expression matrix or a vector
of sample names. It splits TCGA samples into `normal` and `tumor`
according to TCGA sample barcodes and returns a factor.

``` r
show_mat(exp_hub1)
```

    ##        GTEX-S33H-1226-SM-4AD69 GTEX-VJYA-0826-SM-4KL1M TCGA-FB-A545-01
    ## CXCL8                 34.00041                50.99857        1341.029
    ## FN1                  591.01914               960.00492      137998.164
    ## COL3A1              2226.94824              3682.08761      177553.205
    ## ISG15                125.00175               135.00350        1784.980
    ##        GTEX-ZF3C-2026-SM-4WWB5
    ## CXCL8                 388.9918
    ## FN1                  5218.1531
    ## COL3A1              15621.0916
    ## ISG15                 513.9898

``` r
make_tcga_group(exp_hub1[,1:10])
```

    ##  [1] normal normal tumor  normal normal normal tumor  tumor  tumor  tumor 
    ## Levels: normal tumor

``` r
make_tcga_group(colnames(exp_hub1)[1:10])
```

    ##  [1] normal normal tumor  normal normal normal tumor  tumor  tumor  tumor 
    ## Levels: normal tumor

The output of `make_tcga_group()` is a `normal` / `tumor` factor that
can be used directly for differential expression analysis.

# 6 5. Common plots

This chapter collects the most common visualization functions in one
place so that you can find the right function by plot type.

## 6.1 5.1 Volcano plot

`draw_volcano()` accepts a differential expression results table and
uses adjusted P values (`padj`) by default.

``` r
draw_volcano(deseq_data)
```

<img src="README_files/figure-gfm/volcano-1.png" alt="" style="display: block; margin: auto;" />

## 6.2 5.2 PCA, tSNE, heatmap, and Venn

These functions all belong to the category of “turn a matrix into a
plot”:

- `draw_pca()` performs principal component analysis.
- `draw_tsne()` performs tSNE dimensionality reduction.
- `draw_heatmap()` draws a heatmap.
- `draw_venn()` draws a Venn diagram.

``` r
draw_pca(t(iris[, 1:4]), iris$Species)
```

<img src="README_files/figure-gfm/pca-tsne-heatmap-1.png" alt="" style="display: block; margin: auto;" />

``` r
draw_pca(t(iris[, 1:4]), iris$Species, style = "ggplot2")
```

<img src="README_files/figure-gfm/pca-tsne-heatmap-2.png" alt="" style="display: block; margin: auto;" />

``` r
draw_pca(t(iris[, 1:4]), iris$Species, style = "3D")
```

<img src="README_files/figure-gfm/pca-tsne-heatmap-3.png" alt="" style="display: block; margin: auto;" />

``` r
exp_tsne <- matrix(rnorm(50 * 60), nrow = 50)
group_tsne <- factor(rep(c("A", "B"), each = 30))
draw_tsne(exp_tsne, group_tsne, perplexity = 15)
```

<img src="README_files/figure-gfm/pca-tsne-heatmap-4.png" alt="" style="display: block; margin: auto;" />

``` r
exp_heat <- matrix(abs(rnorm(60, sd = 16)), nrow = 10)
exp_heat[, 4:6] <- exp_heat[, 4:6] + 20
colnames(exp_heat) <- paste0("sample", 1:6)
rownames(exp_heat) <- paste0("gene", 1:10)
group_heat <- factor(rep(c("A", "B"), each = 3))

draw_heatmap(exp_heat, group_heat)
```

<img src="README_files/figure-gfm/pca-tsne-heatmap-5.png" alt="" style="display: block; margin: auto;" />

``` r
draw_heatmap(exp_heat, group_heat, split_column = TRUE, show_column_title = TRUE)
```

<img src="README_files/figure-gfm/pca-tsne-heatmap-6.png" alt="" style="display: block; margin: auto;" />

``` r
xset <- list(
  Deseq2 = sample(1:100, 30),
  edgeR = sample(1:100, 30),
  limma = sample(1:100, 30)
)
draw_venn(xset, "test")
draw_venn(xset, "test", color = c("darkgreen", "darkblue", "#B2182B"))
```

## 6.3 5.3 Boxplots and correlation plots

This group is better for looking at the distribution of one or a few
genes across different groups. `draw_boxplot()` and `exp_boxplot()`
mainly draw grouped boxplots, `corheatmap()` and `corscatterplot()` are
for correlation analysis, and `ggheat()` is a ggplot-style heatmap.

``` r
draw_boxplot(t(iris[, 1:4]), iris$Species)
```

<img src="README_files/figure-gfm/box-corr-1.png" alt="" style="display: block; margin: auto;" />

``` r
draw_boxplot(exp_heat, group_heat)
```

<img src="README_files/figure-gfm/box-corr-2.png" alt="" style="display: block; margin: auto;" />

``` r
exp_boxplot(exprSet_hub1)[[1]]
```

<img src="README_files/figure-gfm/box-corr-3.png" alt="" style="display: block; margin: auto;" />

``` r
xg <- rownames(exprSet_hub1)[1:3]
yg <- rownames(exprSet_hub1)[4:7]
corheatmap(exprSet_hub1, xg, yg)
```

<img src="README_files/figure-gfm/box-corr-4.png" alt="" style="display: block; margin: auto;" />

``` r
corscatterplot(iris, "Sepal.Length", "Sepal.Width")
```

<img src="README_files/figure-gfm/box-corr-5.png" alt="" style="display: block; margin: auto;" />

``` r
exp_heat_gg <- matrix(abs(rnorm(60, sd = 16)), nrow = 10)
exp_heat_gg[, 4:6] <- exp_heat_gg[, 4:6] + 20
rownames(exp_heat_gg) <- paste0("sample", 1:10)
colnames(exp_heat_gg) <- paste0("gene", 1:6)
group_heat_gg <- factor(rep(c("A", "B"), each = 5))

ggheat(exp_heat_gg, group_heat_gg)
```

<img src="README_files/figure-gfm/box-corr-6.png" alt="" style="display: block; margin: auto;" />

`draw_boxplot()` returns a boxplot for quickly comparing the expression
of a few genes across groups. `corheatmap()`
returns a correlation heatmap, and `corscatterplot()` returns a pairwise
scatter correlation plot.

## 6.4 5.4 Combined plots

`box_surv()` combines an expression boxplot and a survival plot, which
is useful for single-gene visualization. Rather than producing a
standalone plot, it places differences in expression and survival on the
same page for direct comparison.

``` r
box_surv(
  log2(exp_hub1+1),
  exprSet_hub1,
  meta1
)[[1]]
```

<img src="README_files/figure-gfm/comb-plot-1.png" alt="" style="display: block; margin: auto;" />

# 7 6. Survival analysis

This section revolves around KM curves, Cox regression, and optimal
cut-point estimation. `point_cut()` is used to find the cut point first,
`surv_KM()` and `surv_cox()` are the two most commonly used
survival-analysis entry points, `exp_surv()` bundles survival plots for
multiple genes, and `risk_plot()` is often used for risk-score display.

``` r
point_cut(exprSet_hub1, meta1)
```

    ## 8 rows with less than 5 values were ignored in cut.point calculations

    ## $CXCL8
    ## [1] 9.63068
    ## 
    ## $FN1
    ## [1] 18.12339
    ## 
    ## $COL3A1
    ## [1] 18.78103
    ## 
    ## $ISG15
    ## [1] 11.28338
    ## 
    ## $COL1A2
    ## [1] 18.63055
    ## 
    ## $CXCL10
    ## [1] 6.884411
    ## 
    ## $ICAM1
    ## [1] 12.27501
    ## 
    ## $MMP9
    ## [1] 10.58174

``` r
surv_KM(exprSet_hub1, meta1)
```

    ##     ICAM1 
    ## 0.0218004

``` r
surv_KM(exprSet_hub1, meta1, cut.point = TRUE)
```

    ## 8 rows with less than 5 values were ignored in cut.point calculations

    ##       CXCL8       ICAM1         FN1 
    ## 0.005832445 0.010023406 0.038235032

``` r
surv_cox(exprSet_hub1, meta1)
```

    ##             coef        se         z          p        HR     HRse       HRz
    ## ICAM1 -0.4813703 0.2119765 -2.270866 0.02315511 0.6179361 0.130988 -2.916787
    ##               HRp    HRCILL    HRCIUL
    ## ICAM1 0.003536574 0.4078578 0.9362209

``` r
surv_cox(exprSet_hub1, meta1, cut.point = TRUE)
```

    ## 8 rows with less than 5 values were ignored in cut.point calculations

    ##             coef        se         z           p        HR      HRse       HRz
    ## CXCL8  1.1142476 0.4246572  2.623876 0.008693553 3.0472747 1.2940471  1.582071
    ## FN1    0.4730967 0.2304361  2.053049 0.040067814 1.6049566 0.3698400  1.635725
    ## ICAM1 -0.5388259 0.2118283 -2.543692 0.010968763 0.5834328 0.1235876 -3.370624
    ##                HRp    HRCILL    HRCIUL
    ## CXCL8 0.1136333125 1.3256923 7.0045535
    ## FN1   0.1018971198 1.0216819 2.5212208
    ## ICAM1 0.0007499825 0.3851965 0.8836889

``` r
tmp <- exp_surv(exprSet_hub1, meta1)
patchwork::wrap_plots(tmp) + patchwork::plot_layout(guides = "collect")
```

<img src="README_files/figure-gfm/survival-1.png" alt="" style="display: block; margin: auto;" />

``` r
box_surv(exprSet_hub1, exprSet_hub1, meta1)[[1]]
```

<img src="README_files/figure-gfm/survival-2.png" alt="" style="display: block; margin: auto;" />

``` r
risk_plot(exprSet_hub1, meta1, riskscore = rnorm(nrow(meta1)))
```

<img src="README_files/figure-gfm/survival-3.png" alt="" style="display: block; margin: auto;" />

``` r
lung_demo <- survival::lung
lung_demo$event <- lung_demo$status == 2
lung_demo$sex <- factor(lung_demo$sex, levels = c(1, 2), labels = c("male", "female"))
draw_KM(meta = lung_demo, group_list = lung_demo$sex, event_col = "event")
```

<img src="README_files/figure-gfm/survival-4.png" alt="" style="display: block; margin: auto;" />

Most outputs here are plot objects. `surv_KM()` returns a Kaplan-Meier
survival curve, `surv_cox()` returns a Cox-analysis plot or table, and
`exp_surv()` returns a plot list that can be combined. `draw_KM()` is
more of a general-purpose survival curve function: as long as you
provide an event column and a grouping column, it can draw the plot.

# 8 7. Network and statistics utilities

This group of functions mainly serves lncRNA-miRNA-mRNA network
analysis. `plcortest()` first screens for correlated lncRNA-mRNA pairs,
after which `hypertest()` uses a hypergeometric test to assess shared
miRNAs. The former returns a list, and the latter returns a result table.

``` r
lnc_exp <- matrix(rnorm(15), nrow = 3)
colnames(lnc_exp) <- paste0("S", 1:5)
rownames(lnc_exp) <- paste0("lnc", 1:3)

mRNA_exp <- matrix(rnorm(25), nrow = 5)
colnames(mRNA_exp) <- paste0("S", 1:5)
rownames(mRNA_exp) <- paste0("gene", 1:5)

plcortest(lnc_exp, mRNA_exp, cor_cutoff = 0)
```

    ## $lnc1
    ## logical(0)
    ## 
    ## $lnc2
    ## logical(0)
    ## 
    ## $lnc3
    ## logical(0)

``` r
lnctarget <- data.frame(
  lncRNA = c("lnc1", "lnc1", "lnc2"),
  miRNA = c("miR1", "miR2", "miR2")
)

pctarget <- data.frame(
  mRNA = c("gene1", "gene2", "gene2"),
  miRNA = c("miR1", "miR2", "miR3")
)

hypertest(c("lnc1", "lnc2"), c("gene1", "gene2"), deMIR = c("miR2"), lnctarget = lnctarget, pctarget = pctarget)
```

    ##   lncRNAs Genes Counts listTotal popHits popTotal foldEnrichment
    ## 1    lnc1 gene1      1         2       1        3            1.5
    ## 4    lnc2 gene2      1         1       2        3            1.5
    ## 2    lnc1 gene2      1         2       2        3           0.75
    ##         hyperPValue miRNAs deMIRCounts deMIRs
    ## 1 0.666666666666667   miR1           0       
    ## 4 0.666666666666667   miR2           1   miR2
    ## 2                 1   miR2           1   miR2

`plcortest()` returns a list named by lncRNA, and each element contains
the mRNA names correlated with it. `hypertest()` returns a data.frame
containing the lncRNA and mRNA identifiers, the number of shared miRNAs,
fold enrichment, and the P value from the hypergeometric test.

# 9 8. Enrichment analysis

This section mainly covers KEGG and GO enrichment. `quick_enrich()`
returns a list that usually contains `kk`, `go`, and the corresponding
dotplot objects; if an enrichment analysis returns no enriched terms,
the function reports this instead of attempting to draw an empty plot.
`double_enrich()` is suitable for enriching up- and down-regulated genes
separately and comparing them in one figure.

``` r
g <- quick_enrich(genes)
```

    ## Reading KEGG annotation online: "https://rest.kegg.jp/link/hsa/pathway"...

    ## Reading KEGG annotation online: "https://rest.kegg.jp/list/pathway/hsa"...

    ## 'select()' returned 1:1 mapping between keys and columns
    ## 'select()' returned 1:1 mapping between keys and columns

``` r
names(g)
```

    ## [1] "kk"     "go"     "kk.dot" "go.dot"

``` r
g$kk
```

    ## #
    ## # over-representation test
    ## #
    ## #...@organism     hsa 
    ## #...@ontology     KEGG 
    ## #...@keytype      ENTREZID 
    ## #...@gene     chr [1:511] "960" "3371" "10763" "51029" "2146" "2149" "2316" "7153" ...
    ## #...pvalues adjusted by 'BH' with cutoff <0.05 
    ## #...54 enriched terms found
    ## 'data.frame':    54 obs. of  14 variables:
    ##  $ category      : chr  NA "Human Diseases" "Organismal Systems" "Human Diseases" ...
    ##  $ subcategory   : chr  NA "Substance dependence" "Nervous system" "Substance dependence" ...
    ##  $ ID            : chr  "hsa04082" "hsa05032" "hsa04727" "hsa05033" ...
    ##  $ Description   : chr  "Neuroactive ligand signaling" "Morphine addiction" "GABAergic synapse" "Nicotine addiction" ...
    ##  $ GeneRatio     : chr  "30/236" "20/236" "19/236" "14/236" ...
    ##  $ BgRatio       : chr  "199/9421" "91/9421" "89/9421" "41/9421" ...
    ##  $ RichFactor    : num  0.151 0.22 0.213 0.341 0.114 ...
    ##  $ FoldEnrichment: num  6.02 8.77 8.52 13.63 4.56 ...
    ##  $ zScore        : num  11.47 11.94 11.43 12.99 9.21 ...
    ##  $ pvalue        : num  1.27e-15 5.72e-14 4.38e-13 5.04e-13 5.68e-12 ...
    ##  $ p.adjust      : num  3.25e-13 7.29e-12 3.22e-11 3.22e-11 2.90e-10 ...
    ##  $ qvalue        : num  2.39e-13 5.36e-12 2.36e-11 2.36e-11 2.13e-10 ...
    ##  $ geneID        : chr  "OPRK1/DRD1/ADCY2/GABRB2/GRM5/HTR2C/CHRM3/GABRG1/GRIN2A/GABRB3/PRKCB/HRH3/GNG3/SLC32A1/GNB5/GRIN3A/HTR2A/SLC17A6"| __truncated__ "DRD1/KCNJ3/ADCY2/GABRB2/GABRG1/GABRB3/PRKCB/PDE1A/GNG3/SLC32A1/GNB5/GNAO1/GABRD/PDE2A/KCNJ9/GABRA5/ADCY5/GABRA1/CACNA1B/GABRG2" "ADCY2/GABRB2/GABRG1/GABRB3/PRKCB/SLC12A5/GNG3/SLC32A1/GNB5/GNAO1/GABRD/GLS2/GABRA5/ADCY5/GAD1/GABRA1/CACNA1B/GABRG2/GAD2" "GABRB2/GABRG1/GRIN2A/GABRB3/SLC32A1/GRIN3A/SLC17A6/SLC17A7/GABRD/GABRA5/GRIN2B/GABRA1/CACNA1B/GABRG2" ...
    ##  $ Count         : int  30 20 19 14 29 22 18 15 15 15 ...
    ## #...Citation
    ## S Xu, E Hu, Y Cai, Z Xie, X Luo, L Zhan, W Tang, Q Wang, B Liu, R Wang, W Xie, T Wu, L Xie, G Yu. Using clusterProfiler to characterize multiomics data. Nature Protocols. 2024, 19(11):3292-3320

``` r
g$go
```

    ## #
    ## # over-representation test
    ## #
    ## #...@organism     Homo sapiens 
    ## #...@ontology     GOALL 
    ## #...@keytype      ENTREZID 
    ## #...@gene     chr [1:511] "960" "3371" "10763" "51029" "2146" "2149" "2316" "7153" ...
    ## #...pvalues adjusted by 'BH' with cutoff <0.05 
    ## #...564 enriched terms found
    ## 'data.frame':    564 obs. of  13 variables:
    ##  $ ONTOLOGY      : chr  "BP" "BP" "BP" "BP" ...
    ##  $ ID            : chr  "GO:0006836" "GO:0099504" "GO:0099003" "GO:0007269" ...
    ##  $ Description   : chr  "neurotransmitter transport" "synaptic vesicle cycle" "vesicle-mediated transport in synapse" "neurotransmitter secretion" ...
    ##  $ GeneRatio     : chr  "38/466" "38/466" "40/466" "31/466" ...
    ##  $ BgRatio       : chr  "216/18860" "220/18860" "259/18860" "154/18860" ...
    ##  $ RichFactor    : num  0.176 0.173 0.154 0.201 0.201 ...
    ##  $ FoldEnrichment: num  7.12 6.99 6.25 8.15 8.15 ...
    ##  $ zScore        : num  14.4 14.2 13.5 14.2 14.2 ...
    ##  $ pvalue        : num  1.05e-21 2.06e-21 1.23e-20 1.06e-19 1.06e-19 ...
    ##  $ p.adjust      : num  3.98e-18 3.98e-18 1.59e-17 8.20e-17 8.20e-17 ...
    ##  $ qvalue        : num  3.30e-18 3.30e-18 1.32e-17 6.79e-17 6.79e-17 ...
    ##  $ geneID        : chr  "GFAP/SLC6A15/DRD1/RIMS3/CAMK2A/MEF2C/SYT4/SV2B/SNCG/NAPB/PRKCB/SYT13/SYT7/HRH3/PCLO/RIMS1/SYN2/SYT1/STXBP5/PPFI"| __truncated__ "RIMS3/ATP6V1G2/SYT4/SV2B/SNCG/NAPB/PRKCB/SYT13/DNM3/SYT7/PCLO/RIMS1/SYN2/SYT1/STXBP5/PPFIA2/RAPGEF4/SLC32A1/SLC"| __truncated__ "RIMS3/ATP6V1G2/SYT4/SV2B/SNCG/NAPB/AAK1/PRKCB/SYT13/DNM3/SYT7/PCLO/RIMS1/SYN2/SYT1/STXBP5/PPFIA2/RAPGEF4/SLC32A"| __truncated__ "RIMS3/CAMK2A/MEF2C/SYT4/SV2B/SNCG/NAPB/PRKCB/SYT13/SYT7/HRH3/PCLO/RIMS1/SYN2/SYT1/STXBP5/PPFIA2/SLC32A1/UNC13C/"| __truncated__ ...
    ##  $ Count         : int  38 38 40 31 31 39 41 47 27 23 ...
    ## #...Citation
    ## S Xu, E Hu, Y Cai, Z Xie, X Luo, L Zhan, W Tang, Q Wang, B Liu, R Wang, W Xie, T Wu, L Xie, G Yu. Using clusterProfiler to characterize multiomics data. Nature Protocols. 2024, 19(11):3292-3320

``` r
g$kk.dot
```

<img src="README_files/figure-gfm/unnamed-chunk-12-1.png" alt="" style="display: block; margin: auto;" />

``` r
g$go.dot
```

<img src="README_files/figure-gfm/unnamed-chunk-12-2.png" alt="" style="display: block; margin: auto;" />

``` r
deg_enrich <- data.frame(
  ENTREZID = genes[1:20],
  change = rep(c("up", "down"), 10)
)

double_enrich(deg_enrich)
```

    ## 'select()' returned 1:1 mapping between keys and columns
    ## 'select()' returned 1:1 mapping between keys and columns
    ## 'select()' returned 1:1 mapping between keys and columns
    ## 'select()' returned 1:1 mapping between keys and columns

    ## <tinyarray_double_enrich>
    ## components: kp, gp

You can first use `names(g)` to inspect the structure of
`quick_enrich()` output, and then check `g$kk` and `g$go` separately.
`double_enrich()` returns an object with a custom class, and its print
method summarizes the available components.

# 10 9. Updates

This version focuses on reducing manual data wrangling, improving the
reliability of annotation conversion, and making the plotting workflow
more convenient. The new functions are introduced first, followed by
changes to existing functions.

## 10.1 9.1 New functions

- `get_ids()` retrieves the platform annotation table and returns a
  standardized two-column table with the fixed column names `probe_id`
  and `symbol`. The result can be used directly for expression-matrix
  conversion, differential expression analysis, and enrichment analysis.

## 10.2 9.2 Function updates

- `geo_download()` now returns `group_candidates` in addition to `exp`,
  `pd`, and `gpl`. This makes it easier to perform automatic grouping or
  manually select grouping information after downloading GEO data.

- `get_deg()` and `get_deg_all()` now support `ids = NULL`. If the input
  is already a gene-level expression matrix, you can run differential
  expression analysis directly without providing a probe-to-gene
  mapping table.

- `trans_array()`, `trans_exp_new()`, and `trans_entrezexp()` now
  support `average = TRUE`. When multiple probes or IDs map to the same
  `symbol`, their expression values can be averaged so that the
  resulting gene symbols remain unique.

- `draw_volcano()` uses adjusted P values (`padj`) by default. To use raw
  P values (`P.value`) instead, explicitly set `adjust = FALSE`.

- `get_deg_all()` now issues a warning and skips the heatmap when there
  are no differential genes available for plotting, instead of stopping
  with an error.

- `make_tcga_group()` can now accept either an expression matrix or a
  vector of sample names, which makes it more flexible to use.

##' interaction_to_edges
##'
##' Split interactions by `sep` and return an edges data.frame.
##'
##' @param df interactions data.frame
##' @param a column to replicate
##' @param b column to split
##' @param sep a character string to separate b column
##' @export
##' @return a new data.frame with two columns, one interaction per row
##' @author Xiaojie Sun
##' @examples
##' df = data.frame(a = c("gene1","gene2","gene3"),
##' b = c("d,f,a,b",
##' "c,e,g",
##' "a,b,d"))
##' interaction_to_edges(df)
##' @seealso
##' \code{\link{edges_to_nodes}}

interaction_to_edges = function(df,a = 1,b = 2,sep = ","){
  gs = strsplit(as.character(df[,b]),sep,fixed = TRUE)
  edges = data.frame(a1 = rep(df[,a],times = sapply(gs,length)),
                     a2 = unlist(gs),stringsAsFactors = FALSE)
  unique(edges)
}
utils::globalVariables(c("a1","a2"))

##' edges_to_nodes
##'
##' get nodes from edges
##'
##' @param edges data.frame
##' @export
##' @return nodes data.frame
##' @author Xiaojie Sun
##' @examples
##' df = data.frame(a = c("gene1","gene2","gene3"),
##' b = c("d,f,a,b",
##' "c,e,g",
##' "a,b,d"))
##' edges = interaction_to_edges(df)
##' nodes = edges_to_nodes(edges)
##' @seealso
##' \code{\link{interaction_to_edges}}

edges_to_nodes = function(edges){
  if(ncol(edges) < 2)stop("edges must have at least two columns")
  if(!is.null(colnames(edges))){
    m = colnames(edges)
  }else{
    m = c("A1","A2")
  }
  a = unique(edges[,1])
  b = unique(edges[,2])
  nodes = data.frame(gene = c(a,b),
                     type = c(rep(m[1],times = length(a)),
                              rep(m[2],times = length(b))))
  return(nodes)
}

##' get_deg_all
##'
##' Do differential analysis according to an expression set and group
##' information.
##'
##' @inheritParams draw_pca
##' @inheritParams draw_volcano
##' @inheritParams draw_heatmap
##' @inheritParams multi_deg_all
##' @param ids optional probe-to-symbol annotation table. If `NULL`, `exp` is
##'   treated as a gene-level matrix and no ID conversion is performed.
##' @param entriz logical, if TRUE, convert symbols to Entrez IDs.
##' @return a list with a deg data.frame, volcano plot, PCA plot, heatmap, and
##'   a list with DEGs.
##' @author Xiaojie Sun
##' @importFrom patchwork wrap_plots
##' @importFrom ggplot2 ggsave
##' @importFrom stringr str_split
##' @export
##' @examples
##' \dontrun{
##' if(requireNamespace("Biobase",quietly = TRUE)&
##'    requireNamespace("AnnoProbe",quietly = TRUE)){
##'   gse = "GSE42872"
##'   geo = geo_download(gse,destdir=tempdir())
##'   group_list = rep(c("A","B"),each = 3)
##'   group_list = factor(group_list)
##'   find_anno(geo$gpl)
##'   ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
##'   dcp = get_deg_all(geo$exp,group_list,ids,entriz = FALSE)
##'   head(dcp$deg)
##'   dcp$plots
##' }else{
##'   if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
##'     warning("Package 'AnnoProbe' needed for this function to work.
##'          Please install it by install.packages('AnnoProbe')",call. = FALSE)
##'   }
##'   if(!requireNamespace("Biobase",quietly = TRUE)) {
##'     warning("Package 'Biobase' needed for this function to work.
##'          Please install it by BiocManager::install('Biobase')",call. = FALSE)
##'   }
##' }
##' }
##' @seealso
##' \code{\link{get_deg}};\code{\link{multi_deg_all}}

get_deg_all <- function(exp,
                        group_list,
                        ids = NULL,
                        symmetry = TRUE,
                        my_genes = NULL,
                        show_rownames = FALSE,
                        cluster_cols = TRUE,
                        color_volcano = c("#2874C5", "grey", "#f87669"),
                        logFC_cutoff=1,
                        pvalue_cutoff=0.05,
                        adjust = TRUE,
                        entriz = TRUE,
                        n_cutoff = 2,
                        annotation_legend = FALSE,
                        lab = NA,
                        change_col = "change",
                        species = "human") {
  if(nlevels(group_list)==2){
    deg <-  get_deg(exp,group_list,ids,
                    logFC_cutoff=logFC_cutoff,
                    pvalue_cutoff=pvalue_cutoff,
                    adjust = adjust,
                    entriz = entriz,
                    species = species)
    if (is.null(deg)) {
      return(NULL)
    }
    cgs = get_cgs(deg)
    volcano_plot = draw_volcano(deg,pkg=4,
                                lab =lab,
                                pvalue_cutoff = pvalue_cutoff,
                                logFC_cutoff=logFC_cutoff,
                                adjust = adjust,
                                change_col = change_col,
                                symmetry = symmetry)
    pca_plot = draw_pca(exp,group_list)
    heatmap = draw_heatmap2(exp,group_list,deg,my_genes,
                            show_rownames = show_rownames,
                            n_cutoff = n_cutoff,
                            cluster_cols = cluster_cols,
                            annotation_legend=annotation_legend)
    if(as.numeric(grDevices::dev.cur())!=1) grDevices::graphics.off()
    result = list(
      deg = deg,
      cgs = cgs,
      plots = .tinyarray_bundle_plots(heatmap, pca_plot, volcano_plot)
    )
    message(paste0(nrow(cgs$deg$down)," down genes,",nrow(cgs$deg$up)," up genes"))
  }else{
    result <- multi_deg_all(exp,
                            group_list,
                            ids,
                            logFC_cutoff = logFC_cutoff,
                            pvalue_cutoff = pvalue_cutoff,
                            symmetry = symmetry,
                            my_genes = my_genes,
                            show_rownames = show_rownames,
                            cluster_cols = cluster_cols,
                            color_volcano = color_volcano,
                            adjust = adjust,
                            entriz = entriz,
                            change_col = change_col,
                            species = species)
  }
  return(result)
}

##' point_cut
##'
##' calculate cut point for multiple genes
##'
##' @param exprSet_hub a tumor expression set for hubgenes
##' @param meta meta data corresponds to expression set
##' @importFrom survminer surv_cutpoint
##' @export
##' @return a vector with cutpoint for genes
##' @author Xiaojie Sun
##' @examplesIf requireNamespace("survival", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE)
##' point_cut(exprSet_hub1,meta1)
##' @seealso
##' \code{\link{surv_KM}};\code{\link{surv_cox}}

point_cut <- function(exprSet_hub,meta){
  if(ncol(exprSet_hub)!=nrow(meta))stop("your exprSet_hub is not corresponds to meta")
  kp = apply(exprSet_hub, 1, function(x)length(unique(unique(x))) > 4)
  message(paste0(sum(kp)," rows with less than 5 values were ignored in cut.point calculations"))
  exprSet_hub = exprSet_hub[kp,]
  dat = cbind(t(exprSet_hub),meta)
  cut.point = c()
  for(i in 1:nrow(exprSet_hub)){
    cut = surv_cutpoint(
      dat,
      time = "time",
      event = "event",
      variables = rownames(exprSet_hub)[i]
    )
    cut.point[[i]] = cut[["cutpoint"]][1,1]
  }
  names(cut.point) = rownames(exprSet_hub)
  return(cut.point)
}

##' surv_KM
##'
##' calculate log_rank test p values for genes
##'
##' @inheritParams point_cut
##' @inheritParams t_choose
##' @param cut.point logical, whether to use \code{cut_point}; if FALSE, use
##'   the median as the cutoff.
##' @param min_gn Depending on the expression of a gene, there may be a large
##'   difference in sample counts between the two groups. If the smaller group
##'   contains less than 10 percent of all samples, the gene will be discarded.
##' @importFrom survival Surv
##' @importFrom survival survdiff
##' @export
##' @return  a vector with gene names and log_rank p value
##' @author Xiaojie Sun
##' @examplesIf requireNamespace("survival", quietly = TRUE)
##' surv_KM(exprSet_hub1,meta1)
##' surv_KM(exprSet_hub1,meta1,pvalue_cutoff = 1)
##' surv_KM(exprSet_hub1,meta1,cut.point = TRUE)
##' @seealso
##' \code{\link{point_cut}};\code{\link{surv_cox}}


surv_KM <- function(exprSet_hub,meta,cut.point = FALSE,pvalue_cutoff = 0.05,min_gn = 0.1){
  if(cut.point)cut_point = point_cut(exprSet_hub,meta)
  log_rank_p = c()
  for(i in 1:nrow(exprSet_hub)){
    gene = as.numeric(exprSet_hub[i,])
    if(cut.point){
      meta$group=ifelse(gene>cut_point[i],'high','low')
    }else{
      meta$group=ifelse(gene>stats::median(gene),'high','low')
    }
    data.survdiff=survdiff(Surv(time, event)~group,data=meta)
    log_rank_p[[i]] = 1 - stats::pchisq(data.survdiff$chisq, length(data.survdiff$n) - 1)
    nn = min(table(meta$group))<= min_gn * nrow(meta)
    if(nn) log_rank_p[[i]] = 1
  }
  if(is.list(log_rank_p)) log_rank_p = unlist(log_rank_p)
  names(log_rank_p) = rownames(exprSet_hub)
  log_rank_p=sort(log_rank_p)
  tp = log_rank_p[log_rank_p<pvalue_cutoff]
  return(tp)
  message(paste0(length(tp)," of ",nrow(exprSet_hub)," genes selected"))
}

##' surv_cox
##'
##' calculate cox p values and HR for genes
##'
##' @inheritParams surv_KM
##' @param HRkeep one of "all","protect"or"risk"
##' @param continuous logical, gene expression or gene expression group
##' @importFrom survival Surv
##' @importFrom survival coxph
##' @export
##' @return a matrix with gene names ,cox p value and HR
##' @author Xiaojie Sun
##' @examplesIf requireNamespace("survival", quietly = TRUE)
##' surv_cox(exprSet_hub1,meta1)
##' surv_cox(exprSet_hub1,meta1,cut.point = TRUE,continuous = TRUE)
##' surv_cox(exprSet_hub1,meta1,cut.point = TRUE,continuous = TRUE,pvalue_cutoff = 1)
##' @seealso
##' \code{\link{point_cut}};\code{\link{surv_KM}}


surv_cox <-function(exprSet_hub,meta,cut.point = FALSE,
                    pvalue_cutoff = 0.05,HRkeep = "all",
                    continuous = FALSE,min_gn = 0.1){
  if(cut.point) cut_point = point_cut(exprSet_hub,meta)
  cox_results <-list()
  for(i in 1:nrow(exprSet_hub)){
    if(continuous) {
      gene= as.numeric(exprSet_hub[i,])
      m=coxph(Surv(time, event)~gene, data =  meta)
    }else{
      gene= as.numeric(exprSet_hub[i,])
      if(cut.point){
        meta$group=ifelse(gene>cut_point[i],'high','low')
      }else{
        meta$group=ifelse(gene>stats::median(gene),'high','low')
      }
      meta$group = factor(meta$group,levels = c("low","high"))
      m=coxph(Surv(time, event)~group, data =  meta)
      }
    beta <- stats::coef(m)
    se <- sqrt(diag(stats::vcov(m)))
    HR <- exp(beta)
    HRse <- HR * se

    #summary(m)
    tmp <- cbind(coef = beta, se = se, z = beta/se, p = 1 - stats::pchisq((beta/se)^2, 1),
                 HR = HR, HRse = HRse,
                 HRz = (HR - 1) / HRse, HRp = 1 - stats::pchisq(((HR - 1)/HRse)^2, 1),
                 HRCILL = exp(beta - stats::qnorm(.975, 0, 1) * se),
                 HRCIUL = exp(beta + stats::qnorm(.975, 0, 1) * se))
    if(continuous){
      cox_results[[i]] = tmp['gene',]
    }else{
      cox_results[[i]] = tmp['grouphigh',]
      nn = min(table(meta$group))<= min_gn * nrow(meta)
      if(nn) cox_results[[i]] = rep(NA,times = length(cox_results[[i]]))
    }
  }
  cox_results = do.call(cbind,cox_results)
  cox_results=t(cox_results)
  rownames(cox_results)= rownames(exprSet_hub)
  nn2 = apply(cox_results,1,function(x){all(is.na(x))})
  cox_results = cox_results[!nn2,]
  k = cox_results[,4]<pvalue_cutoff

  if(sum(stats::na.omit(k))==1){
    tmp = matrix(cox_results[k,],nrow = 1)
    colnames(tmp) = colnames(cox_results)
    rownames(tmp) = rownames(cox_results)[k]
    cox_results = tmp
  }else{
    cox_results=cox_results[k,]
  }

  if(HRkeep == "all"){
    return(cox_results)
  }else if(HRkeep == "protect"){
    k = cox_results[,5]<1
    return(cox_results[k, , drop = FALSE])
  }else if(HRkeep == "risk"){
    k = cox_results[,5]>1
    return(cox_results[k, , drop = FALSE])
  }else if(!(HRkeep %in% c("all","protect","risk"))){
    stop('HRkeep should be one of "all","protect"or"risk"')
  }
}

##' draw PCA plots
##'
##' do PCA analysis and warning a PCA plot
##'
##' @param exp A numeric matrix
##' @param group_list A character or factor vector with one label per sample;
##'   character input will be converted to factor and unused levels will be
##'   dropped.
##' @param color color vector
##' @param addEllipses logical,add ellipses or not
##' @param style plot style,"default","ggplot2"and "3D"
##' @param color.label color legend label
##' @param title plot title
##' @param ... other parameters from \code{fviz_pca_ind}
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 scale_color_manual
##' @importFrom ggplot2 scale_fill_manual
##' @importFrom ggplot2 theme_classic
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 labs
##' @importFrom ggplot2 aes
##' @importFrom ggplot2 stat_ellipse
##' @return a PCA plot according to \code{exp} and grouped by \code{group_list}.
##' @author Xiaojie Sun
##' @export
##' @examplesIf requireNamespace("FactoMineR", quietly = TRUE) && requireNamespace("factoextra", quietly = TRUE)
##' draw_pca(t(iris[,1:4]),iris$Species)
##' #change color
##' draw_pca(t(iris[,1:4]),iris$Species,color = c("#E78AC3", "#A6D854", "#FFD92F"))
##'
##' @examplesIf requireNamespace("FactoMineR", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
##' draw_pca(t(iris[,1:4]),iris$Species,style = "ggplot2")
##'
##' @examplesIf requireNamespace("FactoMineR", quietly = TRUE) && requireNamespace("scatterplot3d", quietly = TRUE)
##' draw_pca(t(iris[,1:4]),iris$Species,style = "3D")



##' @seealso
##' \code{\link{draw_heatmap}};\code{\link{draw_volcano}};\code{\link{draw_venn}}

draw_pca <-  function(exp,group_list,
                      color = c("#2874C5","#f87669","#e6b707","#868686","#92C5DE","#F4A582","#66C2A5","#FC8D62","#8DA0CB","#E78AC3","#A6D854","#FFD92F","#E5C494","#B3B3B3"),
                      addEllipses = TRUE,
                      style = "default",
                      color.label = "Group",
                      title = "",
                      ...){
  if(!requireNamespace("FactoMineR",quietly = TRUE)) {
    stop("Package \"FactoMineR\" needed for this function to work.
         Please install it by install.packages('FactoMineR')",call. = FALSE)
  }
  p1 <-  all(apply(exp,2,is.numeric))
  if(!p1) stop("exp must be a numeric matrix")
  group_list <- .tinyarray_prepare_group_list_safe(group_list)
  if (is.null(group_list)) return(NULL)
  dat <- as.data.frame(t(exp))
  dat.pca <- FactoMineR::PCA(dat, graph = FALSE)
  col = color[1:length(levels(group_list))]
  if(style == "default"){
    if(!requireNamespace("factoextra",quietly = TRUE)) {
      stop("Package \"factoextra\" needed for this function to work.
         Please install it by install.packages('factoextra')",call. = FALSE)
    }
    factoextra::fviz_pca_ind(dat.pca,
                             geom.ind = "point",
                             col.ind = group_list,
                             addEllipses = addEllipses,
                             palette = col,
                             legend.title = "Groups",
                             title = title,
                             ...)
  }else if(style == "ggplot2"){
    pdat = data.frame(dat.pca[["ind"]][["coord"]],
                      Group = group_list)
    p = ggplot(pdat,aes(Dim.1,Dim.2))+
      geom_point(aes(Dim.1,Dim.2,fill = Group),
                 shape = 21,color = "black")+
      scale_color_manual(values = color[1:nlevels(group_list)])+
      scale_fill_manual(values = color[1:nlevels(group_list)])+
      theme_classic()+
      theme(legend.position = "top")+
      labs(color = color.label,fill = color.label,title = title)

    if(addEllipses) p = p +
      stat_ellipse(aes(color = Group,fill = Group),
                   geom = "polygon",
                   alpha = 0.3,
                   linetype = 2)
    return(p)
  }else if(style == "3D"){
    if(!requireNamespace("scatterplot3d",quietly = TRUE)) {
      stop("Package \"scatterplot3d\" needed for this function to work.
         Please install it by install.packages('scatterplot3d')",call. = FALSE)
    }
     colors = color[as.numeric(group_list)]
    pdat = data.frame(dat.pca[["ind"]][["coord"]],
                      Group = group_list)
    scatterplot3d::scatterplot3d(pdat[,1:3],
                                 color = "black",
                                 pch = 21,
                                 bg = colors,
                                 main = title)
    graphics::legend("bottom",col = "black",
           legend = levels(group_list),
           pt.bg =  color[1:nlevels(group_list)], pch = 21,
           inset = -0.2, xpd = TRUE,
           horiz = TRUE)
  }
}

if(getRversion() >= "2.15.1")  utils::globalVariables(c(".","Dim.1","Dim.2","Group","pdat"))



##' draw a heatmap plot
##'
##' draw a heatmap plot for an expression matrix grouped by \code{group_list};
##' if \code{scale = TRUE}, the matrix is scaled before plotting.
##'
##' @inheritParams draw_pca
##' @param n  A numeric matrix
##' @param scale_before deprecated parameter
##' @param n_cutoff numeric cutoff used after scaling; values outside
##'   \code{[-n_cutoff, n_cutoff]} are truncated before plotting.
##' @param annotation_legend logical,show annotation legend or not
##' @param color color for heatmap
##' @param color_an color for column annotation
##' @param legend logical,show legend or not
##' @param show_rownames logical,show rownames or not
##' @param scale logical,scale the matrix or not
##' @param main the title of the plot
##' @param split_column split column by group_list
##' @param show_column_title show column title or not
##' @param ... other parameters from pheatmap
##' @return a heatmap plot according to \code{n} and grouped by
##'   \code{group_list}.
##' @author Xiaojie Sun
##' @importFrom pheatmap pheatmap
##' @export
##' @examplesIf requireNamespace("ggplotify", quietly = TRUE) && requireNamespace("pheatmap", quietly = TRUE)
##' #example data
##' exp = matrix(abs(rnorm(60,sd = 16)),nrow = 10)
##' exp[,4:6] <- exp[,4:6]+20
##' colnames(exp) <- paste0("sample",1:6)
##' rownames(exp) <- paste0("gene",1:10)
##' exp[1:4,1:4]
##' group_list = factor(rep(c("A","B"),each = 3))
##' draw_heatmap(exp,group_list)
##' #use iris
##' n = t(iris[,1:4]);colnames(n) = 1:150
##' group_list = iris$Species
##' draw_heatmap(n,group_list)
##' draw_heatmap(n,group_list,color = colorRampPalette(c("green","black","red"))(100),
##'              color_an = c("red","blue","pink") )

##' @seealso
##' \code{\link{draw_pca}};\code{\link{draw_volcano}};\code{\link{draw_venn}}

draw_heatmap <-  function(n,
                          group_list,
                          scale_before = FALSE,
                          n_cutoff = 3,
                          legend = FALSE,
                          show_rownames = FALSE,
                          annotation_legend=FALSE,
                          split_column = FALSE,
                          show_column_title = FALSE,
                          color = grDevices::colorRampPalette(c("#2fa1dd", "white", "#f87669"))(100),
                          color_an = c("#2fa1dd","#f87669","#e6b707","#868686","#92C5DE","#F4A582","#66C2A5","#FC8D62","#8DA0CB","#E78AC3","#A6D854","#FFD92F","#E5C494","#B3B3B3"),
                          scale = TRUE,
                          main = NA,...){
  dots <- list(...)
  cluster_cols <- TRUE
  if ("cluster_cols" %in% names(dots)) {
    cluster_cols <- isTRUE(dots$cluster_cols)
    dots$cluster_cols <- NULL
  }
  if(!requireNamespace("ggplotify",quietly = TRUE)) {
    stop("Package 'ggplotify' needed for this function to work.
         Please install it by install.packages('ggplotify')",call. = FALSE)
  }
  group_list <- .tinyarray_prepare_group_list_safe(group_list)
  if (is.null(group_list)) return(NULL)
  if (!cluster_cols) {
    reordered <- .tinyarray_order_columns_by_group(n, group_list)
    n <- reordered$exp
    group_list <- reordered$group_list
  }
  n = as.data.frame(n)
  if(scale_before) {
    message("scale_before parameter is deprecated")
    scale_before = FALSE
  }
  p1 <-  all(apply(n,2,is.numeric))
  if(!p1) stop("non-numeric matrix detected")
  heatmap_cluster <- .tinyarray_safe_heatmap_clusters(
    nrow_value = nrow(n),
    ncol_value = ncol(n),
    cluster_rows = TRUE,
    cluster_cols = cluster_cols
  )
  if(split_column){
    if(!requireNamespace("circlize",quietly = TRUE)) {
      stop("Package 'circlize' needed for this function to work.
           Please install it by install.packages('circlize')",call. = FALSE)
    }
    if(!requireNamespace("ComplexHeatmap",quietly = TRUE)) {
      stop("Package 'ComplexHeatmap' needed for this function to work.
           Please install it by BiocManager::install('ComplexHeatmap')",call. = FALSE)
    }
    col_fun = circlize::colorRamp2(c(-n_cutoff, 0, n_cutoff), c(color[1], "white", color[100]))
    if(scale){
      n = t(scale(t(n)))
      n[n > n_cutoff] = n_cutoff
      n[n < - n_cutoff] = -n_cutoff
    }
    m0 = ComplexHeatmap::Heatmap(n,
                    column_split = group_list,
                    cluster_rows = heatmap_cluster$cluster_rows,
                    cluster_columns = heatmap_cluster$cluster_cols)
    column_title = suppressWarnings(names(ComplexHeatmap::column_order(m0)))
    top_annotation = ComplexHeatmap::HeatmapAnnotation(
      cluster = ComplexHeatmap::anno_block(gp = grid::gpar(fill = (color_an[1:nlevels(group_list)])[match(column_title,levels(group_list))]),
                           labels = column_title,
                           labels_gp = grid::gpar(col = "white", fontsize = 12)))
    if(show_column_title){
      p = ComplexHeatmap::Heatmap(n,name = " ",
                                  col = col_fun,
                                  top_annotation = top_annotation,
                                  column_split = group_list,
                                  cluster_rows = heatmap_cluster$cluster_rows,
                                  cluster_columns = heatmap_cluster$cluster_cols,
                                  show_heatmap_legend = legend ,
                                  border = sum(dim(n))< 150,
                                  rect_gp = grid::gpar(col = "grey", lwd = 1),
                                  show_column_names = FALSE,
                                  show_row_names = show_rownames)
    }else{
      p = ComplexHeatmap::Heatmap(n,name = " ",
                                  col = col_fun,
                                  top_annotation = top_annotation,
                                  column_split = group_list,
                                  cluster_rows = heatmap_cluster$cluster_rows,
                                  cluster_columns = heatmap_cluster$cluster_cols,
                                  show_heatmap_legend = legend ,
                                  border = sum(dim(n))< 150,
                                  rect_gp = grid::gpar(col = "grey", lwd = 1),
                                  show_column_names = FALSE,
                                  show_row_names = show_rownames ,
                                  column_title = NULL)
    }
  }else{
    annotation_col=data.frame(group=group_list)
    rownames(annotation_col)=colnames(n)
    col = color_an[1:length(levels(group_list))]
    ann_colors = list(group = col)
    if(is.null(names(ann_colors$group)))names(ann_colors$group)=levels(group_list)
    if(scale) {
      scale_row = "row"
      breaks = seq(-n_cutoff,n_cutoff,length.out = length(color))
    } else {
      scale_row = "none"
      breaks = NA
    }
    if(!scale_before){
      heatmap_breaks <- if (scale_row == "row") {
        breaks
      } else {
        .tinyarray_safe_heatmap_breaks(n, length(color))
      }
      p = do.call(
        pheatmap,
        c(
          list(
            n,
            show_colnames = FALSE,
            show_rownames = show_rownames,
            scale = scale_row,
            color = color,
            annotation_col = annotation_col,
            annotation_colors = ann_colors,
            breaks = heatmap_breaks,
            legend = legend,
            silent = TRUE,
            annotation_legend = annotation_legend,
            main = main,
            cluster_rows = heatmap_cluster$cluster_rows,
            cluster_cols = heatmap_cluster$cluster_cols
          ),
          dots
        )
      )
      p = ggplotify::as.ggplot(p)
    }
  }
  return(p)
}


##' draw a volcano plot
##'
##' warning a volcano plot for Differential analysis result in data.frame format.
##'
##' @param deg a data.frame created by Differential analysis
##' @param pvalue_cutoff Cutoff value of pvalue,0.05 by default.
##' @param logFC_cutoff Cutoff value of logFC,1 by default.
##' @param pkg a integer ,means which Differential analysis packages you used,we support three packages by now, 1,2,3,4 respectively means "DESeq2","edgeR","limma(voom)","limma"; pkg = 4 uses `log2FoldChange` as the default x-axis label.
##' @param adjust a logical value; if TRUE, use \code{padj} and plot
##'   \code{-log10(padj)}; otherwise use \code{P.value} and plot
##'   \code{-log10(P.value)}.
##' @param change_col column name for the up/down/status label. Default uses
##'   `change` when it exists; otherwise the function falls back to the
##'   existing automatic classification logic.
##' @param symmetry a logical value ,would you like to get your plot symmetrical
##' @param color color vector
##' @param lab label for  x axis in volcano plot; when `NA`, pkg = 4 uses
##'   `log2FoldChange`, and the other package codes keep their original labels
##' @param xlab.package whether to use the package name as the x axis name
##' @return a volcano plot according to \code{logFC} and either
##'   \code{-log10(P.value)} or \code{-log10(padj)}, depending on \code{adjust}.
##' @author Xiaojie Sun
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 aes
##' @importFrom ggplot2 geom_point
##' @importFrom ggplot2 scale_color_manual
##' @importFrom ggplot2 geom_vline
##' @importFrom ggplot2 geom_hline
##' @importFrom ggplot2 theme_bw
##' @importFrom ggplot2 labs
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 element_text
##' @importFrom ggplot2 scale_x_continuous
##' @importFrom ggplot2 unit
##' @importFrom ggplot2 theme
##' @export
##' @examplesIf requireNamespace("ggplot2", quietly = TRUE)
##' head(deseq_data)
##' draw_volcano(deseq_data)
##' draw_volcano(deseq_data,pvalue_cutoff = 0.01,logFC_cutoff = 2)
##' draw_volcano(deseq_data,color = c("darkgreen", "darkgrey", "#B2182B"))

##' @seealso
##' \code{\link{draw_heatmap}};\code{\link{draw_pca}};\code{\link{draw_venn}}

draw_volcano <- function(deg,lab=NA,xlab.package = TRUE,
                         pvalue_cutoff = 0.05,logFC_cutoff= 1,
                         pkg = 1,adjust = TRUE,change_col = "change",
                         symmetry = FALSE,
                         color = c("#2874C5", "grey","#f87669")){
  if(!is.data.frame(deg)) stop("deg must be a data.frame created by Differential analysis")
  if(pvalue_cutoff>0.1)warning("Your pvalue_cutoff seems too large")
  if(pvalue_cutoff>=1)stop("pvalue_cutoff will never larger than 1")
  pv_col = .tinyarray_pvalue_column(adjust)

  if(!adjust){
    dat = switch(EXPR = pkg,
                 v1 = deg[,c(2,5)],
                 v2 = deg[,c(1,4)],
                 v3 = deg[,c("logFC", "P.Value")],
                 v4 = deg[,c("logFC", "P.Value")])
  }else{
    dat = switch(EXPR = pkg,
                 v1 = deg[,c(2,6)],
                 v2 = deg[,c(1,5)],
                 v3 = deg[,c("logFC", "adj.P.Val")],
                 v4 = deg[,c("logFC", "adj.P.Val")])
  }
  colnames(dat)[1:2]=c("logFC",pv_col)
  #logFC_cutoff <- with(dat,mean(abs(logFC)) + 2*sd(abs(logFC)) )
  if (!is.null(change_col) && length(change_col) == 1L && !is.na(change_col) && nzchar(change_col)) {
    if (change_col %in% colnames(deg)) {
      dat$change = toupper(as.character(deg[[change_col]]))
    } else if (identical(change_col, "change")) {
      dat$change = with(dat, ifelse(logFC > logFC_cutoff & dat[[pv_col]] < pvalue_cutoff, "UP",
                                    ifelse(logFC < -logFC_cutoff & dat[[pv_col]] < pvalue_cutoff, "DOWN", "NOT")))
    } else {
      stop("change_col `", change_col, "` was not found in deg", call. = FALSE)
    }
  } else if ("change" %in% colnames(deg)) {
    dat$change = toupper(as.character(deg$change))
  } else {
    dat$change = with(dat,ifelse(logFC>logFC_cutoff & dat[[pv_col]]<pvalue_cutoff , "UP",
                                 ifelse(logFC< -logFC_cutoff & dat[[pv_col]]<pvalue_cutoff , "DOWN","NOT")))
  }
  if(length(lab) == 1L && is.na(lab)) {
    if (as.integer(pkg) == 4L) {
      lab = "log2FoldChange"
    } else if (xlab.package) {
      lab = c("DESeq2","edgeR","limma(voom)","limma")[pkg]
    } else {
      lab = "logFC"
    }
  }
  dat$neglogp <- -log10(pmax(dat[[pv_col]], .Machine$double.xmin))
  ylab <- if (adjust) {
    "-log10 adjusted P value"
  } else {
    "-log10 P value"
  }
  this_tile <- paste0(nrow(dat[dat$change =='DOWN',]),
                      ' Downstream, ',
                      nrow(dat[dat$change =='UP',]),
                      ' Upstream'
  )
  p <- ggplot(data = dat,
              aes(x = logFC,
                  y = neglogp)) +
    geom_point(alpha=0.4, size=1.75,
               aes(color=change)) +
    scale_color_manual(values=color)+
    geom_vline(xintercept = c(-logFC_cutoff,logFC_cutoff),linetype=4,col="black",linewidth=0.8) +
    geom_hline(yintercept = -log10(pvalue_cutoff),linetype=4,col="black",linewidth=0.8) +
    theme_bw()+
    labs(title = this_tile , x = lab, y = ylab)+
    theme(plot.title = element_text(hjust = 0.5))
  ce = ceiling(max(abs(dat$logFC)))
  if(symmetry)p <- p+scale_x_continuous(limits = c(-ce,ce),expand = c(0,0))
  return(p)
}
utils::globalVariables(c("logFC","P.value","padj","change","neglogp"))

##' draw a venn plot
##'
##' warning a venn plot for deg result created by three packages
##'
##' @inheritParams VennDiagram::venn.diagram
##' @inheritParams VennDiagram::draw.single.venn
##' @param x a list for plot
##' @param color color vector
##' @param reverse logical,reflect the three-set Venn diagram along its central
##' vertical axis of symmetry. Use in combination with rotation
##' to generate all possible set orders
##' @param ... other parameters from venn.diagram
##' @return a venn plot according to \code{x}, \code{y}, and \code{z} using the
##'   list names as labels.
##' @author Xiaojie Sun
##' @export
##' @examplesIf requireNamespace("VennDiagram",quietly = TRUE) && requireNamespace("ggplotify",quietly = TRUE) && requireNamespace("cowplot",quietly = TRUE)
##' x = list(Deseq2=sample(1:100,30),edgeR = sample(1:100,30),limma = sample(1:100,30))
##' draw_venn(x,"test")
##' draw_venn(x,"test",color = c("darkgreen", "darkblue", "#B2182B"))
##' @seealso
##' \code{\link{draw_pca}};\code{\link{draw_volcano}};\code{\link{draw_heatmap}}

draw_venn <- function(x,main,
                      color = c("#2874C5","#f87669","#e6b707","#868686","#66C2A5","#FC8D62","#8DA0CB","#E78AC3","#A6D854","#FFD92F","#E5C494","#B3B3B3"),
                      imagetype ="png",
                      filename=NULL,
                      lwd=1,
                      lty=1,
                      col=color[1:length(x)],
                      fill=color[1:length(x)],
                      cat.col=color[1:length(x)],
                      cat.cex = 1,
                      cat.dist = -0.15,
                      rotation.degree = 0,
                      main.cex = 1,
                      cex=1,
                      alpha = 0.1,
                      reverse=TRUE,
                      ...){
  if(as.numeric(grDevices::dev.cur())!=1) grDevices::graphics.off()
  if(!requireNamespace("VennDiagram",quietly = TRUE)) {
    stop("Package \"VennDiagram\" needed for this function to work.
         Please install it by install.packages('VennDiagram')",call. = FALSE)
  }
  if(!requireNamespace("ggplotify",quietly = TRUE)) {
    stop("Package \"ggplotify\" needed for this function to work.
         Please install it by install.packages('ggplotify')",call. = FALSE)
  }
  if(!requireNamespace("cowplot",quietly = TRUE)) {
    stop("Package \"cowplot\" needed for this function to work.
         Please install it by install.packages('cowplot')",call. = FALSE)
  }
  if(!is.list(x)) stop("x must be a list")
  if(length(x)>7) stop("why do you give me so many elements to compare, I reject!")
  temp_device <- NULL
  if (is.null(filename)) {
    temp_device <- tempfile(fileext = ".png")
    grDevices::png(temp_device)
    on.exit({
      if (grDevices::dev.cur() != 1) {
        grDevices::dev.off()
      }
      if (!is.null(temp_device) && file.exists(temp_device)) {
        unlink(temp_device)
      }
    }, add = TRUE)
  }
  p = VennDiagram::venn.diagram(x = x,
                   imagetype =imagetype,
                   filename=filename,
                   lwd=lwd,
                   lty=lty,
                   col=col,
                   fill=fill,
                   cat.col=fill,
                   cat.cex = cat.cex,
                   cat.dist = cat.dist,
                   rotation.degree = rotation.degree,
                   main = main,
                   main.cex = main.cex,
                   cex=cex,
                   alpha = alpha,
                   reverse=reverse,
                   ...)
  p = ggplotify::as.ggplot(cowplot::as_grob(p))
  file.remove(dir(pattern = ("^VennDiagram.*log$")))
  return(p)
}


##' draw boxplot for expression
##'
##' draw boxplot for expression
##'
##' @inheritParams draw_pca
##' @param method one of kruskal.test,aov,t.test and wilcox.test
##' @param width width of boxplot and error bar
##' @param sort whether the boxplot will be sorted
##' @param drop whether to discard insignificant values
##' @param pvalue_cutoff if drop = TRUE,genes with p-values below the threshold will be drawn
##' @param xlab title of the x axis
##' @param ylab title of the y axis
##' @param grouplab title of group legend
##' @param p.label whether to show the exact p value in the plot; if
##'   `FALSE`, show significance stars
##' @param add_error_bar whether to add error bar
##' @param ... other parameters from stat_compare_means
##' @return a boxplot according to \code{exp} and grouped by \code{group}.
##' @author Xiaojie Sun
##' @importFrom tibble rownames_to_column
##' @importFrom ggplot2 stat_boxplot
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 geom_boxplot
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 aes
##' @importFrom ggplot2 labs
##' @importFrom ggplot2 scale_fill_manual
##' @importFrom ggplot2 element_text
##' @importFrom ggplot2 after_stat
##' @export
##' @examplesIf requireNamespace("tidyr", quietly = TRUE) && requireNamespace("ggpubr", quietly = TRUE)
##' draw_boxplot(t(iris[,1:4]),iris$Species)
##' exp <-  matrix(rnorm(60),nrow = 10)
##' colnames(exp) <- paste0("sample",1:6)
##' rownames(exp) <- paste0("gene",1:10)
##' exp[,4:6] = exp[,4:6] +10
##' exp[1:4,1:4]
##' group_list <- factor(rep(c("A","B"),each = 3))
##' draw_boxplot(exp,group_list)
##' draw_boxplot(exp,group_list,color = c("grey","red"))
##' @seealso
##' \code{\link{draw_heatmap}};\code{\link{draw_volcano}};\code{\link{draw_venn}}

draw_boxplot = function(exp,group_list,
                        method = "kruskal.test",
                        sort = TRUE,
                        drop = FALSE,
                        width = 0.5,
                        pvalue_cutoff = 0.05,
                        xlab = "Gene",
                        ylab = "Expression",
                        grouplab = "Group",
                        p.label = FALSE,
                        add_error_bar = FALSE,
                        color = c("#2874C5","#f87669","#e6b707","#868686","#66C2A5", "#FC8D62", "#8DA0CB", "#E78AC3", "#A6D854",
                                 "#FFD92F", "#E5C494", "#B3B3B3"),
                                 ...){
  if(!requireNamespace("ggpubr",quietly = TRUE)) {
    stop("Package \"ggpubr\" needed for this function to work.
         Please install it by install.packages('ggpubr')",call. = FALSE)
  }
  if(!requireNamespace("tidyr",quietly = TRUE)) {
    stop("Package \"tidyr\" needed for this function to work.
         Please install it by install.packages('tidyr')",call. = FALSE)
  }
  p0 <-  all(apply(exp,2,is.numeric)) & (!is.null(rownames(exp)))
  if(!p0) stop("exp must be a numeric matrix with rownames")
  group_list <- .tinyarray_prepare_group_list_safe(group_list)
  if (is.null(group_list)) return(NULL)
  p1 = method %in% c("kruskal.test","aov","t.test","wilcox.test")
  if(!p1) stop("method should be one of kruskal.test,aov,t.test and wilcox.test")
  if(nlevels(group_list)>2 & method %in% c("t.test","wilcox.test")) stop("group_list parameter must have exactly 2 group")
  if(method=="kruskal.test"){
    x = apply(exp, 1, function(x){
      stats::kruskal.test(x~group_list)$p.value
    })
  }else if(method=="aov"){
    x = apply(exp, 1, function(x){
      summary(stats::aov(x~group_list))[[1]]$`Pr(>F)`[1]
    })
  }else if(method=="t.test"){
    x = apply(exp, 1, function(x){
      stats::t.test(x~group_list)$p.value
    })
  }else if(method=="wilcox.test"){
    x = apply(exp, 1, function(x){
      stats::wilcox.test(x~group_list)$p.value
    })
  }
  if(sum(x< pvalue_cutoff)==0) {
    message("No rows below the threshold,plot all rows")
    drop = FALSE
  }
  if(drop){
    x = x[ x < pvalue_cutoff]
    exp = exp[rownames(exp) %in% names(x),]
  }

  if(length(x)>40) message("it seems to be too many rows")
  dat = rownames_to_column(as.data.frame(t(exp)),var = "sample")
  dat$group = group_list
  dat = tidyr::gather(dat,
               rows,exp,-sample,-group)
  if(sort){
    dat$rows = factor(dat$rows,
                      levels = names(sort(x)),
                      ordered = TRUE)
  }
  col = color[1:length(levels(group_list))]
  p = ggplot(dat,aes(rows,exp,fill = group))+
    geom_boxplot( width = width)+
    theme_bw()+
    theme(legend.position = "top")+
    labs(fill = grouplab,
         x = xlab,
         y = ylab)+
    scale_fill_manual(values = col)
  if(add_error_bar) p = p + stat_boxplot(geom ='errorbar', width = width)
  if(!p.label){
    p = p + ggpubr::stat_compare_means(aes(group = group,
                                           label = ggplot2::after_stat(p.signif)),
                                       method = method,...)
  }else{
    p = p + ggpubr::stat_compare_means(aes(group = group,
                                           label = ggplot2::after_stat(p.format)),
                                       method = method,...)
  }
  if(length(x)>10) p = p +
    theme(axis.text.x = element_text(vjust = 1,hjust = 1,angle = 80))
  return(p)
}

utils::globalVariables(c(".","rows","group","p.signif","p.format"))

##' ggheat
##'
##' draw heatmap plot with annotation by ggplot2
##'
##' @param dat expression matrix for plot
##' @param group group for expression colnames
##' @param cluster logical,cluster in both rows and column or not, default F,now replaced by cluster_rows and cluster_cols.
##' @param cluster_rows logical, if rows (on the plot) should be clustered, default F
##' @param cluster_cols logical, if column (on the plot) should be clustered, default F
##' @param show_rownames logical or NULL. When NULL, the function shows x-axis
##'   labels for up to 100 rows, rotates them 45 degrees for 9 to 100 rows, and
##'   hides them above 100 rows.
##' @param show_colnames logical,show colnames in plot or not, default T
##' @param rotate_rownames logical or NULL. When NULL, the function uses the
##'   automatic rotation rule described in \code{show_rownames}.
##' @param groupname name of group legend
##' @param expname name of exp legend
##' @param fill_mid use median value as geom_tile fill midpoint
##' @param color color for heatmap
##' @param legend_color color for legend
##' @return a ggplot object
##' @author Xiaojie Sun
##' @importFrom pheatmap pheatmap
##' @importFrom tibble rownames_to_column
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 geom_tile
##' @importFrom ggplot2 scale_fill_manual
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 aes
##' @importFrom ggplot2 element_blank
##' @importFrom ggplot2 scale_x_continuous
##' @importFrom ggplot2 element_rect
##' @importFrom ggplot2 scale_fill_gradient2
##' @importFrom ggplot2 scale_x_discrete
##' @importFrom ggplot2 labs
##' @importFrom patchwork wrap_plots
##' @export
##' @examplesIf requireNamespace("ggplot2", quietly = TRUE) && requireNamespace("pheatmap", quietly = TRUE) && requireNamespace("tidyr", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)
##' exp_dat = matrix(sample(100:1000,40),ncol = 4)
##' exp_dat[1:(nrow(exp_dat)/2),] =  exp_dat[1:(nrow(exp_dat)/2),]-1000
##' rownames(exp_dat) = paste0("sample",1:nrow(exp_dat))
##' colnames(exp_dat) = paste0("gene",1:ncol(exp_dat))

##' group = rep(c("A","B"),each = nrow(exp_dat)/2)
##' group = factor(group,levels = c("A","B"))
##' ggheat(exp_dat,group)
##' ggheat(exp_dat,group,cluster_rows = TRUE)
##' ggheat(exp_dat,group,cluster_rows = TRUE,show_rownames = FALSE,
##'        show_colnames = FALSE,groupname = "risk",expname = "expression")



ggheat = function(dat,group,cluster = FALSE,
                  color = c("#2874C5", "white","#f87669"),
                  legend_color = c("#2874C5","#f87669","#e6b707","#868686","#66C2A5", "#FC8D62", "#8DA0CB", "#E78AC3", "#A6D854", "#FFD92F",
                                   "#E5C494", "#B3B3B3"),
                  show_rownames = NULL,show_colnames = TRUE,
                  cluster_rows = FALSE,cluster_cols = FALSE,
                  groupname = "group",expname = "exp",
                  fill_mid = TRUE,
                  rotate_rownames = NULL){
  group <- .tinyarray_prepare_group_list_safe(group, arg = "group")
  if (is.null(group)) return(NULL)
  dat = data.frame(dat)
  ph = pheatmap::pheatmap(
    t(dat),
    silent = TRUE,
    breaks = .tinyarray_safe_heatmap_breaks(t(dat), 100L),
    color = grDevices::colorRampPalette(c("#2fa1dd", "white", "#f87669"))(100),
    cluster_rows = ncol(dat) > 1L,
    cluster_cols = nrow(dat) > 1L
  )
  if(cluster){
    cluster_rows = TRUE
    cluster_cols = TRUE
  }
  if(cluster_rows && !is.null(ph$tree_row)){
    dat = dat[,ph$tree_row$order]
  }
  if(cluster_cols && !is.null(ph$tree_col)){
    dat = dat[ph$tree_col$order,]
    group = group[ph$tree_col$order]
  }

  dat$group = group
  dat2 = tidyr::gather(rownames_to_column(dat,
                                   "samples"),
                gene,exp,-group,-samples)

  dat2$samples = factor(dat2$samples,levels = rownames(dat))
  dat2$gene = factor(dat2$gene,levels = rev(colnames(dat)))

  if(!cluster) dat2 = arrange(dat2,group)
  ng = length(unique(group))
  col = legend_color[1:ng]
  names(col) = levels(group)
  p1 = ggplot(dat, aes(x = 1:nrow(dat), y = 1)) +
    geom_tile(aes(fill = group))+
    scale_fill_manual(values = col)+
    theme(panel.grid = element_blank(),
          panel.background = element_blank(),
          axis.line = element_blank(),
          axis.ticks = element_blank(),
          axis.text = element_blank(),
          axis.title = element_blank()) +
    scale_x_continuous(expand = c(0,0))+
    labs(fill = groupname)

  midpoint = ifelse(fill_mid,stats::median(dat2$exp),0)

  p2 = ggplot(dat2, aes(samples, gene, fill = exp)) +
    geom_tile()+
    theme(panel.grid = element_blank(),
          panel.background = element_rect(fill = NA),
          legend.background = element_rect(fill = NA),
          plot.background = element_rect(fill = NA),
          axis.line = element_blank(),
          axis.ticks = element_blank(),
          axis.title = element_blank()
    ) +
    scale_fill_gradient2(low = color[1],
                         mid = color[2],
                         high = color[3],
                         midpoint = midpoint)+
    scale_x_discrete(expand = c(0,0))+
    labs(fill = expname)
  x_n <- nrow(dat)
  if (is.null(show_rownames)) {
    show_rownames <- x_n <= 100
  }
  if (is.null(rotate_rownames)) {
    rotate_rownames <- x_n > 8 && x_n <= 100
  }
  if(!show_rownames) {
    p2 = p2 + theme(axis.text.x = element_blank())
  } else if (rotate_rownames) {
    p2 = p2 + theme(axis.text.x = element_text(vjust = 1, hjust = 1, angle = 45))
  } else {
    p2 = p2 + theme(axis.text.x = element_text(vjust = 0.5, hjust = 0.5, angle = 0))
  }
  if(!show_colnames) p2 = p2 + theme(axis.text.y = element_blank())
  p = wrap_plots(p1,p2,nrow = 2,heights = c(1,11))
  return(p)
}

utils::globalVariables(c("gene","samples"))

##' draw_tsne
##'
##' draw tsne plot with annotation by ggplot2
##'
##' @inheritParams draw_pca
##' @param perplexity numeric; perplexity parameter for Rtsne
##' @param color.label color legend label
##' @return a ggplot object
##' @author Xiaojie Sun
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 geom_point
##' @importFrom ggplot2 stat_ellipse
##' @importFrom ggplot2 theme_classic
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 aes
##' @export
##' @examplesIf requireNamespace("Rtsne", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)
##' exp <-  matrix(rnorm(10000),nrow = 50)
##' colnames(exp) <- paste0("sample",1:200)
##' rownames(exp) <- paste0("gene",1:50)
##' exp[1:4,1:4]
##' exp[,1:100] = exp[,1:100]+10
##' group_list <- factor(rep(c("A","B"),each = 100))
##' draw_tsne(exp,group_list)

draw_tsne = function(exp,group_list,perplexity=30,
                     color = c("#2874C5","#f87669","#e6b707","#868686","#92C5DE", "#F4A582", "#66C2A5", "#FC8D62", "#8DA0CB", "#E78AC3",
                               "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3"),
                     color.label = "group",
                     addEllipses = TRUE){
  if(!requireNamespace("Rtsne",quietly = TRUE))
  stop("Package \"Rtsne\" needed for this function to work.
         Please install it by install.packages('Rtsne')",call. = FALSE)
  p1 <-  all(apply(exp,2,is.numeric))
  if(!p1) stop("exp must be a numeric matrix")
  group_list <- .tinyarray_prepare_group_list_safe(group_list)
  if (is.null(group_list)) return(NULL)
  tsne_out = Rtsne::Rtsne(t(exp),perplexity = perplexity)
  pdat = data.frame(tsne_out$Y,group_list)
  colnames(pdat) = c("Y1","Y2","group")
  p = ggplot(pdat,aes(Y1,Y2))+
    geom_point(aes(Y1,Y2,fill = group),
               shape = 21,color = "black")+
    scale_color_manual(values = color[1:nlevels(group_list)])+
    scale_fill_manual(values = color[1:nlevels(group_list)])+
    theme_classic()+
    theme(legend.position = "top")+
    labs(color = color.label,fill = color.label)
  if(addEllipses) p = p +
    stat_ellipse(aes(color = group,fill = group),
                 geom = "polygon",
                 alpha = 0.3,
                 linetype = 2)
  return(p)
}

utils::globalVariables(c("Y1","Y2","group","patient","ri","time","event"))

##' draw_KM
##'
##' draw KM-plot with two or more group
##'
##' @inheritParams draw_pca
##' @param meta survival data with time and event column
##' @param time_col colname of time
##' @param event_col colname of event
##' @param legend.title legend title
##' @param legend.labs character vector specifying legend labels
##' @param ... other parameters from ggsurvplot
##' @return a KM-plot
##' @author Xiaojie Sun
##' @importFrom survival survfit
##' @importFrom survival Surv
##' @importFrom survminer ggsurvplot
##' @export
##' @examplesIf requireNamespace("survival", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE)
##' x = survival::lung
##'draw_KM(meta = x,
##'        group_list = x$sex,event_col = "status")

draw_KM = function(meta,
                   group_list,
                   time_col = "time",event_col = "event",
                   legend.title = "Group",
                   legend.labs = NULL,
                   color = c("#2874C5","#f87669","#e6b707","#868686","#92C5DE", "#F4A582", "#66C2A5", "#FC8D62", "#8DA0CB", "#E78AC3",
                             "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3"),
                             ...){
  p1 <-  (time_col %in% colnames(meta)) & (event_col %in% colnames(meta))
  if(!p1){
    stop("meta data must involved time and event columns")
  }else{
    z1 = which(colnames(meta)==time_col)
    z2 = which(colnames(meta)==event_col)
    colnames(meta)[z1] = "time"
    colnames(meta)[z2] = "event"
  }
  group_list <- .tinyarray_prepare_group_list_safe(group_list)
  if (is.null(group_list)) return(NULL)
  if(is.null(legend.labs)) legend.labs = levels(group_list)
  meta$Group = group_list
  sfit <- survival::survfit(Surv(time, event) ~ Group,
                  data = meta)
  p = survminer::ggsurvplot(sfit,pval = T,data = meta,
                 palette = color[1:nlevels(group_list)],
                 legend.title = legend.title,
                 legend.labs = legend.labs,
                 ...)
  return(p$plot)
}

##' corscatterplot
##'
##' draw cor scatter plot with density plot by ggplot2
##' @param dat plot data
##' @param x x
##' @param y y
##' @param color_cor color for cor reg.line
##' @param fill_cor fill for cor reg.line
##' @param fill_x fill for top density plot
##' @param fill_y fill for right density plot
##' @param type whether to use a density plot or a histogram plot for the side panel.
##' @param ... other parameters for ggscatter
##' @return a ggplot object
##' @author Xiaojie Sun
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 geom_density
##' @importFrom ggplot2 theme_void
##' @importFrom ggplot2 theme_bw
##' @importFrom ggplot2 geom_histogram
##' @importFrom patchwork plot_layout
##' @importFrom patchwork plot_spacer
##' @export
##' @examplesIf requireNamespace("ggpubr", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)
##' corscatterplot(iris,"Sepal.Length","Sepal.Width")
corscatterplot = function(dat,x,y,color_cor = "blue",fill_cor = "lightgray",
                          fill_x = "#ff820e",fill_y = "#0000fe",
                          type = "density",...){
  if(!requireNamespace("ggpubr",quietly = TRUE))
    stop("Package \"ggpubr\" needed for this function to work.
         Please install it by install.packages('ggpubr')",call. = FALSE)

  p1 <- ggpubr::ggscatter( dat, x = x, y = y,
                           add = "reg.line", conf.int = TRUE,
                           add.params = list(color = color_cor, fill = fill_cor),...)+
    ggpubr::stat_cor()+
    theme_bw()
  if(type == "density"){
    p2 <- ggplot(dat, do.call(ggplot2::aes, list(x = as.name(x)))) +
      geom_density(fill = fill_x) +
      theme_void()

    p3 <- ggplot(dat, do.call(ggplot2::aes, list(x = as.name(y)))) +
      geom_density(fill = fill_y) +
      coord_flip() +
      theme_void()
  }else if( type == "histogram"){
    p2 <- ggplot(dat, do.call(ggplot2::aes, list(x = as.name(x)))) +
      geom_histogram(fill = fill_x,width = 0.1,color = "black") +
      theme_void()

    p3 <- ggplot(dat, do.call(ggplot2::aes, list(x = as.name(y)))) +
      geom_histogram(fill = fill_y,width = 0.1,color = "black") +
      coord_flip() +
      theme_void()
  }else{
    stop("The `type` parameter must be either 'density' or 'histogram'.")
  }

  p2 + p1 + p3 + plot_spacer() + plot_layout(design = c("AAAAD
       BBBBC
       BBBBC
       BBBBC
       BBBBC"))
}

##' corheatmap
##'
##' draw cor heatmap
##' @param exp A numeric matrix
##' @param x genes or cells from exp
##' @param y genes or cells from exp
##' @param color color for heatmap
##' @return a ggplot object
##' @author Xiaojie Sun
##' @importFrom dplyr case_when
##' @importFrom pheatmap pheatmap
##' @importFrom Hmisc rcorr
##' @export
##' @examplesIf requireNamespace("Hmisc", quietly = TRUE) && requireNamespace("pheatmap", quietly = TRUE)
##' x = rownames(exprSet_hub1)[1:3]
##' y = rownames(exprSet_hub1)[4:7]
##' corheatmap(exprSet_hub1,x,y)

corheatmap = function(exp,x,y,color = c("#2fa1dd", "white", "#f87669")){
  x = unique(x)
  y = unique(y)
  y = setdiff(y,x)
  if (!length(x) || !length(y)) {
    warning("corheatmap requires at least one gene in each set after overlap removal; skipping plot.", call. = FALSE)
    return(NULL)
  }
  da = t(exp[c(x,y),])
  rc = rcorr(da)
  m = rc$r[seq_len(length(x)), (ncol(da)-length(y)+1):ncol(da), drop = FALSE]
  p = rc$P[seq_len(length(x)), (ncol(da)-length(y)+1):ncol(da), drop = FALSE]
  tmp = matrix(ifelse(p<=0.0001,"****",
                      ifelse(p<=0.001,"***",
                             ifelse(p<=0.01,"**",
                                    ifelse(p<=0.05,"*","")))),
               nrow = nrow(p),
               ncol = ncol(p))
  pheatmap(t(m),
           display_numbers = t(tmp),
           angle_col = ifelse(length(x)>10,45,0),
           color = grDevices::colorRampPalette(color)(100),
           breaks = .tinyarray_safe_heatmap_breaks(t(m), 100L),
           border_color = "white",
           width = 7,
           height = 9.1,
           treeheight_col = 0,
           treeheight_row = 0,
           cluster_rows = ncol(m) > 1L,
           cluster_cols = nrow(m) > 1L)
}

##' t_choose
##'
##' choose differential expressed genes by simple t.test
##'
##' @inheritParams get_deg
##' @param genes a vector with some genes
##' @param up_only keep up genes in the result only
##' @param down_only keep down genes in the result only
##' @param pvalue_cutoff p value cut off ,0.05 by defult
##' @export
##' @return a vector with differential expressed genes
##' @author Xiaojie Sun
##' @examples
##' exp = matrix(rnorm(1000),ncol = 10)
##' rownames(exp) = sample(mRNA_anno$gene_id,100)
##' colnames(exp) = c(paste0("TCGA",1:5),paste0("GTEX",1:5))
##' exp2  = trans_exp(exp)
##' exp2[,1:5] = exp2[,1:5]+10
##' group_list = rep(c("A","B"),each = 5)
##' genes = sample(rownames(exp2),3)
##' t_choose(genes,exp2,group_list)

t_choose <- function(genes,exp,group_list,up_only = FALSE,down_only = FALSE,pvalue_cutoff = 0.05){
  if(up_only&down_only)stop("please change neither up_only or down_only to FALSE")
  group_list <- .tinyarray_prepare_group_list_safe(group_list)
  if (is.null(group_list)) return(NULL)
  genes = genes[genes %in% rownames(exp)]
  exp_small = exp[genes,]
  dat = data.frame(t(exp_small),check.names = FALSE)
  dat$group_list = group_list
  p_v <- sapply(1:(ncol(dat)-1), function(i){
    stats::t.test(dat[,i] ~group_list)$p.value
  })
  names(p_v) = colnames(dat)[-ncol(dat)]

  exp_genes = names(p_v[p_v < pvalue_cutoff])

  if(up_only){
    es_up <- sapply(1:(ncol(dat)-1), function(i){
      tmp = stats::t.test(dat[,i] ~group_list)
      k = tmp$estimate[2]-tmp$estimate[1] >0
      return(k)
    })
    up_genes = names(p_v)[p_v < pvalue_cutoff & es_up]
    return(up_genes)
  }else if(down_only){
    es_down <- sapply(1:(ncol(dat)-1), function(i){
      tmp = stats::t.test(dat[,i] ~group_list)
      k = tmp$estimate[2]-tmp$estimate[1] <0
      return(k)
    })
    down_genes = names(p_v)[p_v <pvalue_cutoff & es_down]
    return(down_genes)
  }else{
    return(exp_genes)
  }
}


##' cor.test for all variables
##'
##' cor.test for all variables(each two columns)
##'
##' @param x A numeric matrix or data.frame
##' @param drop drop values
##' @inheritParams cor.one
##' @return a data.frame with cor.test p.value and estimate
##' @author Xiaojie Sun
##' @export
##' @examples
##' x = iris[,-5]
##' cor.full(x)
##' @seealso
##' \code{\link{cor.one}}

cor.full <- function(x,drop = min(x,na.rm = TRUE)-0.001,min.obs = 10){
  p = list()
  ss1 = utils::combn(colnames(x),2)

  for(i in seq_len(ncol(ss1))){
    bt = x[,ss1[1,i]]
    kt = x[,ss1[2,i]]
    k1 = bt > drop
    k2 = kt > drop
    if(sum(k1&k2) < min.obs){
      p[[i]] = c(NA,NA)
    }else{
    cot = stats::cor.test(bt[k1&k2],kt[k1&k2])
    p[[i]] = c(cot$p.value,cot$estimate)
    }
    names(p[[i]]) = c("p.value","cor")
  }
  re = do.call(cbind,p)
  colnames(re) = apply(ss1, 2, paste,collapse =":")
  re = as.data.frame(t(re))
  ks = sapply(rownames(re), function(gs){
    parts = strsplit(gs, ":", fixed = TRUE)[[1]]
    g1 = parts[[1]]
    g2 = parts[[2]]
    sum(x[,g1]>drop & x[,g2]>drop)
  })
  re$obsnumber = ks
  re = stats::na.omit(re)
  return(re)
}



##' cor.test for one variable with all variables
##'
##' cor.test for all variables(each two columns)
##'
##' @param x A numeric matrix or data.frame
##' @param var your chosen variable,only one.
##' @param drop.var drop values in var
##' @param drop.other drop values in other columns
##' @param min.obs minimum number of observations  after dropping
##' @return A data.frame with cor.test p.value and estimate
##' @author Xiaojie Sun
##' @export
##' @examples
##' x = iris[,-5]
##' cor.one(x,"Sepal.Width")
##' @seealso
##' \code{\link{cor.full}}

cor.one <- function(x,var,drop.var = min(x[,var],na.rm = TRUE)-0.001,
                    drop.other = min(x[,-which(colnames(x)==var)],na.rm = TRUE)-0.001,
                    min.obs = 10){
  if(!(var %in% colnames(x))) stop(paste0(var," is not a colname of ",x,",please check it."))
  if(!all(!duplicated(colnames(x)))) stop("unique colnames is required")
  p = list()
  ss = setdiff(colnames(x),var)
  bt = x[,var]
  k1 = bt > drop.var
  for(i in (1:length(ss))){
    kt = x[,ss[[i]]]
    k2 = kt > drop.other
    if(sum(k1&k2) < min.obs){
      p[[i]] = c(NA,NA)
    }else{
      cot = stats::cor.test(bt[k1&k2],kt[k1&k2])
      p[[i]] = c(cot$p.value,cot$estimate)
    }
    names(p[[i]]) = c("p.value","cor")
  }
  re = do.call(cbind,p)
  colnames(re) = ss
  re = as.data.frame(t(re))
  ks = sapply(rownames(re), function(g){
    sum(x[,g]>drop.other)
  })
  re$obsnumber = ks
  re = stats::na.omit(re)
  return(re)
}

.tinyarray_lazy_factory <- function(pkg, fun) {
  force(pkg)
  force(fun)
  cached <- NULL
  function(...) {
    if (is.null(cached)) {
      if (!requireNamespace(pkg, quietly = TRUE)) {
        stop("Package \"", pkg, "\" needed for this function to work.", call. = FALSE)
      }
      cached <<- getExportedValue(pkg, fun)
    }
    call <- match.call()
    call[[1L]] <- quote(.tinyarray_lazy_target)
    eval(
      call,
      envir = list2env(
        list(.tinyarray_lazy_target = cached),
        parent = parent.frame()
      )
    )
  }
}

.tinyarray_register_lazy_functions <- function() {
  ns <- environment(.tinyarray_register_lazy_functions)
  lazy_map <- list(
    dplyr = c("arrange", "case_when", "desc", "distinct", "inner_join", "mutate"),
    ggplot2 = c(
      "aes", "aes_string", "after_stat", "coord_flip", "element_blank",
      "element_line", "element_rect", "element_text", "facet_grid",
      "geom_bar", "geom_boxplot", "geom_density", "geom_histogram",
      "geom_hline", "geom_point", "geom_tile", "geom_vline", "ggplot",
      "ggsave", "labs", "scale_color_manual", "scale_fill_gradient2",
      "scale_fill_manual", "scale_x_continuous", "scale_x_discrete",
      "scale_y_continuous", "stat_boxplot", "stat_ellipse", "theme",
      "theme_bw", "theme_classic", "theme_light", "theme_void", "unit",
      "ylim"
    ),
    Hmisc = c("rcorr"),
    limma = c("contrasts.fit", "eBayes", "lmFit", "makeContrasts", "topTable"),
    patchwork = c("plot_layout", "plot_spacer", "wrap_plots"),
    pheatmap = c("pheatmap"),
    stringr = c(
      "str_detect", "str_extract", "str_remove", "str_remove_all",
      "str_split", "str_starts", "str_sub", "str_to_lower", "str_to_upper",
      "str_wrap"
    ),
    survival = c("Surv", "coxph", "survdiff", "survfit"),
    survminer = c("ggsurvplot", "surv_cutpoint"),
    tibble = c("rownames_to_column", "tibble")
  )
  for (pkg in names(lazy_map)) {
    for (fun in lazy_map[[pkg]]) {
      assign(fun, .tinyarray_lazy_factory(pkg, fun), envir = ns)
    }
  }
  invisible(TRUE)
}

.tinyarray_register_lazy_functions()

.tinyarray_has_pkgs <- function(...) {
  pkgs <- unlist(list(...), use.names = FALSE)
  if (!length(pkgs)) {
    return(TRUE)
  }
  all(vapply(pkgs, requireNamespace, logical(1), quietly = TRUE))
}

.tinyarray_prepare_group_list <- function(group_list, arg = "group_list", min_levels = 2L) {
  if (!is.factor(group_list)) {
    group_list <- suppressWarnings(factor(group_list, levels = unique(group_list)))
  } else {
    group_list <- droplevels(group_list)
  }
  if (nlevels(group_list) < min_levels) {
    stop(arg, " must contain at least ", min_levels, " groups", call. = FALSE)
  }
  group_list
}

.tinyarray_prepare_group_list_safe <- function(group_list, arg = "group_list", min_levels = 2L) {
  tryCatch(
    .tinyarray_prepare_group_list(group_list, arg = arg, min_levels = min_levels),
    error = function(e) {
      warning(conditionMessage(e), call. = FALSE)
      NULL
    }
  )
}

.tinyarray_order_columns_by_group <- function(exp, group_list) {
  ord <- order(group_list, seq_along(group_list))
  list(
    exp = exp[, ord, drop = FALSE],
    group_list = group_list[ord]
  )
}

.tinyarray_safe_heatmap_clusters <- function(nrow_value,
                                             ncol_value,
                                             cluster_rows = TRUE,
                                             cluster_cols = TRUE) {
  list(
    cluster_rows = isTRUE(cluster_rows) && isTRUE(nrow_value > 1L),
    cluster_cols = isTRUE(cluster_cols) && isTRUE(ncol_value > 1L)
  )
}

.tinyarray_safe_heatmap_breaks <- function(mat, n_breaks = 100L) {
  n_breaks <- max(2L, as.integer(n_breaks)[1L])
  vals <- as.numeric(mat)
  vals <- vals[is.finite(vals)]
  if (!length(vals)) {
    return(seq(0, 1, length.out = n_breaks))
  }
  rng <- range(vals)
  if (!is.finite(rng[1L]) || !is.finite(rng[2L])) {
    return(seq(0, 1, length.out = n_breaks))
  }
  if (identical(rng[1L], rng[2L])) {
    delta <- if (rng[1L] == 0) 0.5 else max(abs(rng[1L]) * 0.01, .Machine$double.eps)
    rng <- c(rng[1L] - delta, rng[2L] + delta)
  }
  seq(rng[1L], rng[2L], length.out = n_breaks)
}

.tinyarray_bundle_plots <- function(...) {
  plots <- list(...)
  plots <- plots[!vapply(plots, is.null, logical(1))]
  if (!length(plots)) {
    return(NULL)
  }
  if (length(plots) == 1L) {
    return(plots[[1L]])
  }
  patchwork::wrap_plots(plots) + patchwork::plot_layout(guides = "collect")
}

.tinyarray_geo_drop_redundant_columns <- function(pd,
                                                  drop_url = TRUE,
                                                  drop_characteristics = FALSE) {
  if (!is.data.frame(pd)) {
    pd <- as.data.frame(pd, stringsAsFactors = FALSE, check.names = FALSE)
  }
  if (!ncol(pd)) {
    return(pd[, , drop = FALSE])
  }
  keep <- rep(TRUE, ncol(pd))
  if (isTRUE(drop_characteristics)) {
    # Drop the raw GEOquery characteristics columns, but keep the derived
    # `something:ch1` columns that carry the cleaned sample annotations.
    keep <- keep & !grepl("^characteristics_ch1(\\.|$)", colnames(pd), ignore.case = TRUE)
  }

  if (drop_url) {
    urlish <- vapply(
      pd,
      function(x) {
        non_na <- as.character(x)
        non_na <- non_na[!is.na(non_na) & nzchar(non_na)]
        length(non_na) > 0L && all(grepl("^(https?|ftp)://|^www\\.", non_na, ignore.case = TRUE))
      },
      logical(1)
    )
    keep[urlish] <- FALSE
  }

  if (!any(keep)) {
    return(pd[, 0, drop = FALSE])
  }

  pd[, keep, drop = FALSE]
}

.tinyarray_pvalue_column <- function(adjust = FALSE) {
  if (adjust) "padj" else "P.value"
}

.tinyarray_orgdb_for_species <- function(species) {
  species <- tolower(species)
  if (species == "human") {
    pkg <- "org.Hs.eg.db"
    kegg <- "hsa"
  } else if (species == "mouse") {
    pkg <- "org.Mm.eg.db"
    kegg <- "mmu"
  } else if (species == "rat") {
    pkg <- "org.Rn.eg.db"
    kegg <- "rno"
  } else {
    stop("species should be one of human, mouse, rat", call. = FALSE)
  }
  if (!requireNamespace(pkg, quietly = TRUE)) {
    stop("Package \"", pkg, "\" needed for this function to work.", call. = FALSE)
  }
  list(pkg = pkg, kegg = kegg, orgdb = getExportedValue(pkg, pkg))
}

.tinyarray_require_clusterProfiler <- function() {
  if (!requireNamespace("clusterProfiler", quietly = TRUE)) {
    stop(
      "Package \"clusterProfiler\" needed for this function to work. ",
      "Please install it by install.packages('clusterProfiler') or ",
      "BiocManager::install('clusterProfiler')",
      call. = FALSE
    )
  }
  invisible(TRUE)
}

.tinyarray_dedup_by_key <- function(df, key) {
  if (!key %in% colnames(df)) {
    stop("column `", key, "` not found", call. = FALSE)
  }
  df <- df[!is.na(df[[key]]), , drop = FALSE]
  df <- df[!duplicated(df[[key]]), , drop = FALSE]
  df
}

.tinyarray_pick_probe_symbol_columns <- function(x) {
  if (is.null(x)) {
    return(NULL)
  }
  nms <- names(x)
  if (is.null(nms)) {
    nms <- rep("", ncol(x))
  }
  norm <- tolower(gsub("[^a-z0-9]+", "", nms))
  probe_candidates <- c("probe_id", "probeid", "idref", "id", "probe", "probesetid")
  symbol_candidates <- c("symbol", "symbols", "genesymbol", "genename", "genes")

  probe_idx <- which(norm %in% probe_candidates)
  symbol_idx <- which(norm %in% symbol_candidates)

  if (length(probe_idx) && length(symbol_idx)) {
    return(c(probe_idx[1L], symbol_idx[1L]))
  }
  if (length(probe_idx) && ncol(x) >= 2L) {
    other <- setdiff(seq_len(ncol(x)), probe_idx[1L])
    if (length(other)) {
      return(c(probe_idx[1L], other[1L]))
    }
  }
  if (length(symbol_idx) && ncol(x) >= 2L) {
    other <- setdiff(seq_len(ncol(x)), symbol_idx[1L])
    if (length(other)) {
      return(c(other[1L], symbol_idx[1L]))
    }
  }
  if (ncol(x) >= 2L) {
    return(1:2)
  }
  NULL
}

.tinyarray_standardize_probe_symbol_table <- function(x) {
  if (is.null(x)) {
    return(NULL)
  }
  if (!is.data.frame(x)) {
    x <- as.data.frame(x, stringsAsFactors = FALSE)
  }
  cols <- .tinyarray_pick_probe_symbol_columns(x)
  if (is.null(cols) || length(cols) < 2L) {
    stop("annotation table must have at least 2 columns", call. = FALSE)
  }
  x <- x[, cols[1:2], drop = FALSE]
  colnames(x) <- c("probe_id", "symbol")
  x$probe_id <- as.character(x$probe_id)
  x$symbol <- as.character(x$symbol)
  keep <- !is.na(x$probe_id) & nzchar(x$probe_id) &
    !is.na(x$symbol) & nzchar(x$symbol)
  x <- x[keep, , drop = FALSE]
  x <- unique(x)
  rownames(x) <- NULL
  x
}

.tinyarray_ensembl_annotation_table <- function(exp, species = "human") {
  org <- .tinyarray_orgdb_for_species(species)
  ensembl_ids <- unique(as.character(rownames(exp)))
  anno <- AnnotationDbi::select(
    org$orgdb,
    keys = ensembl_ids,
    columns = c("SYMBOL", "GENETYPE"),
    keytype = "ENSEMBL"
  )
  anno <- anno[!is.na(anno$ENSEMBL) & nzchar(anno$ENSEMBL) &
                 !is.na(anno$SYMBOL) & nzchar(anno$SYMBOL), ,
               drop = FALSE]
  anno$ENSEMBL <- as.character(anno$ENSEMBL)
  anno$SYMBOL <- as.character(anno$SYMBOL)
  anno$GENETYPE <- as.character(anno$GENETYPE)
  unique(anno)
}

.tinyarray_read_cache <- function(file, object = NULL) {
  tryCatch(
    readRDS(file),
    error = function(rds_error) {
      cache_env <- new.env(parent = emptyenv())
      loaded <- tryCatch(
        load(file, envir = cache_env),
        error = function(load_error) {
          stop("Unable to read cache file as RDS or RData: ", file, call. = FALSE)
        }
      )
      if (!is.null(object)) {
        candidates <- unique(as.character(object))
        found <- candidates[candidates %in% loaded]
        if (!length(found)) {
          stop(
            "Cached object `", paste(candidates, collapse = "`, `"),
            "` not found in ", file,
            call. = FALSE
          )
        }
        return(get(found[[1L]], envir = cache_env, inherits = FALSE))
      }
      if (length(loaded) == 1L) {
        return(get(loaded, envir = cache_env, inherits = FALSE))
      }
      mget(loaded, envir = cache_env, inherits = FALSE)
    }
  )
}

.tinyarray_collapse_mean_rows <- function(exp, groups) {
  exp <- as.matrix(exp)
  keep <- !is.na(groups) & groups != ""
  if (!any(keep)) {
    return(exp[0, , drop = FALSE])
  }
  groups <- as.character(groups[keep])
  exp <- exp[keep, , drop = FALSE]
  split_idx <- split(seq_along(groups), factor(groups, levels = unique(groups)))
  collapsed <- t(vapply(split_idx, function(idx) {
    colMeans(exp[idx, , drop = FALSE], na.rm = TRUE)
  }, numeric(ncol(exp))))
  rownames(collapsed) <- names(split_idx)
  collapsed
}

.tinyarray_kegg_term_cache <- new.env(parent = emptyenv())

.tinyarray_resolve_orgdb <- function(OrgDb) {
  if (is.character(OrgDb) && length(OrgDb) == 1L) {
    if (!requireNamespace(OrgDb, quietly = TRUE)) {
      stop("Package \"", OrgDb, "\" needed for this function to work.", call. = FALSE)
    }
    OrgDb <- getExportedValue(OrgDb, OrgDb)
  }
  OrgDb
}

.tinyarray_annoprobe_type_candidates <- function() {
  c("bioc", "soft", "pipe")
}

.tinyarray_dput_string <- function(x) {
  paste(utils::capture.output(dput(x)), collapse = "\n")
}

.tinyarray_with_suppressed_download <- function(expr) {
  result <- NULL
  utils::capture.output({
    result <- force(expr)
  })
  result
}

.tinyarray_run_annoprobe_idmap <- function(gpl, type, destdir = tempdir(), mirror = "tencent") {
  if (!requireNamespace("AnnoProbe", quietly = TRUE)) {
    return(NULL)
  }
  .tinyarray_with_suppressed_download(
    tryCatch(
      suppressMessages(suppressWarnings(
        AnnoProbe::idmap(gpl = gpl, type = type, mirror = mirror, destdir = destdir)
      )),
      error = function(e) e
    )
  )
}

.tinyarray_try_annoprobe_idmap <- function(gpl, type = NULL, destdir = tempdir(), mirror = "tencent") {
  if (!requireNamespace("AnnoProbe", quietly = TRUE)) {
    return(NULL)
  }
  requested <- if (is.null(type)) character() else unique(as.character(type))
  types <- unique(c(requested, .tinyarray_annoprobe_type_candidates()))
  types <- types[nzchar(types)]
  if (!length(types)) {
    return(NULL)
  }
  errors <- list()
  for (tp in types) {
    res <- .tinyarray_run_annoprobe_idmap(gpl = gpl, type = tp, destdir = destdir, mirror = mirror)
    if (is.data.frame(res) || is.matrix(res)) {
      return(list(
        data = res,
        type = tp,
        code = paste0("ids <- AnnoProbe::idmap(\"", gpl, "\", type = \"", tp, "\")"),
        errors = errors
      ))
    }
    errors[[tp]] <- if (inherits(res, "error")) {
      conditionMessage(res)
    } else {
      paste("unexpected result class:", paste(class(res), collapse = "/"))
    }
  }
  structure(
    list(
      errors = errors,
      tried = types
    ),
    class = "tinyarray_annoprobe_failure"
  )
}

.tinyarray_bioc_annotation_code <- function(pkg) {
  paste0(
    "library(",
    pkg,
    ".db); ids <- toTable(",
    pkg,
    "SYMBOL)"
  )
}

.tinyarray_kegg_term_tables <- function(OrgDb, species) {
  species <- tolower(species)
  cache_key <- paste0(species, "::", paste(class(OrgDb), collapse = "/"))
  if (exists(cache_key, envir = .tinyarray_kegg_term_cache, inherits = FALSE)) {
    return(get(cache_key, envir = .tinyarray_kegg_term_cache, inherits = FALSE))
  }
  OrgDb <- .tinyarray_resolve_orgdb(OrgDb)
  gene_ids <- AnnotationDbi::keys(OrgDb, keytype = "ENTREZID")
  term2gene <- tryCatch(
    suppressWarnings(
      clusterProfiler::bitr(
        gene_ids,
        fromType = "ENTREZID",
        toType = "PATH",
        OrgDb = OrgDb,
        drop = FALSE
      )
    ),
    error = function(e) NULL
  )
  if (is.null(term2gene) || !nrow(term2gene)) {
    assign(cache_key, NULL, envir = .tinyarray_kegg_term_cache)
    return(NULL)
  }
  term2gene <- term2gene[!is.na(term2gene$PATH) & nzchar(term2gene$PATH), c("PATH", "ENTREZID"), drop = FALSE]
  if (!nrow(term2gene)) {
    assign(cache_key, NULL, envir = .tinyarray_kegg_term_cache)
    return(NULL)
  }
  term2gene <- unique(term2gene)
  term2gene$PATH <- paste0(.tinyarray_orgdb_for_species(species)$kegg, term2gene$PATH)
  colnames(term2gene) <- c("term", "gene")
  term2name <- unique(term2gene["term"])
  term2name$name <- term2name$term
  res <- list(term2gene = term2gene, term2name = term2name)
  assign(cache_key, res, envir = .tinyarray_kegg_term_cache)
  res
}

.tinyarray_extid2symbol <- function(OrgDb, gene_ids, keytype) {
  OrgDb <- .tinyarray_resolve_orgdb(OrgDb)
  gene_ids <- unique(as.character(gene_ids))
  if (length(gene_ids) == 0L) {
    return(stats::setNames(character(), character()))
  }
  gn.df <- suppressWarnings(
    clusterProfiler::bitr(
      gene_ids,
      fromType = keytype,
      toType = "SYMBOL",
      OrgDb = OrgDb,
      drop = FALSE
    )
  )
  if (!is.data.frame(gn.df) || nrow(gn.df) == 0L) {
    return(stats::setNames(gene_ids, gene_ids))
  }
  gn.df <- unique(gn.df[, c(keytype, "SYMBOL"), drop = FALSE])
  gn.df <- gn.df[!is.na(gn.df[[keytype]]) & !is.na(gn.df$SYMBOL) &
                   nzchar(gn.df[[keytype]]) & nzchar(gn.df$SYMBOL), , drop = FALSE]
  gn.df <- gn.df[!duplicated(gn.df[[keytype]]), , drop = FALSE]
  gn <- as.character(gn.df$SYMBOL)
  names(gn) <- as.character(gn.df[[keytype]])
  unmap_gene_ids <- gene_ids[!gene_ids %in% names(gn)]
  if (length(unmap_gene_ids) != 0L) {
    gn <- c(gn, stats::setNames(unmap_gene_ids, unmap_gene_ids))
  }
  gn
}

.tinyarray_relabel_gene_string <- function(x, symbol_map) {
  if (length(x) != 1L || is.na(x) || !nzchar(x)) {
    return(x)
  }
  gene_ids <- strsplit(x, "/", fixed = TRUE)[[1]]
  mapped <- unname(symbol_map[gene_ids])
  missing <- is.na(mapped) | !nzchar(mapped)
  if (any(missing)) {
    mapped[missing] <- gene_ids[missing]
  }
  paste(mapped, collapse = "/")
}

.tinyarray_make_readable_enrich <- function(x, OrgDb, keyType) {
  if (is.null(x)) {
    return(NULL)
  }
  if (inherits(x, "compareClusterResult")) {
    res <- as.data.frame(x)
    gene_col <- if ("core_enrichment" %in% colnames(res)) "core_enrichment" else "geneID"
    gene_ids <- unique(unlist(strsplit(as.character(res[[gene_col]]), "/", fixed = TRUE)))
    symbol_map <- .tinyarray_extid2symbol(OrgDb, gene_ids = gene_ids, keytype = keyType)
    if (gene_col %in% colnames(res)) {
      res[[gene_col]] <- vapply(res[[gene_col]], .tinyarray_relabel_gene_string, character(1), symbol_map = symbol_map)
    }
    x@compareClusterResult <- res
  } else if (inherits(x, "gseaResult")) {
    res <- x@result
    gene_col <- if ("core_enrichment" %in% colnames(res)) "core_enrichment" else "geneID"
    gene_ids <- names(x@geneList)
    symbol_map <- .tinyarray_extid2symbol(OrgDb, gene_ids = gene_ids, keytype = keyType)
    if (gene_col %in% colnames(res)) {
      res[[gene_col]] <- vapply(res[[gene_col]], .tinyarray_relabel_gene_string, character(1), symbol_map = symbol_map)
    }
    x@result <- res
  } else if (inherits(x, "enrichResult")) {
    res <- x@result
    gene_col <- if ("core_enrichment" %in% colnames(res)) "core_enrichment" else "geneID"
    symbol_map <- .tinyarray_extid2symbol(OrgDb, gene_ids = x@gene, keytype = keyType)
    if (gene_col %in% colnames(res)) {
      res[[gene_col]] <- vapply(res[[gene_col]], .tinyarray_relabel_gene_string, character(1), symbol_map = symbol_map)
    }
    x@result <- res
  } else {
    stop("Unsupported enrichment result class.", call. = FALSE)
  }
  x@gene2Symbol <- symbol_map
  x@keytype <- keyType
  x@readable <- TRUE
  x
}

##' geo_parser
##'
##' Legacy helper for parsing GEO series matrix files.
##' Prefer [geo_download()] for normal use.
##'
##' @param gse GEO accession number
##' @param destdir The destination directory for data downloads
##' @return an ExpressionSet object list
##' @author Xiaojie Sun
##' @export
##' @examples
##' \dontrun{
##' gse = "GSE42872"
##' a = try(geo_parser(gse,destdir=tempdir()), silent = TRUE)
##' }
##' @seealso
##' \code{\link{find_anno}}

geo_parser <- function(gse,destdir = getwd()) {
  series_matrix_file = paste0(gse,'_series_matrix.txt.gz')
  if(!file.exists(paste0(destdir,"/",series_matrix_file))){
    message(paste0("Downloading ", series_matrix_file, " from GEO ..."))
  }
  if(requireNamespace("GEOquery",quietly = TRUE)) {
    eSet = tryCatch({GEOquery::getGEO(gse, destdir = destdir,getGPL = FALSE)
    },error = function(e){555})
  }else{
    eSet = 555
  }

  if(!file.exists(paste0(destdir,"/",series_matrix_file))){
    stop('Unable to download the series matrix file. Please download the series
         matrix file from GEO, place it in your working directory, and then
         rerun this function.')
  }

  if(is.list(eSet)){
    return(eSet)
  }else{
    if(!requireNamespace("Biobase",quietly = TRUE)) {
      stop("Package \"Biobase\" needed for this function to work.
           Please install it by BiocManager::install('Biobase')",call. = FALSE)
    }else{
      lines <- readLines(series_matrix_file)
      pd <- strsplit(gsub("^!|\"", "", lines), "\t", fixed = TRUE)
      width <- max(lengths(pd))
      pd <- do.call(rbind, lapply(pd, function(x) {
        length(x) <- width
        x
      }))
      k <- startsWith(pd[,1], "Sample_")
      pd <- pd[k, ]
      rownames(pd) <- sub("^Sample_", "", pd[,1])
      pd <- t(pd[,-1])
      rownames(pd) = pd[,2]
      metaData <- data.frame(labelDescription=colnames(pd))
      pd <- Biobase::AnnotatedDataFrame(data = data.frame(pd),varMetadata = metaData)
      exp <- utils::read.delim(series_matrix_file,
                               comment.char = "!",
                               row.names = 1) |>
        as.matrix()
      gpl = sub(".*(GPL\\d+).*", "\\1", lines[which(grepl("GPL", lines))[1]])
      eSet = list(Biobase::ExpressionSet(assayData = exp,phenoData = pd,annotation = gpl))
      return(eSet)
    }
  }
}

.tinyarray_geo_annoprobe_url <- function(gse) {
  gse <- toupper(as.character(gse)[1L])
  if (!grepl("^GSE[0-9]+$", gse)) {
    return(NA_character_)
  }
  down <- if (as.numeric(gsub("GSE", "", gse)) < 1000) {
    paste0("/GEOmirror/GSEnnn/", gse, "_eSet.Rdata")
  } else {
    paste0("/GEOmirror/", gsub("[0-9][0-9][0-9]$", "nnn", gse), "/", gse, "_eSet.Rdata")
  }
  paste0("http://49.235.27.111", down)
}

.tinyarray_geo_guess_gse_from_cache_path <- function(path) {
  nm <- basename(path)
  m <- regexpr("GSE[0-9]+", nm, ignore.case = TRUE)
  if (m[1L] < 0L) {
    return(NA_character_)
  }
  toupper(regmatches(nm, m)[1L])
}

.tinyarray_geo_remote_content_length <- function(url, timeout = getOption("timeout")) {
  if (!requireNamespace("httr2", quietly = TRUE)) {
    return(NA_real_)
  }
  timeout_value <- suppressWarnings(as.numeric(timeout)[1L])
  if (is.na(timeout_value) || !is.finite(timeout_value) || timeout_value <= 0) {
    timeout_value <- getOption("timeout")
  }
  timeout_value <- min(30L, max(1L, as.integer(timeout_value)))
  resp <- tryCatch(
    httr2::request(url) |>
      httr2::req_method("HEAD") |>
      httr2::req_timeout(timeout_value) |>
      httr2::req_perform(),
    error = function(e) NULL
  )
  if (is.null(resp)) {
    return(NA_real_)
  }
  cl <- httr2::resp_header(resp, "content-length")
  cl <- suppressWarnings(as.numeric(cl)[1L])
  if (is.na(cl) || !is.finite(cl) || cl <= 0) {
    return(NA_real_)
  }
  cl
}

.tinyarray_geo_cache_is_complete <- function(file,
                                             gse = NULL,
                                             cache_kind = c("rds", "rdata"),
                                             download_timeout = getOption("timeout")) {
  cache_kind <- match.arg(cache_kind)
  if (!file.exists(file)) {
    return(FALSE)
  }
  if (cache_kind == "rdata" && !is.null(gse)) {
    expected <- .tinyarray_geo_remote_content_length(
      .tinyarray_geo_annoprobe_url(gse),
      timeout = download_timeout
    )
    if (!is.na(expected)) {
      actual <- suppressWarnings(as.numeric(file.size(file))[1L])
      if (!is.na(actual) && actual != expected) {
        message(
          "The cache file is incomplete: ", file, ". ",
          "The partial file will be kept for resume."
        )
        return(FALSE)
      }
    }
  }
  TRUE
}

.tinyarray_geo_read_cache_safe <- function(file,
                                           object = NULL,
                                           gse = NULL,
                                           cache_kind = c("rds", "rdata"),
                                           download_timeout = getOption("timeout")) {
  cache_kind <- match.arg(cache_kind)
  if (!.tinyarray_geo_cache_is_complete(
    file = file,
    gse = gse,
    cache_kind = cache_kind,
    download_timeout = download_timeout
  )) {
    return(NULL)
  }
  suppressWarnings(
    tryCatch(
      .tinyarray_read_cache(file, object = object),
      error = function(e) {
        message("The cache file is incomplete: ", file, ". The partial file will be kept for resume.")
        NULL
      }
    )
  )
}

.tinyarray_geo_manual_download_message <- function(gse) {
  series_matrix_file <- paste0(gse, "_series_matrix.txt.gz")
  paste0(
    "Unable to obtain GEO data for ", gse, ". ",
    "Please download `", series_matrix_file, "` from the GEO website, ",
    "place it in your working directory, and rerun `geo_download(\"", gse, "\")`."
  )
}

.tinyarray_geo_try_annoprobe_geoChina <- function(gse,
                                                 destdir = getwd(),
                                                 download_timeout = getOption("timeout")) {
  if (!requireNamespace("curl", quietly = TRUE)) {
    return(555)
  }

  timeout_value <- suppressWarnings(as.numeric(download_timeout)[1L])
  if (is.na(timeout_value) || !is.finite(timeout_value) || timeout_value <= 0) {
    timeout_value <- getOption("timeout")
  }
  timeout_value <- max(1800L, as.integer(timeout_value))

  url <- .tinyarray_geo_annoprobe_url(gse)
  if (is.na(url) || !nzchar(url)) {
    return(555)
  }
  tpf <- file.path(destdir, paste0(gse, "_eSet.Rdata"))
  expected <- .tinyarray_geo_remote_content_length(url, timeout = timeout_value)
  last_error <- NULL

  if (!is.na(expected) && file.exists(tpf)) {
    current_size <- suppressWarnings(as.numeric(file.size(tpf))[1L])
    if (!is.na(current_size) && current_size > expected) {
      unlink(tpf)
    }
  }

  existing_size <- if (file.exists(tpf)) suppressWarnings(as.numeric(file.size(tpf))[1L]) else 0
  if (is.na(existing_size) || existing_size < 0) {
    existing_size <- 0
  }

  if (existing_size > 0) {
    message("The cache file is incomplete. Resuming download from the partial file: ", tpf)
  } else {
    message("Downloading ", basename(tpf), " from the AnnoProbe mirror ...")
  }

  handle <- curl::new_handle()
  curl::handle_setopt(
    handle,
    timeout = 0,
    connecttimeout = timeout_value,
    low_speed_time = timeout_value,
    low_speed_limit = 1,
    resume_from = existing_size
  )

  ok <- tryCatch(
    {
      curl::curl_download(
        url = url,
        destfile = tpf,
        quiet = TRUE,
        mode = if (existing_size > 0) "ab" else "wb",
        handle = handle
      )
      TRUE
    },
    error = function(e) {
      last_error <<- conditionMessage(e)
      FALSE
    }
  )

  actual_size <- if (file.exists(tpf)) suppressWarnings(as.numeric(file.size(tpf))[1L]) else 0
  if (ok) {
    if (!is.na(expected) && !is.na(actual_size) && actual_size < expected) {
      last_error <- paste0("download incomplete (expected ", expected, ", got ", actual_size, ")")
    } else {
      res <- tryCatch(
        .tinyarray_read_cache(tpf, object = c("eSet", "gset")),
        error = function(e) {
          last_error <<- conditionMessage(e)
          NULL
        }
      )
      if (!is.null(res)) {
        return(res)
      }
    }
  }

  if (file.exists(tpf) && suppressWarnings(as.numeric(file.size(tpf))[1L]) > 0) {
    if (!is.null(last_error) && grepl("404", last_error, fixed = TRUE)) {
      return(555)
    }
    warning(
      "Mirror download for ",
      gse,
      " is incomplete; the partial file was kept for resume.",
      call. = FALSE
    )
  } else if (!is.null(last_error)) {
    if (grepl("404", last_error, fixed = TRUE)) {
      return(555)
    }
    warning(
      "Mirror download failed for ",
      gse,
      ": ",
      last_error,
      call. = FALSE
    )
  }
  555
}


##' geo_download
##'
##' Download GEO expression data and return the expression matrix, phenotype
##' data, and platform information.
##'
##' Cached `eSet` objects are stored as `.rds` files in `destdir`; legacy
##' `.Rdata` cache files are still readable, and incomplete cache files will be
##' kept so the download can resume automatically.
##'
##' @param gse GEO accession number or a path to a cached `.rds` or `.RData`
##'   file containing an `ExpressionSet` or a list of `ExpressionSet` objects
##' @param by_annopbrobe whether to try the AnnoProbe cache first and fall back
##'   to GEOquery::getGEO()
##' @param simpd whether to simplify phenotype data by dropping columns with a
##'   single unique value
##' @param colon_remove whether to drop raw `characteristics_ch1*` columns
##'   while keeping the derived `:ch1` phenotype column names intact
##' @param download_timeout timeout in seconds used for GEOquery fallback and
##'   for mirror stall detection during the resumable download
##' @param destdir The destination directory for data downloads
##' @param n For data with more than one ExpressionSet, specify which one to analyze
##' @return a list with `exp`, `pd`, `gpl`, and `group_candidates`, or `NULL`
##'   when the data cannot be read or downloaded
##' @author Xiaojie Sun
##' @export
##' @examples
##' \dontrun{
##' if(requireNamespace("Biobase",quietly = TRUE)){
##'   gse = "GSE42872"
##'   a = geo_download(gse,destdir=tempdir())
##' }
##' }
##' @seealso
##' \code{\link{find_anno}}

geo_download <- function(gse, by_annopbrobe = TRUE,
                         simpd = TRUE, colon_remove = TRUE,
                         download_timeout = getOption("timeout"),
                         destdir = getwd(), n = 1) {
  tryCatch(
    .tinyarray_geo_download_impl(
      gse = gse,
      by_annopbrobe = by_annopbrobe,
      simpd = simpd,
      colon_remove = colon_remove,
      download_timeout = download_timeout,
      destdir = destdir,
      n = n
    ),
    error = function(e) {
      msg <- paste0("geo_download() failed: ", conditionMessage(e))
      tryCatch(
        warning(msg, call. = FALSE),
        error = function(w) message("Warning: ", msg)
      )
      NULL
    }
  )
}

.tinyarray_geo_download_impl <- function(gse, by_annopbrobe = TRUE,
                                         simpd = TRUE, colon_remove = TRUE,
                                         download_timeout = getOption("timeout"),
                                         destdir = getwd(), n = 1) {
  if (length(gse) != 1L || is.na(gse) || !nzchar(trimws(as.character(gse)))) {
    stop("`gse` must be one GEO accession or one cache-file path.", call. = FALSE)
  }
  gse_input <- trimws(as.character(gse))
  direct_cache <- path.expand(gse_input)
  if (!file.exists(direct_cache)) {
    cache_in_destdir <- file.path(destdir, gse_input)
    if (file.exists(cache_in_destdir)) direct_cache <- cache_in_destdir
  }

  if (file.exists(direct_cache) && !dir.exists(direct_cache)) {
    cache_kind <- if (grepl("\\.rds$", direct_cache, ignore.case = TRUE)) "rds" else "rdata"
    eSet <- .tinyarray_geo_read_cache_safe(
      direct_cache,
      object = c("eSet", "gset"),
      gse = if (cache_kind == "rdata") basename(direct_cache) else NULL,
      cache_kind = cache_kind,
      download_timeout = download_timeout
    )
    if (is.null(eSet)) {
      gse_guess <- .tinyarray_geo_guess_gse_from_cache_path(direct_cache)
      if (is.na(gse_guess)) {
        stop(
          "The cache file is incomplete and cannot be resumed because the GEO accession could not be inferred from its filename: ",
          direct_cache,
          call. = FALSE
        )
      }
      message(
        "The cache file is incomplete. The partial file will be kept and the download will resume automatically: ",
        direct_cache
      )
      return(.tinyarray_geo_download_impl(
        gse = gse_guess,
        by_annopbrobe = by_annopbrobe,
        simpd = simpd,
        colon_remove = colon_remove,
        download_timeout = download_timeout,
        destdir = dirname(direct_cache),
        n = n
      ))
    }
    gse_guess <- .tinyarray_geo_guess_gse_from_cache_path(direct_cache)
    gse <- if (!is.na(gse_guess)) gse_guess else basename(direct_cache)
  } else {
    if (grepl("\\.(rds|rdata)$", gse_input, ignore.case = TRUE)) {
      stop("Cache file does not exist: ", gse_input, call. = FALSE)
    }
    gse = toupper(gse_input)
    if (!grepl("^GSE[0-9]+$", gse)) {
      stop(
        "`gse` must be a GEO accession such as \"GSE7305\" or an existing cache-file path.",
        call. = FALSE
      )
    }
    if (by_annopbrobe) {
      cache_file = file.path(destdir,paste0(gse,"_eSet.rds"))
      legacy_cache_file = file.path(destdir,paste0(gse,"_eSet.Rdata"))
      eSet <- NULL
      if (file.exists(cache_file)) {
        eSet <- .tinyarray_geo_read_cache_safe(
          cache_file,
          object = "eSet",
          cache_kind = "rds",
          download_timeout = download_timeout
        )
      }
      if (is.null(eSet) && file.exists(legacy_cache_file)) {
        eSet <- .tinyarray_geo_read_cache_safe(
          legacy_cache_file,
          object = c("eSet", "gset"),
          gse = gse,
          cache_kind = "rdata",
          download_timeout = download_timeout
        )
        if (!is.null(eSet)) {
          saveRDS(eSet, cache_file)
          if (file.exists(legacy_cache_file)) unlink(legacy_cache_file)
        }
      }
      if (is.null(eSet)) {
        eSet <- .tinyarray_geo_try_annoprobe_geoChina(
          gse = gse,
          destdir = destdir,
          download_timeout = download_timeout
        )

        if (!is.list(eSet)) {
          if (file.exists(legacy_cache_file) && file.size(legacy_cache_file) > 0) {
            stop(
              "The AnnoProbe mirror download for ",
              gse,
              " did not finish. The partial cache was kept for resume, so rerun ",
              "`geo_download()` to continue the same download.",
              call. = FALSE
            )
          }
          message("This data is not indexed by AnnoProbe, downloading from GEO ...")
          if (!requireNamespace("GEOquery", quietly = TRUE)) {
            stop("Package \"GEOquery\" needed for this function to work.
         Please install it by BiocManager::install('GEOquery')", call. = FALSE)
          }
          timeout_value <- suppressWarnings(as.numeric(download_timeout)[1L])
          if (is.na(timeout_value) || !is.finite(timeout_value) || timeout_value <= 0) {
            timeout_value <- getOption("timeout")
          }
          timeout_value <- max(1, as.integer(timeout_value))
          old_timeout <- options(timeout = timeout_value)
          eSet <- tryCatch({GEOquery::getGEO(gse, destdir = destdir, getGPL = FALSE)
          }, error = function(e) {555})
          options(old_timeout)
          if (!is.list(eSet)) {
            stop(.tinyarray_geo_manual_download_message(gse), call. = FALSE)
          }
        }
        if (is.list(eSet)) {
          saveRDS(eSet, cache_file)
          if (file.exists(legacy_cache_file)) unlink(legacy_cache_file)
        }
      }
    } else {
      if(!requireNamespace("GEOquery",quietly = TRUE)) {
        stop("Package \"GEOquery\" needed for this function to work.
         Please install it by BiocManager::install('GEOquery')",call. = FALSE)
      }
      timeout_value <- suppressWarnings(as.numeric(download_timeout)[1L])
    if (is.na(timeout_value) || !is.finite(timeout_value) || timeout_value <= 0) {
      timeout_value <- getOption("timeout")
    }
      timeout_value <- max(1, as.integer(timeout_value))
      old_timeout <- options(timeout = timeout_value)
      eSet <- tryCatch({GEOquery::getGEO(gse, destdir = destdir,getGPL = FALSE)
      },error = function(e){555})
      options(old_timeout)
      if (!is.list(eSet)) {
        stop(.tinyarray_geo_manual_download_message(gse), call. = FALSE)
      }
    }
  }
  if (inherits(eSet, "ExpressionSet")) eSet <- list(eSet)
  if(!is.list(eSet)) {
    stop(.tinyarray_geo_manual_download_message(gse), call. = FALSE)
  }
  if(length(n)!=1) stop("only one ExpresssionSet can be analyzed")
  if(length(eSet)==1 & n!=1) {
    n = 1
    warning("this data only has one ExpressionSet object")
  }
  if(!requireNamespace("Biobase",quietly = TRUE)) {
    stop("Package \"Biobase\" needed for this function to work.
         Please install it by BiocManager::install('Biobase')",call. = FALSE)
  }
  exp <- Biobase::exprs(eSet[[n]])
  pd <- Biobase::pData(eSet[[n]])
  if(simpd){
    colname <- vector("character")
    count <- vector("integer")
    for (i in 1:ncol(pd)) {
      colname[i] = colnames(pd)[[i]]
      count[i] = nrow(pd[!duplicated(pd[, i]), ])
    }
      df <- data.frame(colname, count, stringsAsFactors = FALSE)
      df <- df[order(df$count, decreasing = TRUE), , drop = FALSE]
      df <- df[df$count > 1, , drop = FALSE]
      pd <-  pd[, df$colname, drop = FALSE]
      pd <- pd[, !(colnames(pd) %in% c("geo_accession", "supplementary_file")), drop = FALSE]
  }
  pd <- .tinyarray_geo_drop_redundant_columns(
    pd,
    drop_url = TRUE,
    drop_characteristics = colon_remove
  )
  drop_pd_cols <- intersect(colnames(pd), c("last_update_date", "relation", "relation."))
  if (length(drop_pd_cols)) {
    pd <- pd[, setdiff(colnames(pd), drop_pd_cols), drop = FALSE]
  }
  if (ncol(pd) == 0L) {
    pd_full <- Biobase::pData(eSet[[n]])
    pd_full <- .tinyarray_geo_drop_redundant_columns(
      pd_full,
      drop_url = TRUE,
      drop_characteristics = colon_remove
    )
    drop_pd_full_cols <- intersect(colnames(pd_full), c("last_update_date", "relation", "relation."))
    pd_full <- pd_full[, setdiff(colnames(pd_full), drop_pd_full_cols), drop = FALSE]
    if (ncol(pd_full) == 0L) {
      pd <- pd_full[, 0, drop = FALSE]
      warning("pd simplification removed all usable columns; returning an empty pd.", call. = FALSE)
    } else {
      pd <- pd_full[, 1, drop = FALSE]
      warning("pd simplification removed all columns; keeping the first usable column.", call. = FALSE)
    }
  }
  p1 = identical(rownames(pd),colnames(exp))
  p2 = all(rownames(pd) %in% colnames(exp) & colnames(exp) %in% rownames(pd))
  if(!p1) {
    exp = exp[,match(rownames(pd),colnames(exp))]
    if(!p2) {
      exp = exp[,intersect(rownames(pd),colnames(exp))]
      pd = pd[intersect(rownames(pd),colnames(exp)),]
    }
  }
  gpl <- eSet[[n]]@annotation
  pd2 <- as.data.frame(lapply(pd, as.character), stringsAsFactors = FALSE, check.names = FALSE)
  drop_pd2_cols <- intersect(colnames(pd2), c("last_update_date", "relation", "relation."))
  if (length(drop_pd2_cols)) {
    pd2 <- pd2[, setdiff(colnames(pd2), drop_pd2_cols), drop = FALSE]
  }
  rownames(pd2) <- rownames(pd)
  group_candidates <- .tinyarray_geo_group_candidates(
    pd2,
    group_col = "title",
    search_other_cols = TRUE
  )
  re = list(exp=exp,pd=pd2,gpl=gpl,group_candidates=group_candidates)
  if(!is.null(dim(exp)) && nrow(exp)!=0 && any(is.na(exp)|is.nan(exp))) {
    warning("NA or NaN values detected")
  }else if (any(exp<0)) {
    warning("negative values detected")
  } else if(!is.null(dim(exp)) && nrow(exp)!=0){
    message(paste(nrow(exp),"probes,",
                  ncol(exp),"samples",
                  "from",min(exp),
                  "to",max(exp)))}
  return(re)
}

##' Find an annotation table for a GEO platform accession.
##'
##' The function prints a short explanation plus the code used to obtain
##' annotations. It only tests AnnoProbe `type` values when no Bioconductor
##' annotation package source exists for the GPL accession.
##'
##' @param gpl a GEO platform accession
##' @param install whether to install and library the package
##' @param update whether to update the package
##' @return No return value. The function prints available annotation resources
##'   and optionally installs a Bioconductor annotation package.
##' @author Xiaojie Sun
##' @export
##' @examples
##' \dontrun{
##' try(find_anno("GPL570"), silent = TRUE)
##' }
##' @seealso
##' \code{\link{geo_download}}

find_anno <-function(gpl,install = FALSE,update = FALSE){
  gpl = toupper(gpl)
  bioc_msg <- NULL
  anno_msg <- NULL

  if(gpl %in% pkg_all$gpl) {
    qz = pkg_all$bioc_package[pkg_all$gpl== gpl]
    bioc_msg <- .tinyarray_bioc_annotation_code(qz)
    if(install){
      if(!requireNamespace("BiocManager", quietly = TRUE)) {
        stop("Package \"BiocManager\" needed for this function to work. Please install it by install.packages('BiocManager')",
             call. = FALSE)
      }
      if(!suppressMessages(requireNamespace(paste0(qz,".db")))){
        BiocManager::install(paste0(qz,".db"),update = update,ask = FALSE)
        suppressMessages(requireNamespace(paste0(qz,".db")))
      }
    }
  }

  if (is.null(bioc_msg) && requireNamespace("AnnoProbe", quietly = TRUE)) {
    anno_try <- .tinyarray_try_annoprobe_idmap(gpl)
    if (!is.null(anno_try) && !is.null(anno_try$code)) {
      anno_msg <- anno_try$code
    }
  }

  if (!is.null(bioc_msg)) {
    message("The following code can be used to obtain annotations:")
    message(bioc_msg)
  }
  if (!is.null(anno_msg)) {
    if (!is.null(bioc_msg)) {
      message("The following code snippet is also available:")
    } else {
      message("The following code can be used to obtain annotations:")
    }
    message(anno_msg)
  }
  if (is.null(bioc_msg) && is.null(anno_msg)) {
    message(
      "No annotation table is available for ",
      gpl,
      ". Please extract it manually from the GPL platform table. ",
      "You can use `get_gpl_txt(\"",
      gpl,
      "\")` to get the download link for the GPL platform table."
    )
  }
}

##' Get probe-to-symbol ids for a GEO platform accession.
##'
##' Prefer the available Bioconductor annotation package when possible, and
##' fall back to `AnnoProbe::idmap()` when no Bioconductor annotation package
##' source exists. The function prints a short explanation plus the code it
##' used.
##'
##' @param gpl a GEO platform accession
##' @param type optional AnnoProbe `type` setting. If omitted, the function
##'   tries candidate types automatically.
##' @param install whether to install and library the package
##' @param update whether to update the package
##' @return a cleaned two-column data.frame with `probe_id` and `symbol`
##' @author Xiaojie Sun
##' @export
##' @examples
##' \dontrun{
##' get_ids("GPL570")
##' }
##' @seealso
##' \code{\link{find_anno}}

get_ids <- function(gpl, type = NULL, install = FALSE, update = FALSE) {
  gpl <- toupper(gpl)
  anno <- NULL
  anno_type <- NULL
  anno_code <- NULL
  has_bioc_source <- gpl %in% pkg_all$gpl
  bioc_label <- NULL

  if (has_bioc_source) {
    qz <- pkg_all$bioc_package[pkg_all$gpl == gpl]
    bioc_label <- paste0(qz, ".db")
    if (install) {
      if (!requireNamespace("BiocManager", quietly = TRUE)) {
        stop(
          "Package \"BiocManager\" needed for this function to work. Please install it by install.packages('BiocManager')",
          call. = FALSE
        )
      }
      if (!suppressMessages(requireNamespace(bioc_label, quietly = TRUE))) {
        BiocManager::install(bioc_label, update = update, ask = FALSE)
      }
    }
    if (requireNamespace(bioc_label, quietly = TRUE)) {
      anno <- tryCatch(
        AnnotationDbi::toTable(
          getExportedValue(bioc_label, paste0(qz, "SYMBOL"))
        ),
        error = function(e) {
          warning(
            "Failed to read ",
            bioc_label,
            ": ",
            conditionMessage(e),
            call. = FALSE
          )
          NULL
        }
      )
      if (!is.null(anno)) {
        anno_type <- "bioc"
        anno_code <- .tinyarray_bioc_annotation_code(qz)
      }
    } else {
      warning(
        "A Bioconductor annotation package exists for ",
        gpl,
        " but ",
        bioc_label,
        " is not installed; falling back to AnnoProbe.",
        call. = FALSE
      )
    }
  }

  if (!is.null(anno) && !is.null(type)) {
    warning(
      "`type` is ignored because a Bioconductor annotation package source exists.",
      call. = FALSE
    )
  }

  if (is.null(anno)) {
    if (!requireNamespace("AnnoProbe", quietly = TRUE)) {
      warning(
        "Package \"AnnoProbe\" is not available, so the fallback cannot be used for ",
        gpl,
        ".",
        call. = FALSE
      )
      return(NULL)
    }
    requested_type <- if (!is.null(type) && length(type)) unique(as.character(type)) else NULL
    anno_try <- .tinyarray_try_annoprobe_idmap(gpl, type = requested_type, destdir = tempdir())
    if (!is.null(anno_try) && !is.null(anno_try$data)) {
      anno <- anno_try$data
      anno_type <- anno_try$type
      anno_code <- anno_try$code
      if (!is.null(requested_type) && length(requested_type) && !identical(requested_type[[1L]], anno_type)) {
        warning(
          "AnnoProbe::idmap(type = \"",
          requested_type[[1L]],
          "\") failed for ",
          gpl,
          "; using type = \"",
          anno_type,
          "\" instead.",
          call. = FALSE
        )
      }
    } else {
      warning(
        "AnnoProbe::idmap() failed for ",
        gpl,
        " with the available `type` settings. Please use `get_gpl_txt(\"",
        gpl,
        "\")` to get the download link for the GPL platform table.",
        call. = FALSE
      )
      return(NULL)
    }
  }
  anno <- .tinyarray_standardize_probe_symbol_table(anno)
  if (is.null(anno) || !nrow(anno)) {
    message(
      "No annotation table is available for ",
      gpl,
      ". Please extract it manually from the GPL platform table. ",
      "You can use `get_gpl_txt(\"",
      gpl,
      "\")` to get the download link for the GPL platform table."
    )
    return(NULL)
  }
  if (!is.null(anno_code)) {
    message("The following code was used to obtain the annotation table:")
    message(anno_code)
  }
  attr(anno, "source_type") <- anno_type
  anno
}

##' get count from GEO
##'
##' get RNA-seq count file from GEO database
##'
##' @inheritParams geo_download
##' @param download download the txt file or not
##' @return No return value. When `download = TRUE`, the count file is saved in
##'   `destdir`.
##' @author Xiaojie Sun
##' @export
##' @examples
##' get_count_txt("GSE162550",destdir = tempdir())
##' @seealso
##' \code{\link{geo_download}}

get_count_txt <- function(gse,destdir = getwd(),download = FALSE){
  if(!grepl("^(GSE|gse)\\d+$", gse))stop("wrong GSE accession")
  gse = toupper(gse)
  url = paste0("https://www.ncbi.nlm.nih.gov/geo/download/?type=rnaseq_counts&acc=",
             gse,
             "&format=file&file=",
             gse,
             "_raw_counts_GRCh38.p13_NCBI.tsv.gz")
  message(url)

  if(download){utils::download.file(url,destfile = file.path(destdir,paste0(gse,"_raw_counts_GRCh38.p13_NCBI.tsv.gz")))
    message("If the download fails, check that your data is RNA-seq data.")}
}


##' get gpl txt from GEO
##'
##' get gpl annotation txt file from GEO database
##'
##' @inheritParams geo_download
##' @inheritParams get_count_txt
##' @param gpl gpl accession from GEO database
##' @return No return value. When `download = TRUE`, the GPL annotation file is
##'   saved in `destdir`.
##' @author Xiaojie Sun
##' @export
##' @examples
##' get_gpl_txt("GPL23270",destdir = tempdir())
##' @seealso
##' \code{\link{geo_download}}

get_gpl_txt = function(gpl,destdir = getwd(),download = FALSE){
  if(!grepl("^(GPL|gpl)\\d+$", gpl))stop("wrong GPL accession")
  gpl = toupper(gpl)
  url = paste0("https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=",
               gpl,
               "&targ=self&form=text&view=data")
  message(url)
  if(download){utils::download.file(url,destfile = file.path(destdir,paste0(gpl,".txt")))
    message("If the download fails, check that your data is microarray data.")}
}


utils::globalVariables(c("pkg_all","exists_anno_list","gset"))

##' exp_surv
##'
##' draw surv plot for a hub gene expression matrix for tumor samples
##'
##' @inheritParams surv_KM
##' @inheritParams exp_boxplot
##' @importFrom survival survfit
##' @importFrom survival Surv
##' @importFrom survminer ggsurvplot
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 element_text
##' @export
##' @return survival plots list for all genes
##' @author Xiaojie Sun
##' @examplesIf requireNamespace("survival", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)
##' tmp = exp_surv(exprSet_hub1,meta1)
##' patchwork::wrap_plots(tmp)+patchwork::plot_layout(guides = "collect")
##' tmp2 = exp_surv(exprSet_hub1,meta1,cut.point = TRUE)
##' patchwork::wrap_plots(tmp2)+patchwork::plot_layout(guides = "collect")
##' @seealso
##' \code{\link{exp_boxplot}};\code{\link{box_surv}};\code{\link{draw_venn}}


exp_surv <- function(exprSet_hub,meta,cut.point = FALSE,color = c("#2874C5", "#f87669")){
  if(cut.point)cut_point = point_cut(exprSet_hub,meta)

  splots <- lapply(rownames(exprSet_hub), function(g){
    i = which(rownames(exprSet_hub)== g)
    if(cut.point){
      meta$gene=ifelse(as.numeric(exprSet_hub[g,]) > cut_point[[i]],'high','low')
    }else{
      meta$gene=ifelse(as.numeric(exprSet_hub[g,]) > stats::median(as.numeric(exprSet_hub[g,])),'high','low')
    }
    if(length(unique(meta$gene))==1) stop(paste0("gene",g,"with too low expression"))
    sfit1=survfit(Surv(time, event)~gene, data=meta)
    p = ggsurvplot(sfit1,pval =TRUE,
                              palette = rev(color),
                              data = meta,
                              legend = c(0.8,0.8),
                              title = rownames(exprSet_hub)[[i]]
    )
    p2 = p$plot+
      theme(plot.title = element_text(hjust = 0.5))
    return(p2)
  })
  return(splots)
}

##' exp_boxplot
##'
##' draw box plot for a hub gene expression matrix
##'
##' @param exp_hub an expression matrix for hubgenes
##' @param xlab x axis title
##' @param ylab y axis title
##' @param p.label whether to show the exact p-value, if FALSE show significance
##'   symbols
##' @param color color for boxplot
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 aes_string
##' @importFrom ggplot2 geom_boxplot
##' @importFrom ggplot2 scale_fill_manual
##' @importFrom ggplot2 theme_classic
##' @importFrom ggplot2 theme
##' @export
##' @return box plots list for all genes in the matrix
##' @author Xiaojie Sun
##' @examplesIf requireNamespace("ggpubr",quietly = TRUE)
##' k = exp_boxplot(log2(exp_hub1+1));k[[1]]
##' @seealso
##' \code{\link{exp_surv}};\code{\link{box_surv}}

exp_boxplot <-  function(exp_hub,
                         xlab = NULL,
                         ylab = NULL,
                         p.label = FALSE,
                         color = c("#2fa1dd","#f87669")){
  if(nrow(exp_hub)>15)warning(paste0(nrow(exp_hub)," figures will be produced"))
  if(!requireNamespace("ggpubr",quietly = TRUE)) {
    stop("Package \"ggpubr\" needed for this function to work.
         Please install it by install.packages('ggpubr')",call. = FALSE)
  }
  group_list = make_tcga_group(exp_hub)
  k = rownames(exp_hub)
  rownames(exp_hub) = paste0("gene",1:nrow(exp_hub))
  dat = as.data.frame(t(exp_hub))
  dat$group_list = group_list
  color = color[1:length(levels(group_list))]
  names(color) = levels(group_list)
  bplots = lapply(1:nrow(exp_hub),function(i){
    mapping <- do.call(ggplot2::aes, list(
      x = as.name("group_list"),
      y = as.name(rownames(exp_hub)[[i]]),
      fill = as.name("group_list")
    ))
    p <- ggplot(dat, mapping)+
      geom_boxplot()+
      scale_fill_manual(values = color)+
      labs(
        x = if (is.null(xlab)) "" else xlab,
        y = if (is.null(ylab)) k[[i]] else ylab
      )+
      theme_classic()+
      theme(legend.position = "none")
    if (p.label) {
      p <- p + ggpubr::stat_compare_means(
        method = "t.test",
        aes(label = ggplot2::after_stat(p.format))
      )
    } else {
      p <- p + ggpubr::stat_compare_means(
        method = "t.test",
        aes(label = ggplot2::after_stat(p.signif))
      )
    }
    p
  })
  return(bplots)
}

##' box_surv
##'
##' draw box plot for a hub gene expression matrix
##'
##' @inheritParams exp_boxplot
##' @inheritParams exp_surv
##' @param xlab x axis title for the boxplot
##' @param ylab y axis title for the boxplot
##' @param p.label whether to show the exact p-value, if FALSE show significance
##'   symbols
##' @importFrom patchwork plot_layout
##' @importFrom patchwork plot_annotation
##' @importFrom ggplot2 theme
##' @export
##' @return patchwork result for hub genes boxplot and survival plot
##' @author Xiaojie Sun
##' @examplesIf requireNamespace("survival", quietly = TRUE) && requireNamespace("ggpubr", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)
##' k = box_surv(log2(exp_hub1+1),exprSet_hub1,meta1);k[[1]]
##' @seealso
##' \code{\link{exp_boxplot}};\code{\link{exp_surv}}

box_surv <-function(exp_hub,
                    exprSet_hub,
                    meta,
                    xlab = "Group",
                    ylab = "Expression",
                    p.label = FALSE){
  splots = exp_surv(exprSet_hub,meta)
  splots = lapply(splots, function(x) x + theme(plot.title = element_blank()))
  boxplots = exp_boxplot(exp_hub, xlab = xlab, ylab = ylab, p.label = p.label)
  gene_names = rownames(exp_hub)
  if (is.null(gene_names) || length(gene_names) != nrow(exp_hub)) {
    gene_names = rownames(exprSet_hub)
  }
  if (is.null(gene_names) || length(gene_names) != nrow(exp_hub)) {
    gene_names = paste0("gene", seq_len(nrow(exp_hub)))
  }
  layout <- '
  ABB
  '
  wp = lapply(1:nrow(exprSet_hub),function(i){
    p <- boxplots[[i]] + splots[[i]] + plot_layout(design = layout)
    p <- p & theme(legend.position = "none")
    p + plot_annotation(
      title = gene_names[[i]],
      theme = theme(plot.title = element_text(hjust = 0.5))
    )
  })
  return(wp)
}



##' risk_plot
##'
##' draw risk plot
##'
##' @inheritParams exp_surv
##' @inheritParams draw_heatmap
##' @param riskscore a numeric vector of riskscore
##' @importFrom survminer surv_cutpoint
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 aes
##' @importFrom ggplot2 geom_point
##' @importFrom ggplot2 scale_color_manual
##' @importFrom ggplot2 geom_vline
##' @importFrom ggplot2 scale_x_continuous
##' @importFrom ggplot2 theme_bw
##' @importFrom ggplot2 theme
##' @importFrom ggplot2 element_blank
##' @importFrom ggplot2 labs
##' @importFrom ggplot2 scale_color_manual
##' @importFrom ggplot2 scale_y_continuous
##' @importFrom ggplot2 element_line
##' @importFrom patchwork wrap_plots
##' @export
##' @return risk plot
##' @author Xiaojie Sun
##' @examplesIf requireNamespace("survival", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE) && requireNamespace("pheatmap", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)
##' risk_plot(exprSet_hub1,meta1,riskscore = rnorm(nrow(meta1)))
##' @seealso
##' \code{\link{exp_boxplot}};\code{\link{box_surv}};\code{\link{draw_venn}}

risk_plot = function(exprSet_hub,meta,riskscore,
                     cut.point = FALSE,color = c("#2fa1dd","#f87669"),n_cutoff = 3){
  if(ncol(exprSet_hub) != nrow(meta))stop("your exprSet_hub is not corresponds to meta")
  if(length(riskscore) != nrow(meta))stop("riskscore is not corresponds to meta")
  if (nrow(exprSet_hub)>30) {
    warning("seems too many of genes in heatmap")
  }
  if (nrow(exprSet_hub)>100) {
    stop("too many of genes in heatmap")
  }
  meta$riskscorefp = riskscore
  if(cut.point){
    cut = surv_cutpoint(
      meta,
      time = "time",
      event = "event",
      variables = "riskscorefp")[["cutpoint"]][1,1]
  }else{
    cut = stats::median(riskscore)
  }
  meta$Risk = ifelse(riskscore>cut,'high','low')
  meta$Risk = factor(meta$Risk,levels = c("low","high"))
  fp_dat=data.frame(patientid=1:length(riskscore),
                    riskscore=as.numeric(sort(riskscore)),
                    ri = meta$Risk[order(riskscore)])
  sur_dat=data.frame(patientid=1:length(riskscore),
                     time=meta[order(riskscore),'time'] ,
                     event=meta[order(riskscore),'event'])
  sur_dat$event=ifelse(sur_dat$event==0,'alive','death')

  # exp

  exp_dat = scale(t(exprSet_hub[,order(riskscore)]))
  exp_dat[exp_dat > n_cutoff] = n_cutoff
  exp_dat[exp_dat < -n_cutoff] = -n_cutoff
  # first plot
  p1=ggplot(fp_dat,aes(x=patientid,y=riskscore,color = ri))+
    geom_point()+
    scale_color_manual(values = color)+
    geom_vline(xintercept = sum(riskscore<cut),lty = 2)+
    scale_x_continuous(expand=c(0,0))+
    theme_bw()+
    theme(legend.position = "none",
          axis.line = element_line(color='black'),
          plot.background = element_blank(),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank())+
    labs(x = "",y = "Riskscore")
  p1
  # second plot
  p2=ggplot(sur_dat,aes(x=patientid,y=time))+
    geom_point(aes(col=event))+
    geom_vline(xintercept = sum(riskscore<cut),lty = 2)+
    scale_color_manual(values = color)+
    scale_x_continuous(expand=c(0,0))+
    scale_y_continuous(expand=c(0,0))+
    theme_bw()+
    labs(color = "Event",y = "Time",x = "")+
    theme(axis.line = element_line(color='black'),
          plot.background = element_blank(),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank())
  p2
  # third plot
  p3 = ggheat(exp_dat,
              meta$Risk[order(riskscore)],
              show_rownames = F,
              color = c(color[[1]],"white",color[[2]]),
              legend_color = color,groupname = "Risk",expname = "Expression")

  n = ifelse(nrow(exprSet_hub)<15,3,
             ifelse(nrow(exprSet_hub)<20,4,5))
  wrap_plots(p1,p2,p3,heights = c(1,1,n))
}
utils::globalVariables(c("patientid","genekitr"))

##' plcortest
##'
##' make cor.test for given lncRNA and mRNA
##' @param lnc_exp lncRNA expression set
##' @param mRNA_exp mRNA expression set which nrow equal to lncRNA_exp
##' @param cor_cutoff cor estimate cut_off, default 0
##' @export
##' @return a list with cor.test result,names are lncRNAs, element are mRNAs
##' @author Xiaojie Sun
##' @examples
##' # to update
##' @seealso
##' \code{\link{hypertest}}

plcortest <- function(lnc_exp, mRNA_exp,cor_cutoff=0) {
  jp = list()
  for(i in 1:nrow(lnc_exp)){
    x = cbind(lnc_exp[i,],t(mRNA_exp))
    jp[[i]] = vector()
    for(j in 2: ncol(x)){
      k = stats::cor.test(x[,1],x[,j])$p.value <0.05
      k2 = stats::cor.test(x[,1],x[,j])$estimate >cor_cutoff
      if(k&k2){jp[[i]] = c(jp[[i]],colnames(x)[[j]])}
    }
  }
  names(jp) = rownames(lnc_exp)
  return(jp)
}


##' hypertest
##'
##' make hypertest for given lncRNA and mRNA common miRNAs
##' @param lnc lncRNA names
##' @param pc mRNA names
##' @param deMIR miRNA names , default NULL
##' @param lnctarget a data.frame with two column,lncRNA in the first column ,miRNA in the second column
##' @param pctarget a data.frame with two column,mRNA in the first column ,miRNA in the second column
##' @export
##' @return a data.frame with hypertest result
##' @author Xiaojie Sun
##' @examples
##' # to update
##' @seealso
##' \code{\link{plcortest}}

hypertest = function(lnc,pc,deMIR = NULL,lnctarget,pctarget){
  pcs = unique(pctarget[,1])
  pctarget = lapply(unique(pctarget[,1]), function(x){
    pctarget[,2][pctarget[,1]==x]
  })
  names(pctarget) = unique(pcs)

  lncs = unique(lnctarget[,1])
  lnctarget = lapply(unique(lnctarget[,1]), function(x){
    lnctarget[,2][lnctarget[,1]==x]
  })
  names(lnctarget) = lncs

  mir1 <- unique(unlist(lnctarget))
  mir2 <- unique(unlist(pctarget))

  mirs <- union(mir1,mir2) # union set
  popTotal <- length(mirs) # size of the union set

  ceLNC <- lnc[lnc %in% names(lnctarget)] # lncRNAs with miRNA targets
  cePC <- pc[pc %in% names(pctarget)] # protein-coding genes with miRNA targets
  #ceMIR <- mir[mir %in% mirs]

  hyperOutput <- list()
  i <- 0
  for (lncID in ceLNC) {
    listTotal <- length(lnctarget[[lncID]]) # number of miRNAs linked to this lncRNA
    for (gene in cePC) {
      i = i + 1
      ovlp <- intersect(lnctarget[[lncID]], pctarget[[gene]]) # overlap

      popHits <- length(pctarget[[gene]])
      Counts <- length(ovlp)

      ovlpMIRs <- paste(ovlp, collapse = ',')
      foldEnrichment <- Counts/listTotal*popTotal/popHits
      pValue <- stats::phyper(Counts-1, popHits, popTotal-popHits,
                       listTotal, lower.tail=FALSE, log.p=FALSE)

      ceMIR <- Reduce(intersect, list(ovlp, deMIR))
      deMIRs <- paste(ceMIR, collapse = ',')
      deMIRCounts <- length(ceMIR)

      hyperOutput[[i]] <- c(lncID, gene, Counts, listTotal,
                            popHits,popTotal,foldEnrichment,pValue,ovlpMIRs,
                            deMIRCounts, deMIRs)

    }
  }

  #hyperOutput <- Reduce(rbind, hyperOutput)  ## slower
  hyperOutput <- do.call(rbind, hyperOutput)
  #hyperOutput <- rbind_list(hyperOutput) ## not test

  colnames(hyperOutput) <- c('lncRNAs','Genes','Counts','listTotal',
                             'popHits','popTotal','foldEnrichment','hyperPValue','miRNAs',
                             'deMIRCounts','deMIRs')
  hyperOutput <- as.data.frame(as.matrix(hyperOutput),
                               stringsAsFactors=FALSE)
  hyperOutput <- hyperOutput[as.numeric(hyperOutput$Counts)>0,]

  #hyperOutput$FDR <- p.adjust(as.numeric(as.character(hyperOutput$pValue)),
  #method = 'fdr')
  #hyperOutput <- hyperOutput[hyperOutput$Counts>0,]
  #hyperOutput$lncRNAs <- ensembl2symbolFun(hyperOutput$lncRNAs)
  #hyperOutput$gene <- ensembl2symbolFun(hyperOutput$gene)

  if (is.null(deMIR)) {
    hyperOutput <- hyperOutput[,! colnames(hyperOutput) %in%
                                 c('deMIRCounts','deMIRs')]
  }
  hyperOutput = hyperOutput[order(hyperOutput$hyperPValue),]
  return (hyperOutput)
  message(paste0(sum(hyperOutput$hyperPValue<0.05)," pairs p<0.05"))
}

##' quick_enrich
##'
##' Perform enrichment analysis for a gene list; human is the default species.
##'
##' @param genes a gene symbol or Entrez ID vector
##' @param kkgo_file optional RDS cache filename for KEGG and GO results;
##'   when `NULL`, the function recomputes on every call and does not write a cache
##' @param destdir destination directory used to save the cache file when
##'   `kkgo_file` is provided
##' @inheritParams trans_ensembl_exp
##' @return enrichment results and dotplots
##' @author Xiaojie Sun
##' @importFrom ggplot2 facet_grid
##' @export
##' @examples
##' \dontrun{
##' head(genes)
##' g = quick_enrich(genes,destdir = tempdir())
##' names(g)
##' g$kk
##' g$go
##' }
##' @seealso
##' \code{\link{double_enrich}}

quick_enrich <- function(genes,
                         kkgo_file = NULL,
                         destdir = getwd(),
                         species = "human"){
  if(any(is.na(suppressWarnings(as.numeric(genes))))){
    org = .tinyarray_orgdb_for_species(species)
    .tinyarray_require_clusterProfiler()
    s2e <- clusterProfiler::bitr(genes, fromType = "SYMBOL",
                toType = "ENTREZID",
                OrgDb = org$orgdb)
    s2e <- .tinyarray_dedup_by_key(s2e, "SYMBOL")
    genes = s2e$ENTREZID
  }
  compute_quick_enrich <- function(genes, species) {
    org = .tinyarray_orgdb_for_species(species)
    orgdb = org$orgdb
    .tinyarray_require_clusterProfiler()
    kk <- tryCatch(
      clusterProfiler::enrichKEGG(
        gene = genes,
        organism = org$kegg,
        pvalueCutoff = 0.05
      ),
      error = function(e) {
        message("KEGG enrichment failed: ", conditionMessage(e))
        NULL
      }
    )
    if (!is.null(kk)) {
      kk <- .tinyarray_make_readable_enrich(kk, OrgDb = orgdb, keyType = "ENTREZID")
      if (nrow(kk@result) == 0L) {
        message("No KEGG pathway enriched")
        kk <- NULL
      }
    } else {
      message("No KEGG pathway enriched")
    }
    go <- clusterProfiler::enrichGO(genes,
                                    OrgDb = orgdb,
                                    ont="all",
                                    readable = FALSE)
    go <- .tinyarray_make_readable_enrich(go, OrgDb = orgdb, keyType = "ENTREZID")
    if (!is.null(go) && nrow(go@result) == 0L) {
      message("No GO terms enriched")
      go <- NULL
    }
    list(kk = kk, go = go)
  }
  if (is.null(kkgo_file) || !length(kkgo_file) || !nzchar(kkgo_file[1L])) {
    obj = compute_quick_enrich(genes, species)
  } else {
    f = file.path(destdir,kkgo_file)
    legacy_f = if (grepl("\\.rds$", f, ignore.case = TRUE)) sub("\\.rds$", ".Rdata", f, ignore.case = TRUE) else f
    if(file.exists(f)){
      obj = .tinyarray_read_cache(f)
    }else if(legacy_f != f && file.exists(legacy_f)){
      obj = .tinyarray_read_cache(legacy_f)
      saveRDS(obj, file = f)
    }else{
      obj = compute_quick_enrich(genes, species)
      saveRDS(obj, file = f)
    }
  }
  kk = obj$kk
  go = obj$go
  k1 = if (is.null(kk)) 0L else sum(kk@result$p.adjust <0.05, na.rm = TRUE)
  k2 = if (is.null(go)) 0L else sum(go@result$p.adjust <0.05, na.rm = TRUE)
  if (k1 == 0L) {
    kk.dot = "no pathway enriched"
  } else{
    kk.dot = clusterProfiler::dotplot(kk)
  }

  if (k2 == 0L) {
    go.dot = "no terms enriched"
  } else{
    go.dot = clusterProfiler::dotplot(go, split="ONTOLOGY",font.size =10,showCategory = 5)+
      facet_grid(ONTOLOGY~., scales="free")
  }
  result = list(kk = kk,go = go,kk.dot = kk.dot,go.dot = go.dot)
  return(result)
}

##' draw enrichment bar plots for both up and down genes
##'
##' draw enrichment bar plots for both up and down genes,for human only.
##'
##' @param deg a data.frame contains at least two columns:"ENTREZID" and "change"
##' @param n how many terms will you perform for up and down genes respectively
##' @param color color for bar plot
##' @inheritParams quick_enrich
##' @return a list with kegg and go bar plot according to up and down genes enrichment result.
##' @author Xiaojie Sun
##' @importFrom stringr str_to_lower
##' @importFrom stringr str_wrap
##' @importFrom dplyr mutate
##' @importFrom dplyr arrange
##' @importFrom ggplot2 ggplot
##' @importFrom ggplot2 geom_bar
##' @importFrom ggplot2 coord_flip
##' @importFrom ggplot2 theme_light
##' @importFrom ggplot2 ylim
##' @importFrom ggplot2 scale_x_discrete
##' @importFrom ggplot2 scale_y_continuous
##' @importFrom ggplot2 theme
##' @export
##' @examples
##' \dontrun{
##' deg = data.frame(
##'   ENTREZID = genes[1:20],
##'   change = rep(c("up", "down"), 10)
##' )
##' double_enrich(deg)
##' }
##' @seealso
##' \code{\link{quick_enrich}}

double_enrich <- function(deg,n = 10,color = c("#2874C5", "#f87669"),species = "human"){
  deg$change = tolower(deg$change)
  up = quick_enrich(deg$ENTREZID[deg$change=="up"],"up.rds",destdir = tempdir(),species = species)
  down = quick_enrich(deg$ENTREZID[deg$change=="down"],"down.rds",destdir = tempdir(),species = species)
  ud_enrich = function(df){
    if (is.null(df) || !nrow(df)) {
      return(NULL)
    }
    padj = pmax(as.numeric(df$p.adjust), .Machine$double.xmin)
    df$pl = ifelse(df$change == "up",-log10(padj),log10(padj))
    df = df[order(df$change, df$pl), , drop = FALSE]
    df$Description = factor(df$Description,levels = unique(df$Description),ordered = TRUE)
    lm = if (any(is.finite(df$pl))) {
      c(floor(min(df$pl, na.rm = TRUE)), ceiling(max(df$pl, na.rm = TRUE)))
    } else {
      c(-1, 1)
    }
    ggplot(df, aes(x=Description, y= pl)) +
      geom_bar(stat='identity', aes(fill=change), width=.7)+
      scale_fill_manual(values = color)+
      coord_flip()+
      theme_light() +
      ylim(lm)+
      scale_x_discrete(labels=function(x) vapply(strwrap(x, width = 30), paste, character(1), collapse = "\n"))+
      theme(
        panel.border = element_blank()
      )
  }
  kk_ok = !is.null(up$kk) & !is.null(down$kk)
  go_ok = !is.null(up$go) & !is.null(down$go)
  if (kk_ok || go_ok) {
    result = list()
    take_top_terms = function(x) {
      if (is.null(x)) {
        return(NULL)
      }
      if (!nrow(x)) {
        return(x[0, , drop = FALSE])
      }
      x[seq_len(min(n, nrow(x))), , drop = FALSE]
    }
    if (kk_ok) {
      up$kk@result$change = "up"
      down$kk@result$change = "down"
      kk = rbind(take_top_terms(up$kk@result), take_top_terms(down$kk@result))
      result$kp = tryCatch(
        ud_enrich(kk),
        error = function(e) {
          warning("KEGG plot failed: ", conditionMessage(e))
          "KEGG plot unavailable"
        }
      )
    } else {
      warning("No KEGG pathway enriched; showing GO result only")
    }
    if (go_ok) {
      up$go@result$change = "up"
      down$go@result$change = "down"
      go = rbind(take_top_terms(up$go@result), take_top_terms(down$go@result))
      result$gp = tryCatch(
        ud_enrich(go),
        error = function(e) {
          warning("GO plot failed: ", conditionMessage(e))
          "GO plot unavailable"
        }
      )
    } else {
      warning("No GO term enriched; showing KEGG result only")
    }
  } else {
    warning("No KEGG or GO pathway was enriched; returning results from quick_enrich")
    result = list(up = up,
                  down = down)
  }
  class(result) <- c("tinyarray_double_enrich", class(result))
  return(result)

}

##' @export
print.tinyarray_double_enrich <- function(x, ...) {
  cat("<tinyarray_double_enrich>\n")
  if (length(x)) {
    cat("components:", paste(names(x), collapse = ", "), "\n")
  }
  invisible(x)
}

utils::globalVariables(c("change","pl","Description"))

.tinyarray_geo_stopwords <- function() {
  c(
    "sample", "samples", "patient", "patients", "subject", "subjects",
    "series", "array", "chip", "microarray", "expression", "profiling",
    "profile", "profiles", "data", "study", "studies", "gene", "genes",
    "rna", "mrna", "dna", "gse", "gsm", "gpl", "geo", "platform",
    "human", "mouse", "rat", "male", "female"
  )
}

.tinyarray_geo_control_patterns <- function() {
  c(
    "\\bcontrol\\b",
    "\\bctrl\\b",
    "\\bnormal\\b",
    "\\bhealthy\\b",
    "\\buntreat(?:ed)?\\b",
    "\\bvehicle\\b",
    "\\bsham\\b",
    "\\bmock\\b",
    "\\bbaseline\\b",
    "\\bwild\\s*type\\b",
    "\\bwt\\b",
    "\\badjacent\\s+normal\\b",
    "\\bnon[-_\\s]*tumou?r\\b",
    "\\bnon[-_\\s]*cancer\\b",
    "\\bnoncancer\\b",
    "\\bnon[-_\\s]*obese\\b",
    "\\bnonobese\\b",
    "\\bbenign\\b"
  )
}

.tinyarray_geo_is_control_level <- function(x) {
  x <- .tinyarray_geo_clean_text(x)
  x <- tolower(x)
  patterns <- .tinyarray_geo_control_patterns()
  vapply(x, function(value) {
    if (is.na(value) || !nzchar(value)) {
      return(FALSE)
    }
    any(vapply(patterns, function(pattern) {
      grepl(pattern, value, ignore.case = TRUE, perl = TRUE)
    }, logical(1)))
  }, logical(1))
}

.tinyarray_geo_resolve_pdata <- function(x) {
  if (is.data.frame(x)) {
    return(x)
  }
  if (is.list(x) && !is.null(x$pd) && is.data.frame(x$pd)) {
    return(x$pd)
  }
  stop("x must be a data.frame or a list with a data.frame element `pd`.", call. = FALSE)
}

.tinyarray_geo_clean_text <- function(x) {
  x <- as.character(x)
  x <- trimws(x)
  x[x == ""] <- NA_character_
  x <- sub("\\s*[,;].*$", "", x, perl = TRUE)
  x <- gsub("\\s*\\[[^\\]]*\\]\\s*$", "", x, perl = TRUE)
  x <- gsub("\\s*\\([^\\)]*\\)\\s*$", "", x, perl = TRUE)
  x <- gsub("([a-z0-9])([A-Z])", "\\1 \\2", x, perl = TRUE)
  x <- gsub("\\b(gse|gsm|gpl)\\d+\\b", " ", x, ignore.case = TRUE, perl = TRUE)
  x <- gsub("[^[:alnum:]\\- ]+", " ", x, perl = TRUE)
  x <- gsub("\\s+[A-Za-z]*\\d+[A-Za-z]*\\s*$", "", x, perl = TRUE)
  x <- gsub("\\s+", " ", x, perl = TRUE)
  x <- trimws(x)
  x[x == ""] <- NA_character_
  x
}

.tinyarray_geo_label_display <- function(x) {
  x <- as.character(x)
  x <- trimws(x)
  x[x == ""] <- NA_character_
  x <- gsub("\\s+", "_", x, perl = TRUE)
  x
}

.tinyarray_geo_is_numeric_levels <- function(levels) {
  levels <- as.character(levels)
  levels <- levels[!is.na(levels) & nzchar(levels)]
  if (!length(levels)) {
    return(FALSE)
  }
  numeric_levels <- suppressWarnings(as.numeric(levels))
  all(!is.na(numeric_levels))
}

.tinyarray_geo_escape_regex <- function(x) {
  gsub("([][{}()+*^$.|\\\\?])", "\\\\\\1", x, perl = TRUE)
}

.tinyarray_geo_rough_tokens <- function(x) {
  x <- .tinyarray_geo_clean_text(x)
  x <- gsub("[_-]+", " ", x, perl = TRUE)
  split <- strsplit(x, " ", fixed = TRUE)
  lapply(split, function(tokens) {
    tokens <- tokens[nzchar(tokens)]
    tokens
  })
}

.tinyarray_geo_common_affixes <- function(x) {
  token_list <- .tinyarray_geo_rough_tokens(unique(x))
  token_list <- token_list[lengths(token_list) > 0L]
  if (length(token_list) < 2L) {
    return(list(prefix = character(), suffix = character()))
  }

  prefix <- token_list[[1L]]
  for (i in 2:length(token_list)) {
    current <- token_list[[i]]
    n <- min(length(prefix), length(current))
    if (!n) {
      prefix <- character()
      break
    }
    idx <- which(tolower(prefix[seq_len(n)]) != tolower(current[seq_len(n)]))
    if (length(idx)) {
      prefix <- prefix[seq_len(idx[[1L]] - 1L)]
    } else {
      prefix <- prefix[seq_len(n)]
    }
    if (!length(prefix)) break
  }

  suffix <- token_list[[1L]]
  for (i in 2:length(token_list)) {
    current <- token_list[[i]]
    n <- min(length(suffix), length(current))
    if (!n) {
      suffix <- character()
      break
    }
    suffix1 <- rev(suffix)
    current1 <- rev(current)
    idx <- which(tolower(suffix1[seq_len(n)]) != tolower(current1[seq_len(n)]))
    if (length(idx)) {
      suffix <- rev(suffix1[seq_len(idx[[1L]] - 1L)])
    } else {
      suffix <- rev(suffix1[seq_len(n)])
    }
    if (!length(suffix)) break
  }

  list(prefix = prefix, suffix = suffix)
}

.tinyarray_geo_strip_affixes <- function(x, prefix = character(), suffix = character()) {
  x <- .tinyarray_geo_clean_text(x)
  if (!length(x)) {
    return(x)
  }

  strip_prefix <- function(value, tokens) {
    if (!length(tokens) || is.na(value) || !nzchar(value)) {
      return(value)
    }
    if (length(tokens) == 1L && nchar(tokens[[1L]]) < 4L) {
      return(value)
    }
    pattern <- paste0(
      "^",
      paste(vapply(tokens, .tinyarray_geo_escape_regex, character(1)), collapse = "(?:[-_\\s]+)"),
      "(?:[-_\\s]+)?"
    )
    sub(pattern, "", value, perl = TRUE)
  }

  strip_suffix <- function(value, tokens) {
    if (!length(tokens) || is.na(value) || !nzchar(value)) {
      return(value)
    }
    if (length(tokens) == 1L && nchar(tokens[[1L]]) < 4L) {
      return(value)
    }
    pattern <- paste0(
      "(?:[-_\\s]+)?",
      paste(vapply(tokens, .tinyarray_geo_escape_regex, character(1)), collapse = "(?:[-_\\s]+)"),
      "$"
    )
    sub(pattern, "", value, perl = TRUE)
  }

  out <- vapply(x, function(value) {
    if (is.na(value) || !nzchar(value)) {
      return(NA_character_)
    }
    value <- strip_prefix(value, prefix)
    value <- strip_suffix(value, suffix)
    value <- gsub("^[-_\\s]+|[-_\\s]+$", "", value, perl = TRUE)
    value <- gsub("\\s+", " ", value, perl = TRUE)
    value <- trimws(value)
    if (!nzchar(value)) {
      NA_character_
    } else {
      value
    }
  }, character(1), USE.NAMES = FALSE)
  out
}

.tinyarray_geo_alt_labels <- function(label) {
  label <- as.character(label)[1L]
  if (!length(label) || is.na(label) || !nzchar(label)) {
    return(character())
  }
  tokens <- strsplit(label, " ", fixed = TRUE)[[1L]]
  tokens <- tokens[nzchar(tokens)]
  if (length(tokens) != 1L) {
    return(character())
  }
  if (!grepl("-", tokens, fixed = TRUE)) {
    return(character())
  }
  if (grepl("^non[-_]", tolower(tokens))) {
    return(character())
  }
  pieces <- strsplit(tokens, "-", fixed = TRUE)[[1L]]
  pieces <- pieces[nzchar(pieces)]
  pieces <- pieces[grepl("[[:alpha:]]", pieces)]
  if (length(pieces) < 2L) {
    return(character())
  }
  unique(pieces)
}

.tinyarray_geo_trim_shared_tokens <- function(label, reference) {
  label_tokens <- strsplit(as.character(label), " ", fixed = TRUE)[[1L]]
  ref_tokens <- strsplit(as.character(reference), " ", fixed = TRUE)[[1L]]
  label_tokens <- label_tokens[nzchar(label_tokens)]
  ref_tokens <- ref_tokens[nzchar(ref_tokens)]
  if (!length(label_tokens) || !length(ref_tokens)) {
    return(list(label = as.character(label), reference = as.character(reference)))
  }
  while (
    length(label_tokens) > 1L &&
      length(ref_tokens) > 1L &&
      identical(tolower(label_tokens[1L]), tolower(ref_tokens[1L]))
  ) {
    label_tokens <- label_tokens[-1L]
    ref_tokens <- ref_tokens[-1L]
  }
  while (
    length(label_tokens) > 1L &&
      length(ref_tokens) > 1L &&
      identical(tolower(utils::tail(label_tokens, 1L)), tolower(utils::tail(ref_tokens, 1L)))
  ) {
    label_tokens <- utils::head(label_tokens, -1L)
    ref_tokens <- utils::head(ref_tokens, -1L)
  }
  label2 <- paste(label_tokens, collapse = " ")
  ref2 <- paste(ref_tokens, collapse = " ")
  if (!nzchar(label2) || !nzchar(ref2)) {
    return(list(label = as.character(label), reference = as.character(reference)))
  }
  list(label = label2, reference = ref2)
}

.tinyarray_geo_tokenize <- function(x, stopwords = .tinyarray_geo_stopwords()) {
  cleaned <- .tinyarray_geo_clean_text(x)
  token_list <- strsplit(cleaned, " ", fixed = TRUE)
  stopwords_lower <- tolower(stopwords)
  lapply(token_list, function(tokens) {
    tokens <- tokens[nzchar(tokens)]
    tokens <- tokens[grepl("[[:alpha:]]", tokens)]
    tokens <- tokens[!tolower(tokens) %in% stopwords_lower]
    unique(tokens)
  })
}

.tinyarray_geo_candidate_labels <- function(tokens) {
  tokens <- tokens[nzchar(tokens)]
  if (!length(tokens)) {
    return(character())
  }
  unique(tokens)
}

.tinyarray_geo_reference_label <- function(phrases, labels) {
  if (!length(labels)) {
    return(NULL)
  }
  control_labels <- labels[.tinyarray_geo_is_control_level(labels)]
  if (length(control_labels)) {
    control_metrics <- vapply(control_labels, function(label) {
      sum(!is.na(phrases) & phrases == label)
    }, integer(1))
    control_labels <- control_labels[order(-control_metrics, nchar(control_labels), control_labels)]
    return(control_labels[[1L]])
  }
  metrics <- t(vapply(labels, function(label) {
    present <- !is.na(phrases) & phrases == label
    c(
      avg_tokens = if (any(present)) mean(lengths(strsplit(phrases[present], " ", fixed = TRUE))) else Inf,
      support = sum(present)
    )
  }, numeric(2)))
  metrics <- as.data.frame(metrics, stringsAsFactors = FALSE)
  metrics$label <- rownames(metrics)
  metrics <- metrics[order(metrics$avg_tokens, -metrics$support, nchar(metrics$label), metrics$label), , drop = FALSE]
  if (!nrow(metrics) || !is.finite(metrics$avg_tokens[1L])) {
    return(NULL)
  }
  metrics$label[[1L]]
}

.tinyarray_geo_build_candidate <- function(phrases,
                                           keep_idx,
                                           present,
                                           label_raw,
                                           ref_raw,
                                           colname,
                                           n_total,
                                           level_order = NULL,
                                           ref = NULL,
                                           method = c("phrase", "token"),
                                           seen = NULL) {
  method <- match.arg(method)
  if (sum(present) < 2L || sum(!present) < 2L) {
    return(NULL)
  }
  trimmed <- .tinyarray_geo_trim_shared_tokens(label_raw, ref_raw)
  display_label <- .tinyarray_geo_label_display(trimmed$label)
  display_ref <- .tinyarray_geo_label_display(trimmed$reference)
  if (!nzchar(display_label) || !nzchar(display_ref) || identical(display_label, display_ref)) {
    return(NULL)
  }
  if (length(keep_idx) != length(present)) {
    return(NULL)
  }
  group <- rep(NA_character_, n_total)
  group[keep_idx] <- ifelse(present, display_label, display_ref)
  group <- factor(group, levels = c(display_label, display_ref))
  group <- .tinyarray_geo_order_levels(group, level_order = level_order, ref = ref)
  group <- droplevels(group)
  if (length(unique(stats::na.omit(group))) < 2L) {
    return(NULL)
  }
  if (nlevels(group) > 1L && sum(!is.na(group)) < 2L) {
    return(NULL)
  }
  if (nlevels(group) > 5L && .tinyarray_geo_is_numeric_levels(base::levels(group))) {
    return(NULL)
  }
  sig <- paste(ifelse(is.na(group), "<NA>", as.character(group)), collapse = "\r")
  if (!is.null(seen) && length(seen) && sig %in% seen) {
    return(NULL)
  }
  counts2 <- sort(table(group), decreasing = TRUE)
  list(
    column = colname,
    label = display_label,
    reference = display_ref,
    label_raw = label_raw,
    reference_raw = ref_raw,
    candidate = paste(base::levels(group), collapse = "-"),
    group = group,
    counts = counts2,
    coverage = mean(!is.na(group)),
    n_groups = nlevels(group),
    levels = base::levels(group),
    support = sum(present),
    reference_support = sum(!present),
    label_words = length(strsplit(display_label, "_", fixed = TRUE)[[1L]]),
    label_table = counts2,
    method = method,
    signature = sig
  )
}

.tinyarray_geo_order_levels <- function(group, level_order = NULL, ref = NULL) {
  if (!is.factor(group)) {
    group <- factor(group, levels = unique(group))
  }
  lev <- base::levels(group)
  if (length(lev) < 2L) {
    return(group)
  }

  resolve_level <- function(value) {
    if (is.null(value) || !length(value)) {
      return(NA_character_)
    }
    value <- as.character(value)[1L]
    if (value %in% lev) {
      return(value)
    }
    hit <- lev[tolower(lev) == tolower(value)]
    if (length(hit) == 1L) {
      return(hit)
    }
    NA_character_
  }

  ord <- lev
  if (!is.null(level_order)) {
    wanted <- vapply(as.character(level_order), resolve_level, character(1), USE.NAMES = FALSE)
    wanted <- wanted[!is.na(wanted) & nzchar(wanted)]
    ord <- c(wanted, setdiff(lev, wanted))
  }
  if (!is.null(ref)) {
    ref_level <- resolve_level(ref)
    if (!is.null(ref_level) && !is.na(ref_level) && nzchar(ref_level)) {
      ord <- c(ref_level, setdiff(ord, ref_level))
    }
  }
  if (is.null(level_order) && is.null(ref)) {
    control_levels <- lev[.tinyarray_geo_is_control_level(lev)]
    if (length(control_levels)) {
      ord <- c(control_levels, setdiff(ord, control_levels))
    }
  }
  ord <- unique(ord[nzchar(ord)])
  factor(group, levels = ord)
}

.tinyarray_geo_infer_from_column <- function(x,
                                             colname,
                                             min_group_size = 2L,
                                             max_groups = 20L,
                                             min_coverage = 0.8,
                                             stopwords = .tinyarray_geo_stopwords(),
                                             level_order = NULL,
                                             ref = NULL) {
  cleaned <- .tinyarray_geo_clean_text(x)
  keep <- !is.na(cleaned)
  if (sum(keep) < 2L) {
    return(NULL)
  }

  cleaned <- cleaned[keep]
  keep_idx <- which(keep)
  affixes <- .tinyarray_geo_common_affixes(cleaned)
  stripped <- .tinyarray_geo_strip_affixes(
    cleaned,
    prefix = affixes$prefix,
    suffix = affixes$suffix
  )
  keep2 <- !is.na(stripped) & nzchar(stripped)
  if (sum(keep2) < 2L) {
    return(NULL)
  }

  stripped <- stripped[keep2]
  keep_idx <- keep_idx[keep2]
  total_n <- length(x)
  # Merge labels that differ only in case while retaining the most common
  # spelling. This prevents values such as `c`/`C` from becoming separate
  # groups because of inconsistent metadata capitalization.
  case_key <- tolower(stripped)
  for (key in unique(case_key)) {
    idx <- !is.na(case_key) & case_key == key
    spellings <- sort(table(stripped[idx]), decreasing = TRUE)
    stripped[idx] <- names(spellings)[1L]
  }

  display <- .tinyarray_geo_label_display(stripped)
  label_counts <- table(display)
  valid_levels <- names(label_counts)[label_counts >= min_group_size]
  assigned <- !is.na(display) & display %in% valid_levels
  coverage <- sum(assigned) / total_n
  if (coverage < min_coverage) {
    return(NULL)
  }
  if (length(valid_levels) < 2L || length(valid_levels) > max_groups) {
    return(NULL)
  }

  group <- rep(NA_character_, total_n)
  group[keep_idx[assigned]] <- display[assigned]
  group <- factor(group, levels = unique(display[assigned]))
  group <- .tinyarray_geo_order_levels(group, level_order = level_order, ref = ref)
  group <- droplevels(group)
  if (nlevels(group) > 5L && .tinyarray_geo_is_numeric_levels(base::levels(group))) {
    return(NULL)
  }

  make_candidate <- function(group, method) {
    lev <- base::levels(group)
    counts2 <- table(group)
    reference <- lev[1L]
    labels <- setdiff(lev, reference)
    list(
      column = colname,
      label = paste(labels, collapse = "|"),
      reference = reference,
      label_raw = paste(labels, collapse = "|"),
      reference_raw = reference,
      candidate = paste(lev, collapse = "-"),
      group = group,
      counts = sort(counts2, decreasing = TRUE),
      coverage = mean(!is.na(group)),
      n_groups = nlevels(group),
      levels = lev,
      support = as.integer(min(counts2)),
      reference_support = as.integer(counts2[[reference]]),
      label_words = max(lengths(strsplit(labels, "_", fixed = TRUE))),
      label_table = sort(counts2, decreasing = TRUE),
      method = method,
      signature = paste(ifelse(is.na(group), "<NA>", as.character(group)), collapse = "\r")
    )
  }

  candidates <- list(make_candidate(
    group,
    if (nlevels(group) == 2L) "groups" else "multigroup"
  ))

  # For a genuinely binary column, keep harmless alternative labels obtained
  # from a hyphenated group name. Membership is unchanged; only the display
  # label differs. This preserves useful choices such as Disease/Normal versus
  # Ovary-Disease/Normal without collapsing any third group into the reference.
  if (nlevels(group) == 2L) {
    ref_level <- base::levels(group)[1L]
    target_level <- base::levels(group)[2L]
    alt_labels <- .tinyarray_geo_alt_labels(gsub("_", " ", target_level, fixed = TRUE))
    for (alt in alt_labels) {
      alt <- .tinyarray_geo_label_display(alt)
      alt_values <- ifelse(
        is.na(group),
        NA_character_,
        ifelse(as.character(group) == target_level, alt, as.character(group))
      )
      alt_group <- factor(alt_values, levels = c(ref_level, alt))
      alt_group <- .tinyarray_geo_order_levels(alt_group, level_order = level_order, ref = ref)
      candidates[[length(candidates) + 1L]] <- make_candidate(alt_group, "alt_hyphen")
    }
  }

  order_idx <- order(
    vapply(candidates, `[[`, integer(1), "label_words"),
    vapply(candidates, `[[`, character(1), "candidate"),
    vapply(candidates, `[[`, character(1), "method")
  )
  candidates[order_idx]
}

.tinyarray_geo_candidate_table <- function(candidates) {
  if (!length(candidates)) {
    return(data.frame())
  }
  candidate_name <- character(length(candidates))
  columns <- vapply(candidates, `[[`, character(1), "column")
  for (col in unique(columns)) {
    idx <- which(columns == col)
    if (length(idx) == 1L) {
      candidate_name[idx] <- paste0(col, "_choice")
    } else {
      candidate_name[idx] <- paste0(col, "_choice", seq_along(idx))
    }
  }
  data.frame(
    idx = seq_along(candidates),
    candidate_name = candidate_name,
    column = vapply(candidates, `[[`, character(1), "column"),
    candidate = vapply(candidates, `[[`, character(1), "candidate"),
    label = vapply(candidates, `[[`, character(1), "label"),
    reference = vapply(candidates, `[[`, character(1), "reference"),
    method = vapply(candidates, `[[`, character(1), "method"),
    n_groups = vapply(candidates, `[[`, integer(1), "n_groups"),
    coverage = round(vapply(candidates, `[[`, numeric(1), "coverage"), 3),
    support = vapply(candidates, `[[`, integer(1), "support"),
    label_words = vapply(candidates, `[[`, integer(1), "label_words"),
    groups = vapply(candidates, function(x) paste(x$levels, collapse = " | "), character(1)),
    counts = vapply(
      candidates,
      function(x) paste(names(x$counts), as.integer(x$counts), sep = ":", collapse = ", "),
      character(1)
    ),
    stringsAsFactors = FALSE
  )
}

.tinyarray_geo_collect_candidates <- function(pd,
                                              group_col = "title",
                                              min_group_size = 2L,
                                              max_groups = 20L,
                                              min_coverage = 0.8,
                                              stopwords = .tinyarray_geo_stopwords(),
                                              levels = NULL,
                                              ref = NULL,
                                              search_other_cols = TRUE) {
  pd <- .tinyarray_geo_resolve_pdata(pd)
  preferred_cols <- unique(c(
    group_col,
    if (tolower(group_col) != "title" && "title" %in% names(pd)) "title" else character()
  ))
  preferred_cols <- preferred_cols[preferred_cols %in% names(pd)]
  cols <- if (isTRUE(search_other_cols)) {
    unique(c(preferred_cols, setdiff(names(pd), preferred_cols)))
  } else {
    preferred_cols
  }
  candidates <- list()
  for (col in cols) {
    candidate <- .tinyarray_geo_infer_from_column(
      pd[[col]],
      colname = col,
      min_group_size = min_group_size,
      max_groups = max_groups,
      min_coverage = min_coverage,
      stopwords = stopwords,
      level_order = levels,
      ref = ref
    )
    if (!is.null(candidate)) {
      candidates[[col]] <- candidate
    }
  }
  candidates
}

.tinyarray_geo_flatten_candidates <- function(candidates) {
  if (!length(candidates)) {
    return(list())
  }
  out <- list()
  out_names <- character()
  for (col in names(candidates)) {
    inner <- candidates[[col]]
    if (!length(inner)) {
      next
    }
    for (candidate in inner) {
      out[[length(out) + 1L]] <- candidate
      out_names <- c(out_names, paste(col, candidate$candidate, sep = "_"))
    }
  }
  names(out) <- out_names
  out
}

.tinyarray_geo_deduplicate_candidates <- function(candidates) {
  if (!length(candidates)) {
    return(list())
  }
  sig <- vapply(
    candidates,
    function(x) {
      group <- x$group
      paste(ifelse(is.na(group), "<NA>", as.character(group)), collapse = "\r")
    },
    character(1)
  )
  candidates[!duplicated(sig)]
}

.tinyarray_geo_group_candidates <- function(pd,
                                            group_col = "title",
                                            min_group_size = 2L,
                                            max_groups = 20L,
                                            min_coverage = 0.8,
                                            stopwords = .tinyarray_geo_stopwords(),
                                            levels = NULL,
                                            ref = NULL,
                                            search_other_cols = TRUE) {
  candidates <- .tinyarray_geo_collect_candidates(
    pd = pd,
    group_col = group_col,
    min_group_size = min_group_size,
    max_groups = max_groups,
    min_coverage = min_coverage,
    stopwords = stopwords,
    levels = levels,
    ref = ref,
    search_other_cols = search_other_cols
  )
  out <- list()
  flat <- .tinyarray_geo_flatten_candidates(candidates)
  flat <- .tinyarray_geo_deduplicate_candidates(flat)
  if (length(flat)) {
    tbl <- .tinyarray_geo_candidate_table(flat)
    out <- stats::setNames(lapply(flat, function(x) x$group), tbl$candidate_name)
  }
  out
}

#' auto_geo_group
#'
#' Infer sample groups from GEO phenotype metadata.
#'
#' The function first tries the user-specified `group_col`, then `title`. If no
#' stable grouping is found, it scans the remaining columns and returns a table
#' of candidate groupings for manual selection. Both binary and multigroup
#' factors are supported. Rare groups are left as `NA` rather than being
#' relabeled as another group.
#'
#' Labels that differ only in capitalization are merged, retaining their most
#' common spelling. Shared affixes and trailing sample identifiers are removed
#' when they can be identified consistently.
#'
#' @param x A GEO result list with a `pd` data frame, or a phenotype data frame.
#' @param group_col Column name to try first. The default is `title`.
#' @param choice Candidate index to select. Use `0` to cancel. If `NULL`, the
#'   function returns the candidate table without selecting.
#' @param min_group_size Minimum number of samples required for a split label.
#' @param max_groups Maximum number of group labels allowed in one column.
#' @param min_coverage Minimum fraction of samples that must be assigned a label.
#' @param search_other_cols Whether to search other columns when `group_col` and
#'   `title` fail.
#' @param stopwords Character vector of generic words to ignore during tokenization.
#' @param levels Optional character vector giving the desired factor level order.
#' @param ref Optional reference level to place first.
#' @param verbose Whether to print messages and candidate tables.
#'
#' @return A list with `group_list`, `source_col`, `selected_index`,
#'   `candidate_table`, `candidates`, and `pdata`.
#'
#' @export
#'
#' @examples
#' pd <- data.frame(
#'   title = c("Control sample", "Control sample", "Treat sample", "Treat sample"),
#'   stringsAsFactors = FALSE
#' )
#' rownames(pd) <- paste0("s", 1:4)
#' res <- auto_geo_group(
#'   pd,
#'   choice = 1,
#'   levels = c("Control", "Treat"),
#'   ref = "Control",
#'   verbose = FALSE
#' )
#' res$source_col
#' levels(res$group_list)
#' table(res$group_list)
auto_geo_group <- function(x,
                           group_col = "title",
                           choice = NULL,
                           min_group_size = 2L,
                           max_groups = 20L,
                           min_coverage = 0.8,
                           search_other_cols = TRUE,
                           stopwords = .tinyarray_geo_stopwords(),
                           levels = NULL,
                           ref = NULL,
                           verbose = TRUE) {
  pd <- .tinyarray_geo_resolve_pdata(x)
  if (!nrow(pd)) {
    stop("pdata has no rows.", call. = FALSE)
  }
  preferred_cols <- unique(c(
    group_col,
    if (tolower(group_col) != "title" && "title" %in% names(pd)) "title" else character()
  ))
  preferred_cols <- preferred_cols[preferred_cols %in% names(pd)]
  candidate_order <- if (isTRUE(search_other_cols)) {
    unique(c(preferred_cols, setdiff(names(pd), preferred_cols)))
  } else {
    preferred_cols
  }

  candidates_by_col <- .tinyarray_geo_collect_candidates(
    pd = pd,
    group_col = group_col,
    min_group_size = min_group_size,
    max_groups = max_groups,
    min_coverage = min_coverage,
    stopwords = stopwords,
    levels = levels,
    ref = ref,
    search_other_cols = search_other_cols
  )

  candidates <- .tinyarray_geo_flatten_candidates(candidates_by_col)
  candidates <- .tinyarray_geo_deduplicate_candidates(candidates)
  if (!length(candidates)) {
    if (verbose) {
      if (search_other_cols) {
        message(
          "No stable grouping was found in `", group_col, "`, `title`, or other columns. ",
          "If grouping information exists, extract it manually from the GPL table."
        )
      } else {
        message(
          "No stable grouping was found in `", paste(c(group_col, if (tolower(group_col) != "title" && "title" %in% names(pd)) "title" else character()), collapse = "`, `"), "`."
        )
      }
    }
    return(structure(
      list(
        group_list = NULL,
        source_col = NULL,
        source_candidate = NULL,
        selected_index = 0L,
        candidate_table = data.frame(),
        candidates = list(),
        pdata = pd
      ),
      class = "geo_auto_group"
    ))
  }

  cand_tbl <- .tinyarray_geo_candidate_table(candidates)
  cand_tbl$column_rank <- match(cand_tbl$column, candidate_order)
  cand_tbl$column_rank[is.na(cand_tbl$column_rank)] <- length(candidate_order) + 1L
  cand_tbl <- cand_tbl[order(cand_tbl$column_rank, -cand_tbl$coverage, -cand_tbl$support, cand_tbl$label_words, cand_tbl$column, cand_tbl$candidate), , drop = FALSE]
  if (nrow(cand_tbl)) {
    cand_tbl$idx <- seq_len(nrow(cand_tbl))
  }
  candidates <- candidates[rownames(cand_tbl)]
  names(candidates) <- cand_tbl$candidate_name

  if (verbose) {
    print(cand_tbl)
    message("Choose a candidate index, or set `choice = 0` to cancel.")
  }

  if (is.null(choice)) {
    if (nrow(cand_tbl) == 1L) {
      choice <- 1L
    } else if (interactive()) {
      choice <- utils::menu(c(paste0(cand_tbl$idx, ": ", cand_tbl$candidate_name), "0: none"))
    } else {
      return(structure(
        list(
          group_list = NULL,
          source_col = NULL,
          source_candidate = NULL,
          source_name = NULL,
          selected_index = NA_integer_,
          candidate_table = cand_tbl,
          candidates = candidates,
          pdata = pd
        ),
        class = "geo_auto_group"
      ))
    }
  }
  if (length(choice) != 1L || is.na(choice) || choice < 0L || choice > nrow(cand_tbl)) {
    stop("choice must be between 0 and ", nrow(cand_tbl), call. = FALSE)
  }
  if (choice == 0L) {
    return(structure(
      list(
        group_list = NULL,
        source_col = NULL,
        source_candidate = NULL,
        source_name = NULL,
        selected_index = 0L,
        candidate_table = cand_tbl,
        candidates = candidates,
        pdata = pd
      ),
      class = "geo_auto_group"
    ))
  }

  selected <- candidates[[choice]]
  if (!is.null(rownames(pd)) && length(rownames(pd)) == length(selected$group)) {
    names(selected$group) <- rownames(pd)
  }
  structure(
    list(
      group_list = selected$group,
      source_col = selected$column,
      source_candidate = selected$candidate,
      source_name = names(candidates)[choice],
      selected_index = choice,
      candidate_table = cand_tbl,
      candidates = candidates,
      pdata = pd
    ),
    class = "geo_auto_group"
  )
}

#' Print a GEO auto-group result
#'
#' @param x An object returned by `auto_geo_group()`.
#' @param ... Additional arguments passed to `print()`.
#' @export
print.geo_auto_group <- function(x, ...) {
  if (!is.null(x$group_list)) {
    cat("<geo_auto_group: selected>\n")
    cat("source column:", x$source_col, "\n")
    if (!is.null(x$source_name)) {
      cat("source name:", x$source_name, "\n")
    }
    if (!is.null(x$source_candidate)) {
      cat("source candidate:", x$source_candidate, "\n")
    }
    print(table(x$group_list))
    return(invisible(x))
  }
  cat("<geo_auto_group: candidates>\n")
  if (nrow(x$candidate_table)) {
    print(x$candidate_table)
  } else {
    cat("no candidates\n")
  }
  invisible(x)
}

##' make_tcga_group
##'
##' make tcga group for given tcga sample names or expression matrix
##'
##' @param exp sample names or an expression matrix/data.frame with sample
##'   names in its columns
##' @export
##' @return a group factor with normal and tumor, corresponding to the sample
##'   names
##' @author Xiaojie Sun
##' @examples
##' k = make_tcga_group(exp_hub1);table(k)
##' k2 = make_tcga_group(colnames(exp_hub1));table(k2)
##' @seealso
##' \code{\link{sam_filter}};\code{\link{match_exp_cl}}

make_tcga_group <- function(exp){
  samples <- if (!is.null(dim(exp))) colnames(exp) else as.character(exp)
  if (is.null(samples) || !length(samples)) stop("no sample names detected,please check it")
  k1 = startsWith(samples,"TCGA")
  if(!any(k1))stop("no TCGA samples detected,please check it")
  k2 = suppressWarnings(as.numeric(substr(samples,14,15))<10)
  group_list = ifelse(k1&k2,"tumor","normal")
  group_list = factor(group_list,levels = c("normal","tumor"))
  return(group_list)
}

##' trans_exp
##'
##' Transform rownames of an expression matrix from Ensembl IDs to gene
##' symbols.
##'
##' @param exp expression matrix with Ensembl IDs as rownames
##' @param mrna_only only keep mrna rows in result
##' @param lncrna_only only keep lncrna rows in result
##' @param gtex logical, whether to include GTEx data
##' @return a transformed expression set with symbol
##' @author Xiaojie Sun
##' @export
##' @examples
##' exp = matrix(rnorm(1000),ncol = 10)
##' rownames(exp) = sample(mRNA_anno$gene_id,100)
##' colnames(exp) = paste0("sample",1:10)
##' k  = trans_exp(exp)
##' @seealso
##' \code{\link{trans_array}}

trans_exp = function(exp,mrna_only = FALSE,
                     lncrna_only = FALSE,gtex = FALSE){
  kd = any(grepl("\\.", rownames(exp)))
  lanno = lnc_anno
  manno = mRNA_anno
  if(!kd){
    lanno$gene_id = sub("\\.\\d*$", "", lanno$gene_id)
    manno$gene_id = sub("\\.\\d*$", "", manno$gene_id)
  }
  map_by_annotation <- function(annotation) {
    idx <- match(rownames(exp), annotation$gene_id)
    keep <- !is.na(idx)
    res <- exp[keep, , drop = FALSE]
    anno <- annotation[idx[keep], , drop = FALSE]
    keep <- !duplicated(anno$gene_name) & !is.na(anno$gene_name) & nzchar(anno$gene_name)
    res <- res[keep, , drop = FALSE]
    rownames(res) <- anno$gene_name[keep]
    res
  }
  mRNA_exp = map_by_annotation(manno)
  lnc_exp = map_by_annotation(lanno)
  message(paste0(nrow(mRNA_exp),
                 " of genes successfully mapping to mRNA,",
                 nrow(lnc_exp),
                 " of genes successfully mapping to lncRNA"))
  if(mrna_only){
    return(mRNA_exp)
  }else if(lncrna_only){
      return(lnc_exp)
  }else{
    expa  = rbind(mRNA_exp,lnc_exp)
    k = !duplicated(rownames(expa))
    expa = expa[k,]
    return(expa)
    message(paste0(sum(!k),
                   " duplicated genes removed"))
    }
}
utils::globalVariables(c("lnc_anno","mRNA_anno"))

##' trans_array
##'
##' transform rownames for microarray or rnaseq expression matrix
##'
##' @param exp microarray or RNA-seq expression matrix
##' @param ids data.frame with original rownames and new rownames
##' @param from colname for original rownames
##' @param to colname for new rownames
##' @param average logical, if TRUE, average rows that map to the same target
##'   symbol; if FALSE, keep the first matched row for each symbol.
##' @return a transformed expression set with new rownames
##' @author Xiaojie Sun
##' @export
##' @examples
##' exp = matrix(1:50,nrow = 10)
##' rownames(exp) = paste0("g",1:10)
##' ids = data.frame(probe_id = paste0("g",1:10),
##'                 symbol = paste0("G",c(1:9,9)))
##' trans_array(exp,ids)
##' trans_array(exp,ids,average = TRUE)
##' @seealso
##' \code{\link{trans_exp}}

trans_array = function(exp,ids,from = "probe_id",
                       to = "symbol",average = FALSE){
  if(!is.character(ids[,from])) ids[,from] = as.character(ids[,from])
  # FIX: keep only IDs that are present in the expression matrix first.
  # Reason: this avoids carrying unmatched annotation rows into dedup/average.
  keep = ids[,from] %in% rownames(exp)
  ids = ids[keep, , drop = FALSE]
  exp = exp[ids[,from], , drop = FALSE]
  keep = !is.na(ids[,to]) & ids[,to] != ""
  ids = ids[keep, , drop = FALSE]
  exp = exp[keep, , drop = FALSE]
  if(average){
    exp = .tinyarray_collapse_mean_rows(exp, ids[,to])
    message(paste0(nrow(exp)," rownames transformed after averaging duplicate rows"))
  }else{
    keep = !duplicated(ids[,to])
    ids = ids[keep, , drop = FALSE]
    exp = exp[keep, , drop = FALSE]
    rownames(exp)=ids[,to]
    message(paste0(nrow(exp)," rownames transformed after duplicate rows removed"))
  }
  return(exp)
}


##' sam_filter
##'
##' drop duplicated samples from the same patients
##'
##' @param exp TCGA or TCGA+GTEx expression matrix from GDC or Xena
##' @return a transformed expression set without duplicated samples
##' @author Xiaojie Sun
##' @export
##' @examples
##' exp_hub1[1:4,1:4]
##' dim(exp_hub1)
##' exp2 = sam_filter(exp_hub1)
##' dim(exp2)
##' g = make_tcga_group(exp_hub1);table(g)
##' table(!duplicated(substr(colnames(exp_hub1[,g=="tumor"]),1,12)))
##' @seealso
##' \code{\link{make_tcga_group}};\code{\link{match_exp_cl}}

sam_filter = function(exp){
  exp = exp[,order(colnames(exp))]
  n1 = ncol(exp)
  group = make_tcga_group(exp)
  exptumor = exp[,group == "tumor"]

  expnormol = exp[,group == "normal"]
  exptumor = exptumor[,!duplicated(substr(colnames(exptumor),1,12))]
  expnormol = expnormol[,!duplicated(substr(colnames(expnormol),1,12))]

  exp = cbind(exptumor,expnormol)
  message(paste("filtered",n1-ncol(exp),"samples."))
  return(exp)
}


##' match_exp_cl
##'
##' match exp and clinical data from TCGA
##'
##' @param exp TCGA  expression set
##' @param cl TCGA clinical data.frame
##' @param id_column which column contains patient IDs, either a column number or
##'   column name
##' @param sample_centric logical, default TRUE; keep all samples from the same
##'   patients. If FALSE, keep only one tumor sample for each patient.
##' @return a transformed clinical data.frame with sample ids.
##' @author Xiaojie Sun
##' @export
##' @examples
##' a = match_exp_cl(exp_hub1,meta1[,2:4],"X_PATIENT")
##' exp_matched = a[[1]]
##' cl_matched = a[[2]]
##' b = match_exp_cl(exp_hub1,meta1[,2:4],"X_PATIENT",sample_centric = FALSE)
##' exp_matched = b[[1]]
##' cl_matched = b[[2]]
##' @seealso
##' \code{\link{make_tcga_group}};\code{\link{sam_filter}}

match_exp_cl = function(exp,cl,id_column = "id",sample_centric = TRUE){
  if (is.numeric(id_column)) {
    if (length(id_column) != 1L || is.na(id_column) || id_column < 1L || id_column > ncol(cl)) {
      stop("id_column must be a valid column number or column name", call. = FALSE)
    }
    colnames(cl)[id_column] = "id"
  } else {
    if (!id_column %in% colnames(cl)) {
      stop("id_column not found in cl", call. = FALSE)
    }
    colnames(cl)[colnames(cl) == id_column] = "id"
  }
  cl = cl[cl$id %in% substr(colnames(exp),1,12),]
  exp = exp[,substr(colnames(exp),1,12) %in% cl$id]
  patient <- substr(colnames(exp),1,12)
  if(nrow(cl)==0) stop("your exp or cl doesn't match,please check them.")
  da = data.frame(sample_id = colnames(exp),
                  id = patient)
  cl = merge(da,cl,by ="id",all.y = TRUE)
  cl = cl[match(colnames(exp),cl$sample_id),]
  if(!sample_centric) {
    Group = make_tcga_group(exp)
    exp = exp[,Group=="tumor"]
    cl = cl[Group=="tumor",]
    cl = cl[order(colnames(exp)),]
    exp = exp[,sort(colnames(exp))]
    exp = exp[,!duplicated(cl$id)]
    cl = cl[!duplicated(cl$id),]
  }
  k = identical(colnames(exp),cl$sample_id)
  if(k)message("match successfully")
  rownames(cl) = cl$sample_id
  return(list(exp_matched = exp,
              cl_matched = cl))
  message("New version of tinyarray canceled global assigning inside the package,
          please obtain exp_matched and cl_matched by split this list result.")
}

##' trans_ensembl_exp
##'
##' Transform rownames of an expression set from Ensembl IDs to gene symbols
##' using the latest gene annotation.
##'
##' @param exp expression matrix with Ensembl IDs as rownames
##' @param mrna_only only keep mrna rows in result
##' @param lncrna_only only keep lncrna rows in result
##' @param species choose human, mouse, or rat; default is human
##' @param average logical, if TRUE, average rows that map to the same target
##'   symbol; if FALSE, keep the first matched row for each symbol.
##' @return a transformed expression set with symbol
##' @author Xiaojie Sun
##' @export
##' @examplesIf requireNamespace("AnnotationDbi", quietly = TRUE) && requireNamespace("org.Hs.eg.db", quietly = TRUE)
##' exp = matrix(rnorm(1000),ncol = 10)
##' rownames(exp) = sample(mRNA_anno$gene_id,100)
##' colnames(exp) = c(paste0("TCGA",1:5),paste0("GTEX",1:5))
##' k  = trans_ensembl_exp(exp)
##' @seealso
##' \code{\link{trans_exp_new}}
trans_ensembl_exp = function(exp,mrna_only = FALSE,
                             lncrna_only = FALSE,
                             species = "human",
                             average = FALSE){
  if(is.data.frame(exp)){exp = as.matrix(exp)}
  if(mrna_only & lncrna_only) stop("please change neither mrna_only or lncrna_only to FALSE")
  rownames(exp) = sub("\\.\\d+$", "", rownames(exp))
  anno = .tinyarray_ensembl_annotation_table(exp, species = species)
  if(mrna_only){
    anno = anno[grepl("protein", tolower(anno$GENETYPE)), , drop = FALSE]
  }else if(lncrna_only){
    anno = anno[grepl("lncrna", tolower(anno$GENETYPE)), , drop = FALSE]
  }
  exp = trans_array(exp,ids = anno,from = "ENSEMBL",to = "SYMBOL",average = average)
  return(exp)
}

##' trans_exp_new
##'
##' Route expression-matrix ID conversion by type.
##'
##' @param exp expression matrix
##' @param mrna_only only keep mrna rows in result when `type = "ensembl"`
##' @param lncrna_only only keep lncrna rows in result when `type = "ensembl"`
##' @param species choose human, mouse, or rat; default is human
##' @param average logical, if TRUE, average rows that map to the same target
##'   symbol; if FALSE, keep the first matched row for each symbol.
##' @param type conversion type, one of `ensembl`, `array`, or `entrez`
##' @param ids annotation table used when `type = "array"`
##' @param id alias of `ids`; kept for compatibility
##' @return a transformed expression set with symbol
##' @author Xiaojie Sun
##' @export
##' @examplesIf requireNamespace("AnnotationDbi", quietly = TRUE) && requireNamespace("org.Hs.eg.db", quietly = TRUE)
##' exp = matrix(rnorm(1000),ncol = 10)
##' rownames(exp) = sample(mRNA_anno$gene_id,100)
##' colnames(exp) = c(paste0("TCGA",1:5),paste0("GTEX",1:5))
##' k  = trans_exp_new(exp)
##' @seealso
##' \code{\link{trans_ensembl_exp}};\code{\link{trans_array}};\code{\link{trans_entrezexp}}
trans_exp_new = function(exp, mrna_only = FALSE,
                         lncrna_only = FALSE,
                         species = "human",
                         average = FALSE,
                         type = c("ensembl", "array", "entrez"),
                         ids = NULL,
                         id = NULL){
  if (is.data.frame(exp)) {
    exp <- as.matrix(exp)
  }
  if (is.null(type) || !length(type)) {
    type <- "ensembl"
  }
  if (identical(tolower(type[[1L]]), "enrtez")) {
    warning("`type = \"enrtez\"` is treated as `type = \"entrez\"`.", call. = FALSE)
    type[[1L]] <- "entrez"
  }
  type <- match.arg(tolower(type[[1L]]), choices = c("ensembl", "array", "entrez"))

  if (type == "ensembl") {
    return(trans_ensembl_exp(
      exp = exp,
      mrna_only = mrna_only,
      lncrna_only = lncrna_only,
      species = species,
      average = average
    ))
  }

  if (!is.null(id)) {
    if (is.null(ids)) {
      ids <- id
    } else {
      warning("`id` is ignored because `ids` is already provided.", call. = FALSE)
    }
  }
  if (!is.null(ids) && type != "array") {
    warning("`ids` is only used when `type = \"array\"`.", call. = FALSE)
  }

  if (type == "array") {
    if (is.null(ids)) {
      stop("please provide `ids` when `type = \"array\"`.", call. = FALSE)
    }
    ids <- .tinyarray_standardize_probe_symbol_table(ids)
    if (!nrow(ids)) {
      stop("`ids` does not contain any valid `probe_id` / `symbol` pairs.", call. = FALSE)
    }
    return(trans_array(exp, ids = ids, from = "probe_id", to = "symbol", average = average))
  }

  trans_entrezexp(exp, species = species, average = average)
}

##' trans_entrezexp
##'
##' Transform rownames of an expression set from Entrez IDs to gene symbols
##' using \code{clusterProfiler::bitr()}.
##'
##' @param entrezexp expression matrix with Entrez IDs as rownames
##' @param species choose human, mouse, or rat; default is human
##' @param average logical, if TRUE, average rows that map to the same target
##'   symbol; if FALSE, keep the first matched row for each symbol.
##' @return a transformed expression set with symbol
##' @author Xiaojie Sun
##' @export
##' @examplesIf requireNamespace("clusterProfiler", quietly = TRUE) && requireNamespace("org.Hs.eg.db", quietly = TRUE)
##' exp = matrix(rnorm(200),ncol = 10)
##' rownames(exp) = c("79691", "56271", "8662", "10394", "55630", "159162", "23541",
##'                   "79723", "54413", "22927", "92342", "23787", "5550", "8924",
##'                   "55274", "866", "8844", "353299", "587", "1473")
##' colnames(exp) = paste0("s",1:10)
##' exp2 = trans_entrezexp(exp)
##' @seealso
##' \code{\link{trans_exp}}
trans_entrezexp = function(entrezexp,species="human",average = FALSE){
  org = .tinyarray_orgdb_for_species(species)
  or = org$orgdb
  if(is.data.frame(entrezexp)){entrezexp = as.matrix(entrezexp)}
  .tinyarray_require_clusterProfiler()
  re = clusterProfiler::bitr(rownames(entrezexp),fromType = "ENTREZID",toType = "SYMBOL",OrgDb = or)
  exp = trans_array(entrezexp,ids = re,from = "ENTREZID",to = "SYMBOL",average = average)
  return(exp)
}

##' @rdname trans_entrezexp
##' @export
trans_entrez_exp <- trans_entrezexp

#' cod
#'
#' Example TCGA expression matrix used in grouping and plotting demos.
#'
#' @examples
#'   cod
"cod"

#' deg
#'
#' Example limma differential analysis result for GSE42872.
#'
#' @examples
#'   head(deg)
"deg"

#' deseq_data
#'
#' Example DESeq2 differential analysis result.
#'
#' @examples
#'   head(deseq_data)
"deseq_data"

#' exists_anno_list
#'
#' GEO GPL platforms with direct AnnoProbe annotation support.
#'
#' @examples
#'   exists_anno_list
"exists_anno_list"

#' exp_hub1
#'
#' Example TCGA+GTEx expression matrix.
#'
#' @examples
#'   exp_hub1[1:4,1:4]
"exp_hub1"


#' exprSet_hub1
#'
#' Example TCGA CPM matrix with tumor samples only.
#'
#' @examples
#'   exprSet_hub1[1:4,1:4]
"exprSet_hub1"


#' genes
#'
#' Small example vector of Entrez IDs.
#'
#' @examples
#'   genes
"genes"

#' lnc_anno
#'
#' Compact lncRNA annotation table for TCGA expression matrices.
#'
#' Only the columns needed by the package are kept, so the bundled object stays
#' small and can be used offline without extra annotation files.
#'
#' @examples
#'   head(lnc_anno)
"lnc_anno"

#' mRNA_anno
#'
#' Compact mRNA annotation table for TCGA expression matrices.
#'
#' Only the columns needed by the package are kept, so the bundled object stays
#' small and can be used offline without extra annotation files.
#'
#' @examples
#'   head(mRNA_anno)
"mRNA_anno"

#' meta1
#'
#' Example clinical table matched to exprSet_hub1.
#'
#' @examples
#'   head(meta1)
"meta1"

#' pkg_all
#'
#' Lookup table for GEO platform annotation packages.
#'
#' @examples
#'   head(pkg_all)
"pkg_all"

##' @importFrom utils packageDescription
.onAttach <- function(libname, pkgname) {
    packageStartupMessage("Welcome to tinyarray.
If you use tinyarray in published research, please acknowledge:
We thank Dr. Jianming Zeng (University of Macau), the members of his bioinformatics team and biotrainee, especially Xiaojie Sun, for generously sharing their experience and code.
")
}

##' multi_deg
##'
##' Do differential analysis according to an expression set and group
##' information.
##'
##' @inheritParams get_deg
##' @param ids optional probe-to-symbol annotation table. If `NULL`, `exp` is
##'   treated as a gene-level matrix and no ID conversion is performed.
##' @return a deg data.frame with one element per contrast
##' @author Xiaojie Sun
##' @importFrom limma lmFit
##' @importFrom limma eBayes
##' @importFrom limma topTable
##' @importFrom limma makeContrasts
##' @importFrom limma contrasts.fit
##' @importFrom dplyr mutate
##' @importFrom dplyr inner_join
##' @export
##' @examples
##' \dontrun{
##' if(requireNamespace("Biobase",quietly = TRUE)&
##'    requireNamespace("AnnoProbe",quietly = TRUE)){
##'   gse = "GSE474"
##'   geo = geo_download(gse,destdir=tempdir())
##'   geo$exp[1:4,1:4]
##'   geo$exp=log2(geo$exp+1)
##'   group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),
##'   "MObese",ifelse(stringr::str_detect(geo$pd$title,"NonObese"),
##'   "NonObese","Obese"))
##'   group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
##'   find_anno(geo$gpl)
##'   ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
##'   deg = multi_deg(geo$exp,group_list,ids,adjust = FALSE,entriz = FALSE)
##'   names(deg)
##'   head(deg[[1]])
##'   head(deg[[2]])
##'   head(deg[[3]])
##' }else{
##'   if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
##'     warning("Package 'AnnoProbe' needed for this function to work.
##'          Please install it by install.packages('AnnoProbe')",call. = FALSE)
##'   }
##'   if(!requireNamespace("Biobase",quietly = TRUE)) {
##'     warning("Package 'Biobase' needed for this function to work.
##'          Please install it by BiocManager::install('Biobase')",call. = FALSE)
##'   }
##' }
##' }
##' @seealso
##' \code{\link{get_deg}};\code{\link{multi_deg_all}}

multi_deg <- function(exp,
                      group_list,
                      ids = NULL,
                      logFC_cutoff = 1,
                      pvalue_cutoff = 0.05,
                      adjust = TRUE,
                      species = "human",
                      entriz = TRUE) {
  p1 <-  all(apply(exp,2,is.numeric))
  if(!p1) stop("exp must be a numeric matrix")
  group_list <- .tinyarray_prepare_group_list_safe(group_list)
  if (is.null(group_list)) return(NULL)
  if(ncol(exp)!=length(group_list))stop("wrong group_list or exp")
  px <- levels(group_list)
  design = stats::model.matrix(~0+group_list)
  colnames(design) = px
  cmb = utils::combn(px,2,simplify = FALSE)
  contrast_names = vapply(cmb,function(v) paste0(v[2],"-",v[1]),character(1))
  contrast.matrix = limma::makeContrasts(contrasts = contrast_names, levels = design)
  colnames(contrast.matrix) = contrast_names

  fit <- limma::lmFit(exp, design)
  fit2 <- limma::contrasts.fit(fit, contrast.matrix)
  fit2 <- limma::eBayes(fit2)
  deg = lapply(seq_along(contrast_names), function(i){
    limma::topTable(fit2,coef = i,number = Inf)
  })
  names(deg) = colnames(contrast.matrix)

  if (is.null(ids)) {
    ids <- NULL
  } else {
    if(ncol(ids)!=2)stop("wrong ids parameter, it should be a data.frame with probe_id and symbol")
    colnames(ids) = c("probe_id","symbol")
    ids = stats::na.omit(ids)
    if(!is.character(ids$probe_id)) ids$probe_id = as.character(ids$probe_id)
    ids = .tinyarray_dedup_by_key(ids,"symbol")
    ids = .tinyarray_dedup_by_key(ids,"probe_id")
  }

  for(i in seq_along(deg)){
    deg[[i]]$probe_id <- rownames(deg[[i]])
    if (is.null(ids)) {
      deg[[i]]$symbol <- as.character(deg[[i]]$probe_id)
    } else {
      deg[[i]] <- dplyr::inner_join(deg[[i]],ids,by="probe_id")
    }
    if(adjust){
      k1 = (deg[[i]]$adj.P.Val < pvalue_cutoff)&(deg[[i]]$logFC < -logFC_cutoff)
      k2 = (deg[[i]]$adj.P.Val < pvalue_cutoff)&(deg[[i]]$logFC > logFC_cutoff)
    }else{
      k1 = (deg[[i]]$P.Value < pvalue_cutoff)&(deg[[i]]$logFC < -logFC_cutoff)
      k2 = (deg[[i]]$P.Value < pvalue_cutoff)&(deg[[i]]$logFC > logFC_cutoff)
    }
    change = ifelse(k1,"down",ifelse(k2,"up","stable"))
    deg[[i]]$change <- change
    if(entriz){
      org = .tinyarray_orgdb_for_species(species)
      .tinyarray_require_clusterProfiler()
      s2e <- tryCatch({
        clusterProfiler::bitr(unique(deg[[i]]$symbol), fromType = "SYMBOL",
             toType = c("ENTREZID"),
             OrgDb = org$orgdb)
      },error = function(e){666})
      if(!is.character(s2e)){
        s2e <- s2e[!duplicated(s2e$SYMBOL),]
        before_n <- nrow(deg[[i]])
        deg2 <- dplyr::inner_join(deg[[i]],s2e,by=c("symbol"="SYMBOL"))
        if (nrow(deg2) < max(1L, floor(before_n * 0.1))) {
          warning(
            "Entrez ID conversion kept fewer than 10% of rows; please check the expression row names, ids, or species.",
            call. = FALSE
          )
        } else {
          deg[[i]] <- deg2
        }
      }else{
        warning("Entrez ID conversion failed, please check the previous warning.")
      }
    }
  }
  return(deg)
}

##' count unique values in every columns for data.frame
##'
##' in geo analysis,this function can help you simplify pdata, delete columns with unique values,which can't be used as group vector
##' @param x A data.frame.
##' @return The simple data.frame of columns unique values count in \code{x}
##' @export
##' @examples
##' dumd(iris)
##' data(ToothGrowth)
##' x = ToothGrowth
##' dumd(ToothGrowth)

dumd <- function(x){
  colname <- colnames(x)
  if (is.null(colname)) {
    colname <- paste0("V", seq_len(ncol(x)))
  }
  count <- vapply(seq_len(ncol(x)), function(i) {
    sum(!duplicated(x[, i, drop = TRUE]))
  }, integer(1))
  df <- data.frame(colname = colname, count = count, stringsAsFactors = FALSE)
  df[order(df$count, decreasing = TRUE), , drop = FALSE]
}

##' intersect_all
##'
##' calculate intersect  set for two or more elements
##'
##' @param ... some vectors or a list with some vectors
##' @export
##' @return vector
##' @author Xiaojie Sun
##' @examples
##' x1 = letters[1:4]
##' x2 = letters[3:6]
##' x3 = letters[3:4]
##' re =intersect_all(x1,x2,x3)
##' re2 = intersect_all(list(x1,x2,x3))
##' re3 = union_all(x1,x2,x3)
##' @seealso
##' \code{\link{union_all}}
intersect_all <- function(...){
  li = list(...)
  if(length(li)==1 & is.list(li[[1]])) li = li[[1]]
  if(length(li)==0)return(character(0))
  result = li[[1]]
  for(k in seq_along(li)){
    result = intersect(result,li[[k]])
  }
  return(result)
}

##' union_all
##'
##' calculate union set for two or more elements
##'
##' @param ... some vectors or a list with some vectors
##' @export
##' @return vector
##' @author Xiaojie Sun
##' @examples
##' x1 = letters[1:4]
##' x2 = letters[3:6]
##' x3 = letters[3:4]
##' re =intersect_all(x1,x2,x3)
##' re2 = intersect_all(list(x1,x2,x3))
##' re3 = union_all(x1,x2,x3)
##' @seealso
##' \code{\link{intersect_all}}
union_all <-  function(...){
  li = list(...)
  if(length(li)==1 & is.list(li[[1]])) li = li[[1]]
  if(length(li)==0)return(character(0))
  result = li[[1]]
  for(k in seq_along(li)){
    result = union(result,li[[k]])
  }
  return(result)
}

##' get_deg
##'
##' Do differential analysis according to an expression set and group
##' information.
##'
##' @inheritParams draw_pca
##' @inheritParams draw_volcano
##' @inheritParams trans_ensembl_exp
##' @param ids optional probe-to-symbol annotation table. If `NULL`, `exp` is
##'   treated as a gene-level matrix and no ID conversion is performed.
##' @param entriz whether to convert symbols to Entrez IDs
##' @return a deg data.frame
##' @author Xiaojie Sun
##' @importFrom limma lmFit
##' @importFrom limma eBayes
##' @importFrom limma topTable
##' @importFrom dplyr mutate
##' @importFrom dplyr inner_join
##' @export
##' @examples
##' \dontrun{
##' if(requireNamespace("Biobase",quietly = TRUE)&
##'    requireNamespace("AnnoProbe",quietly = TRUE)){
##'   gse = "GSE42872"
##'   a = geo_download(gse,destdir=tempdir())
##'   find_anno(geo$gpl)
##'   ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
##'   Group = rep(c("control","treat"),each = 3)
##'   Group = factor(Group)
##'   deg = get_deg(geo$exp,Group,ids,entriz = FALSE)
##'   head(deg)
##' }else{
##'   if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
##'     warning("Package 'AnnoProbe' needed for this function to work.
##'          Please install it by install.packages('AnnoProbe')",call. = FALSE)
##'   }
##'   if(!requireNamespace("Biobase",quietly = TRUE)) {
##'     warning("Package 'Biobase' needed for this function to work.
##'          Please install it by BiocManager::install('Biobase')",call. = FALSE)
##'   }
##' }
##' }
##' @seealso
##' \code{\link{multi_deg}};\code{\link{get_deg_all}}

get_deg <- function(exp,
                    group_list,
                    ids = NULL,
                    logFC_cutoff=1,
                    pvalue_cutoff=0.05,
                    adjust = TRUE,
                    entriz = TRUE,
                    species = "human") {
  group_list <- .tinyarray_prepare_group_list_safe(group_list)
  if (is.null(group_list)) return(NULL)
  if(nlevels(group_list)==2){
    if(ncol(exp)!=length(group_list))stop("wrong group_list or exp")
    design=stats::model.matrix(~group_list)
    fit=lmFit(exp,design)
    fit=eBayes(fit)
    deg=topTable(fit,coef=2,number = Inf)

    if("ID" %in% colnames(deg)){
      deg$probe_id <- deg$ID
    }else{
      deg$probe_id <- rownames(deg)
    }
    if (is.null(ids)) {
      deg$symbol <- as.character(deg$probe_id)
    } else {
      if(ncol(ids)!=2)stop("wrong ids parameter, it should be a data.frame with probe_id and symbol")
      colnames(ids) = c("probe_id","symbol")
      if(!is.character(ids$probe_id)) ids$probe_id = as.character(ids$probe_id)
      ids = stats::na.omit(ids)
      ids = .tinyarray_dedup_by_key(ids,"symbol")
      deg <- inner_join(deg,ids,by="probe_id")
    }
    if(adjust){
      k1 = (deg$adj.P.Val < pvalue_cutoff)&(deg$logFC < -logFC_cutoff)
      k2 = (deg$adj.P.Val < pvalue_cutoff)&(deg$logFC > logFC_cutoff)
    }else{
      k1 = (deg$P.Value < pvalue_cutoff)&(deg$logFC < -logFC_cutoff)
      k2 = (deg$P.Value < pvalue_cutoff)&(deg$logFC > logFC_cutoff)
    }

    change = ifelse(k1,
                    "down",
                    ifelse(k2,
                           "up",
                           "stable"))
    deg$change <- change

    if(entriz){
      org = .tinyarray_orgdb_for_species(species)
      .tinyarray_require_clusterProfiler()
      s2e <- tryCatch({
        clusterProfiler::bitr(deg$symbol,fromType = "SYMBOL",toType = "ENTREZID",OrgDb = org$orgdb)
      },error = function(e){666})
      if(!is.character(s2e)){
        before_n <- nrow(deg)
        deg2 <- inner_join(deg,s2e,by=c("symbol"="SYMBOL"))
        if (nrow(deg2) < max(1L, floor(before_n * 0.1))) {
          warning(
            "Entrez ID conversion kept fewer than 10% of rows; please check the expression row names, ids, or species.",
            call. = FALSE
          )
        } else {
          deg <- deg2
          deg <- deg[!duplicated(deg$symbol),]
        }
      }else{
        warning("Entrez ID conversion failed, please check the previous warning.")
      }
    }
  }else{
    deg <-multi_deg(exp = exp,
                    group_list = group_list,
                    ids = ids,
                    logFC_cutoff = logFC_cutoff,
                    pvalue_cutoff = pvalue_cutoff,
                    adjust = adjust,
                    entriz = entriz,
                    species = species)
  }
  return(deg)
}

##' get_cgs
##'
##' extract DEGs from deg data.frame
##'
##' @inheritParams draw_volcano
##' @return a list with upgenes,downgenes,diffgenes.
##' @author Xiaojie Sun
##' @export
##' @examples
##' \dontrun{
##' #two group
##' gse = "GSE42872"
##' geo = geo_download(gse,destdir=tempdir())
##' group_list = rep(c("A","B"),each = 3)
##' ids = AnnoProbe::idmap('GPL6244',destdir=tempdir())
##' deg = get_deg(geo$exp,group_list,ids)
##' cgs = get_cgs(deg)
##' #mutigroup
##' gse = "GSE474"
##' geo = geo_download(gse,destdir=tempdir())
##' geo$exp[1:4,1:4]
##' geo$exp=log2(geo$exp+1)
##' group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
##' ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
##' group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
##' find_anno(geo$gpl)
##' ids = AnnoProbe::idmap(geo$gpl,destdir = tempdir())
##' deg = multi_deg(geo$exp,group_list,ids,adjust = FALSE)
##' cgs = get_cgs(deg)
##' }
##' @seealso
##' \code{\link{geo_download}};\code{\link{draw_volcano}};\code{\link{draw_venn}}


get_cgs <- function(deg){
  if (is.null(deg)) {
    warning("deg is NULL; skipping gene extraction.", call. = FALSE)
    return(NULL)
  }
  if(!is.list(deg) & !is.data.frame(deg)){
    warning("deg is not a data.frame or list returned by limma", call. = FALSE)
    return(NULL)
  }
  cgs = list()
  if(is.data.frame(deg)) deg = list(deg = deg)
  for(i in 1:length(deg)){
    cgs[[i]] = list(up = data.frame(upgenes =deg[[i]]$symbol[deg[[i]]$change=="up"],
                                    upprobes = deg[[i]]$probe_id[deg[[i]]$change=="up"],
                                    stringsAsFactors = FALSE),
                    down = data.frame(downgenes = deg[[i]]$symbol[deg[[i]]$change=="down"],
                                      downprobes = deg[[i]]$probe_id[deg[[i]]$change=="down"],
                                      stringsAsFactors = FALSE),
                    diff = data.frame(diffgenes = deg[[i]]$symbol[deg[[i]]$change!="stable"],
                                      diffprobes = deg[[i]]$probe_id[deg[[i]]$change!="stable"],
                                      stringsAsFactors = FALSE)
    )
  }
  names(cgs) = names(deg)
  return(cgs)
}

##' draw_volcano2
##'
##' Print one or more volcano plots for differential analysis results in a
##' data.frame.
##'
##' @inheritParams draw_volcano
##' @param ... other parameters from draw_volcano
##' @return one or more volcano plot
##' @author Xiaojie Sun
##' @importFrom patchwork wrap_plots
##' @importFrom patchwork plot_layout
##' @export
##' @examples
##' \dontrun{
##' if(requireNamespace("Biobase",quietly = TRUE)&
##'    requireNamespace("AnnoProbe",quietly = TRUE)){
##' #two group
##' gse = "GSE42872"
##' geo = geo_download(gse,destdir=tempdir())
##' group_list = rep(c("A","B"),each = 3)
##' ids = AnnoProbe::idmap('GPL6244',destdir = tempdir())
##' deg = get_deg(geo$exp,group_list,ids)
##' draw_volcano2(deg)
##' #multigroup
##' gse = "GSE474"
##' geo = geo_download(gse,destdir=tempdir())
##' geo$exp[1:4,1:4]
##' geo$exp=log2(geo$exp+1)
##' group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
##' ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
##' group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
##' find_anno(geo$gpl)
##' ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
##' deg = multi_deg(geo$exp,group_list,ids,adjust = FALSE,entriz = FALSE)
##' draw_volcano2(deg)
##' draw_volcano2(deg,color = c("darkgreen","grey","darkred"))
##' }else{
##'   if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
##'     warning("Package 'AnnoProbe' needed for this function to work.
##'          Please install it by install.packages('AnnoProbe')",call. = FALSE)
##'   }
##'   if(!requireNamespace("Biobase",quietly = TRUE)) {
##'     warning("Package 'Biobase' needed for this function to work.
##'          Please install it by BiocManager::install('Biobase')",call. = FALSE)
##'   }
##' }
##' }
##' @seealso
##' \code{\link{geo_download}};\code{\link{draw_volcano}};\code{\link{draw_venn}}

draw_volcano2 = function(deg,
                         pkg=4,
                         lab,
                         change_col = "change",
                         ...
){
  lab_missing = missing(lab)
  if(!is.list(deg) & !is.data.frame(deg))stop("deg is a data.frame or list returned by limma")
  if(is.data.frame(deg)) {
    volcano_plots = draw_volcano(
      deg ,
      pkg = pkg,
      change_col = change_col,
      lab = if(lab_missing) "logFC" else lab,
      ...
    )
  }else{
    volcano_plots <- lapply(1:length(deg),
                            function(k) {
                              lab_k = if(lab_missing) {
                                if(!is.null(names(deg)) && nzchar(names(deg)[k])) names(deg)[k] else "logFC"
                              } else if(length(lab)==length(deg)) {
                                lab[[k]]
                              } else {
                                lab
                              }
                              draw_volcano(
                                deg[[k]] ,
                                pkg = pkg,
                                change_col = change_col,
                                lab = lab_k,
                                ...
                              )
                            })
    volcano_plots = wrap_plots(volcano_plots)+
      plot_layout(design = paste(rep(LETTERS[1:length(deg)]),collapse = "")) +
      plot_layout(guides = 'collect')
  }
  return(volcano_plots)
}

##' draw heatmap plots
##'
##' Print heatmap plots for an expression matrix grouped by \code{group_list}.
##'
##' @inheritParams draw_volcano
##' @inheritParams draw_heatmap
##' @inheritParams draw_pca
##' @param heat_union logical ,use union or intersect DEGs for heatmap
##' @param my_genes genes for pheatmap
##' @param ... other parameters from draw_heatmap
##' @return a heatmap plot according to \code{exp} and grouped by
##'   \code{group_list}.
##' @author Xiaojie Sun
##' @importFrom pheatmap pheatmap
##' @export
##' @examples
##' \dontrun{
##' if(requireNamespace("Biobase",quietly = TRUE)&
##'    requireNamespace("AnnoProbe",quietly = TRUE)){
##'   gse = "GSE474"
##'   geo = geo_download(gse,destdir=tempdir())
##'   geo$exp[1:4,1:4]
##'   geo$exp=log2(geo$exp+1)
##'   group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
##'   ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
##'   group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
##'   find_anno(geo$gpl)
##'   ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
##'   deg = multi_deg(geo$exp,group_list,ids,adjust = FALSE,entriz = FALSE)
##'   draw_heatmap2(geo$exp,group_list,deg)
##' }else{
##'   if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
##'     warning("Package 'AnnoProbe' needed for this function to work.
##'          Please install it by install.packages('AnnoProbe')",call. = FALSE)
##'   }
##'   if(!requireNamespace("Biobase",quietly = TRUE)) {
##'     warning("Package 'Biobase' needed for this function to work.
##'          Please install it by BiocManager::install('Biobase')",call. = FALSE)
##'   }
##' }
##' }
##' @seealso
##' \code{\link{draw_pca}};\code{\link{draw_volcano}};\code{\link{draw_venn}}

draw_heatmap2 <- function(exp,
                          group_list,
                          deg,
                          my_genes = NULL,
                          heat_union = TRUE,...
){
  if (!is.data.frame(deg)) {
    m = unique(do.call(rbind, lapply(deg, function(x) {
      x[, c("probe_id", "symbol"), drop = FALSE]
    })))
    m = m[!is.na(m$symbol) & nzchar(m$symbol), , drop = FALSE]
    m = m[!duplicated(m$symbol), , drop = FALSE]
  } else {
    m = deg[, c("probe_id", "symbol"), drop = FALSE]
  }
  if(all(m$probe_id %in% rownames(exp))){
    exp = exp[match(m$probe_id, rownames(exp)), , drop = FALSE]
    rownames(exp) = m$symbol
  }else if(all(m$symbol %in% rownames(exp))) {
    exp = exp[match(m$symbol, rownames(exp)), , drop = FALSE]
  }
  if(is.null(my_genes)){
    cgs = get_cgs(deg)
    if(length(cgs)==1){
      cg = cgs[[1]]$diff$diffgenes
    }else{
      if(heat_union){
        cg = union_all(lapply(cgs,function(x)x$diff$diffgenes))
      }else{
        cg = intersect_all(lapply(cgs,function(x)x$diff$diffgenes))
      }
    }
    }else{
      cg = m$symbol[m$symbol %in% my_genes]
    }
  if(length(cg)==0){
    warning("no genes left for heatmap; skipping heatmap plot.", call. = FALSE)
    return(NULL)
  }
  cg <- intersect(cg, rownames(exp))
  if(length(cg)==0){
    warning("no genes left for heatmap after matching expression rownames; skipping heatmap plot.", call. = FALSE)
    return(NULL)
  }
  heatmap = draw_heatmap(exp[cg, , drop = FALSE],
                         group_list,
                         ...)
  return(heatmap)
}

##' plot_deg
##'
##' Plot PCA, volcano, heatmap, and Venn plots for differential analysis
##' results.
##'
##' @inheritParams multi_deg_all
##' @param  deg result of multi_deg or get_deg function
##' @return plots
##' @author Xiaojie Sun
##' @export
##' @examples
##' \dontrun{
##' if(requireNamespace("Biobase",quietly = TRUE)&
##'    requireNamespace("AnnoProbe",quietly = TRUE)){
##' gse = "GSE474"
##' geo = geo_download(gse,destdir=tempdir())
##' geo$exp[1:4,1:4]
##' geo$exp=log2(geo$exp+1)
##' group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
##' ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
##' group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
##' find_anno(geo$gpl)
##' ids = AnnoProbe::idmap(geo$gpl,destdir = tempdir())
##' deg = get_deg(geo$exp,group_list,ids,adjust = FALSE,entriz = FALSE)
##' plot_deg(geo$exp,group_list,deg)
##' }else{
##'   if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
##'     warning("Package 'AnnoProbe' needed for this function to work.
##'          Please install it by install.packages('AnnoProbe')",call. = FALSE)
##'   }
##'   if(!requireNamespace("Biobase",quietly = TRUE)) {
##'     warning("Package 'Biobase' needed for this function to work.
##'          Please install it by BiocManager::install('Biobase')",call. = FALSE)
##'   }
##' }
##' }
plot_deg = function(exp,
                    group_list,
                    deg,
                    symmetry = TRUE,
                    my_genes = NULL,
                    show_rownames = FALSE,
                    cluster_cols = TRUE,
                    color_volcano = c("#2874C5", "grey", "#f87669"),
                    pvalue_cutoff = 0.05,
                    logFC_cutoff = 1,
                    adjust = TRUE,
                    change_col = "change",
                    annotation_legend = FALSE,
                    lab = NA,
                    species = "human"
){
  cgs = get_cgs(deg)
  volcano_plot = draw_volcano2(deg,
                               pkg = 4,
                               symmetry = symmetry,
                               color = color_volcano,
                               pvalue_cutoff = pvalue_cutoff,
                               logFC_cutoff = logFC_cutoff,
                               adjust = adjust,
                               lab = lab,
                               change_col = change_col
  )
  pca_plot = draw_pca(exp,group_list)
  heatmap = draw_heatmap2(exp,group_list,deg,my_genes,
                          show_rownames = show_rownames,
                          cluster_cols = cluster_cols,
                          annotation_legend = annotation_legend)
  x = lapply(cgs,function(x)x$diff$diffprobes)
  if (all(vapply(x, length, integer(1)) == 0L)) {
    warning("no DEGs found across contrasts; skipping venn plot.", call. = FALSE)
    venn = NULL
  } else {
    venn = draw_venn(x," ")
  }
  if(as.numeric(grDevices::dev.cur())!=1) grDevices::graphics.off()
  plotlist = list(heatmap,pca_plot,venn,volcano_plot)
  layout <- '
  AABBCC
  AABBCC
  DDDDDD
  DDDDDD
  '
  plots = if (is.null(heatmap)) {
    .tinyarray_bundle_plots(pca_plot, venn, volcano_plot)
  } else {
    wrap_plots(plotlist) +
      plot_layout(design = layout) +
      plot_layout(guides = 'collect')
  }
  return(plots)
}

##' multi_deg_all
##'
##' Do differential analysis according to an expression set and group
##' information.
##'
##' @inheritParams draw_pca
##' @inheritParams draw_volcano
##' @inheritParams pheatmap::pheatmap
##' @inheritParams draw_heatmap
##' @inheritParams draw_heatmap2
##' @inheritParams multi_deg
##' @param color_volcano color for volcano
##' @return a list with deg data.frame, volcano plot and a list with DEGs.
##' @author Xiaojie Sun
##' @importFrom patchwork wrap_plots
##' @importFrom stringr str_split
##' @importFrom dplyr union_all
##' @importFrom patchwork plot_layout
##' @export
##' @examples
##' \dontrun{
##' if(requireNamespace("Biobase",quietly = TRUE)&
##'    requireNamespace("AnnoProbe",quietly = TRUE)){
##' gse = "GSE474"
##' geo = geo_download(gse,destdir=tempdir())
##' geo$exp[1:4,1:4]
##' geo$exp=log2(geo$exp+1)
##' group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
##' ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
##' group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
##' find_anno(geo$gpl)
##' ids = AnnoProbe::idmap(geo$gpl,destdir = tempdir())
##' dcp = multi_deg_all(geo$exp,
##' group_list,ids,adjust = FALSE,entriz = FALSE)
##' dcp[[3]]
##' }else{
##'   if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
##'     warning("Package 'AnnoProbe' needed for this function to work.
##'          Please install it by install.packages('AnnoProbe')",call. = FALSE)
##'   }
##'   if(!requireNamespace("Biobase",quietly = TRUE)) {
##'     warning("Package 'Biobase' needed for this function to work.
##'          Please install it by BiocManager::install('Biobase')",call. = FALSE)
##'   }
##' }
##' }
##' @seealso
##' \code{\link{geo_download}};\code{\link{draw_volcano}};\code{\link{draw_venn}}

multi_deg_all <- function(exp,
                          group_list,
                          ids = NULL,
                          symmetry = TRUE,
                          my_genes = NULL,
                          show_rownames = FALSE,
                          cluster_cols = TRUE,
                          color_volcano = c("#2874C5", "grey", "#f87669"),
                          pvalue_cutoff = 0.05,
                          logFC_cutoff = 1,
                          adjust = TRUE,
                          entriz = TRUE,
                          change_col = "change",
                          annotation_legend = FALSE,
                          lab = NA,
                          species = "human"
                          ) {
  deg = multi_deg(
    exp,
    group_list,
    ids,
    logFC_cutoff = logFC_cutoff,
    pvalue_cutoff = pvalue_cutoff,
    adjust = adjust,
    entriz = entriz,
    species = species
  )
  if (is.null(deg)) {
    return(NULL)
  }
  #exp = data.frame(exp)
  #exp = exp[match(deg[[1]]$probe_id,rownames(exp)),]
  cgs = get_cgs(deg)
  volcano_plot = draw_volcano2(deg,
                               pkg = 4,
                               symmetry = symmetry,
                               color = color_volcano,
                               pvalue_cutoff = pvalue_cutoff,
                               logFC_cutoff = logFC_cutoff,
                               adjust = adjust,
                               lab = lab,
                               change_col = change_col
                               )
  pca_plot = draw_pca(exp,group_list)
  heatmap = draw_heatmap2(exp,group_list,deg,my_genes,
                          show_rownames = show_rownames,
                          cluster_cols = cluster_cols,
                          annotation_legend = annotation_legend)
  x = lapply(cgs,function(x)x$diff$diffprobes)
  if (all(vapply(x, length, integer(1)) == 0L)) {
    warning("no DEGs found across contrasts; skipping venn plot.", call. = FALSE)
    venn = NULL
  } else {
    venn = draw_venn(x," ")
  }
  if(as.numeric(grDevices::dev.cur())!=1) grDevices::graphics.off()
  plotlist = list(heatmap,pca_plot,venn,volcano_plot)
  layout <- '
  AABBCC
  AABBCC
  DDDDDD
  DDDDDD
  '
  result = list(
    deg = deg,
    cgs = cgs,
    plots = if (is.null(heatmap)) {
      .tinyarray_bundle_plots(pca_plot, venn, volcano_plot)
    } else {
      wrap_plots(plotlist) +
        plot_layout(design = layout) +
        plot_layout(guides = 'collect')
    }
  )
  diffprobes = lapply(cgs,function(x)x$diff$diffprobes)
  message(paste0(length(union_all(diffprobes))," DEGs in all,",length(intersect_all(diffprobes))," DEGs in common."))
  return(result)
  }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{trans_ensembl_exp}
\alias{trans_ensembl_exp}
\title{trans_ensembl_exp}
\usage{
trans_ensembl_exp(
  exp,
  mrna_only = FALSE,
  lncrna_only = FALSE,
  species = "human",
  average = FALSE
)
}
\arguments{
\item{exp}{expression matrix with Ensembl IDs as rownames}

\item{mrna_only}{only keep mrna rows in result}

\item{lncrna_only}{only keep lncrna rows in result}

\item{species}{choose human, mouse, or rat; default is human}

\item{average}{logical, if TRUE, average rows that map to the same target
symbol; if FALSE, keep the first matched row for each symbol.}
}
\value{
a transformed expression set with symbol
}
\description{
Transform rownames of an expression set from Ensembl IDs to gene symbols
using the latest gene annotation.
}
\examples{
\dontshow{if (requireNamespace("AnnotationDbi", quietly = TRUE) && requireNamespace("org.Hs.eg.db", quietly = TRUE)) withAutoprint(\{ # examplesIf}
exp = matrix(rnorm(1000),ncol = 10)
rownames(exp) = sample(mRNA_anno$gene_id,100)
colnames(exp) = c(paste0("TCGA",1:5),paste0("GTEX",1:5))
k  = trans_ensembl_exp(exp)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{trans_exp_new}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/11_surv_box_plot.R
\name{exp_boxplot}
\alias{exp_boxplot}
\title{exp_boxplot}
\usage{
exp_boxplot(
  exp_hub,
  xlab = NULL,
  ylab = NULL,
  p.label = FALSE,
  color = c("#2fa1dd", "#f87669")
)
}
\arguments{
\item{exp_hub}{an expression matrix for hubgenes}

\item{xlab}{x axis title}

\item{ylab}{y axis title}

\item{p.label}{whether to show the exact p-value, if FALSE show significance
symbols}

\item{color}{color for boxplot}
}
\value{
box plots list for all genes in the matrix
}
\description{
draw box plot for a hub gene expression matrix
}
\examples{
\dontshow{if (requireNamespace("ggpubr",quietly = TRUE)) withAutoprint(\{ # examplesIf}
k = exp_boxplot(log2(exp_hub1+1));k[[1]]
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{exp_surv}};\code{\link{box_surv}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6_multi_deg_all.R
\name{plot_deg}
\alias{plot_deg}
\title{plot_deg}
\usage{
plot_deg(
  exp,
  group_list,
  deg,
  symmetry = TRUE,
  my_genes = NULL,
  show_rownames = FALSE,
  cluster_cols = TRUE,
  color_volcano = c("#2874C5", "grey", "#f87669"),
  pvalue_cutoff = 0.05,
  logFC_cutoff = 1,
  adjust = TRUE,
  change_col = "change",
  annotation_legend = FALSE,
  lab = NA,
  species = "human"
)
}
\arguments{
\item{exp}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{deg}{result of multi_deg or get_deg function}

\item{symmetry}{a logical value ,would you like to get your plot symmetrical}

\item{my_genes}{genes for pheatmap}

\item{show_rownames}{boolean specifying if column names are be shown.}

\item{cluster_cols}{boolean values determining if columns should be clustered or \code{hclust} object.}

\item{color_volcano}{color for volcano}

\item{pvalue_cutoff}{Cutoff value of pvalue,0.05 by default.}

\item{logFC_cutoff}{Cutoff value of logFC,1 by default.}

\item{adjust}{a logical value; if TRUE, use \code{padj} and plot
\code{-log10(padj)}; otherwise use \code{P.value} and plot
\code{-log10(P.value)}.}

\item{change_col}{column name for the up/down/status label. Default uses
`change` when it exists; otherwise the function falls back to the
existing automatic classification logic.}

\item{annotation_legend}{boolean value showing if the legend for annotation 
tracks should be drawn.}

\item{lab}{label for  x axis in volcano plot; when `NA`, pkg = 4 uses
`log2FoldChange`, and the other package codes keep their original labels}

\item{species}{choose human, mouse, or rat; default is human}
}
\value{
plots
}
\description{
Plot PCA, volcano, heatmap, and Venn plots for differential analysis
results.
}
\examples{
\dontrun{
if(requireNamespace("Biobase",quietly = TRUE)&
   requireNamespace("AnnoProbe",quietly = TRUE)){
gse = "GSE474"
geo = geo_download(gse,destdir=tempdir())
geo$exp[1:4,1:4]
geo$exp=log2(geo$exp+1)
group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
find_anno(geo$gpl)
ids = AnnoProbe::idmap(geo$gpl,destdir = tempdir())
deg = get_deg(geo$exp,group_list,ids,adjust = FALSE,entriz = FALSE)
plot_deg(geo$exp,group_list,deg)
}else{
  if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
    warning("Package 'AnnoProbe' needed for this function to work.
         Please install it by install.packages('AnnoProbe')",call. = FALSE)
  }
  if(!requireNamespace("Biobase",quietly = TRUE)) {
    warning("Package 'Biobase' needed for this function to work.
         Please install it by BiocManager::install('Biobase')",call. = FALSE)
  }
}
}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/11_surv_box_plot.R
\name{box_surv}
\alias{box_surv}
\title{box_surv}
\usage{
box_surv(
  exp_hub,
  exprSet_hub,
  meta,
  xlab = "Group",
  ylab = "Expression",
  p.label = FALSE
)
}
\arguments{
\item{exp_hub}{an expression matrix for hubgenes}

\item{exprSet_hub}{a tumor expression set for hubgenes}

\item{meta}{meta data corresponds to expression set}

\item{xlab}{x axis title for the boxplot}

\item{ylab}{y axis title for the boxplot}

\item{p.label}{whether to show the exact p-value, if FALSE show significance
symbols}
}
\value{
patchwork result for hub genes boxplot and survival plot
}
\description{
draw box plot for a hub gene expression matrix
}
\examples{
\dontshow{if (requireNamespace("survival", quietly = TRUE) && requireNamespace("ggpubr", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)) withAutoprint(\{ # examplesIf}
k = box_surv(log2(exp_hub1+1),exprSet_hub1,meta1);k[[1]]
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{exp_boxplot}};\code{\link{exp_surv}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/12_cor_and_hyper.R
\name{hypertest}
\alias{hypertest}
\title{hypertest}
\usage{
hypertest(lnc, pc, deMIR = NULL, lnctarget, pctarget)
}
\arguments{
\item{lnc}{lncRNA names}

\item{pc}{mRNA names}

\item{deMIR}{miRNA names , default NULL}

\item{lnctarget}{a data.frame with two column,lncRNA in the first column ,miRNA in the second column}

\item{pctarget}{a data.frame with two column,mRNA in the first column ,miRNA in the second column}
}
\value{
a data.frame with hypertest result
}
\description{
make hypertest for given lncRNA and mRNA common miRNAs
}
\examples{
# to update
}
\seealso{
\code{\link{plcortest}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/10_survival_cut_cox_km.R
\name{point_cut}
\alias{point_cut}
\title{point_cut}
\usage{
point_cut(exprSet_hub, meta)
}
\arguments{
\item{exprSet_hub}{a tumor expression set for hubgenes}

\item{meta}{meta data corresponds to expression set}
}
\value{
a vector with cutpoint for genes
}
\description{
calculate cut point for multiple genes
}
\examples{
\dontshow{if (requireNamespace("survival", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE)) withAutoprint(\{ # examplesIf}
point_cut(exprSet_hub1,meta1)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{surv_KM}};\code{\link{surv_cox}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{ggheat}
\alias{ggheat}
\title{ggheat}
\usage{
ggheat(
  dat,
  group,
  cluster = FALSE,
  color = c("#2874C5", "white", "#f87669"),
  legend_color = c("#2874C5", "#f87669", "#e6b707", "#868686", "#66C2A5", "#FC8D62",
    "#8DA0CB", "#E78AC3", "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3"),
  show_rownames = NULL,
  show_colnames = TRUE,
  cluster_rows = FALSE,
  cluster_cols = FALSE,
  groupname = "group",
  expname = "exp",
  fill_mid = TRUE,
  rotate_rownames = NULL
)
}
\arguments{
\item{dat}{expression matrix for plot}

\item{group}{group for expression colnames}

\item{cluster}{logical,cluster in both rows and column or not, default F,now replaced by cluster_rows and cluster_cols.}

\item{color}{color for heatmap}

\item{legend_color}{color for legend}

\item{show_rownames}{logical or NULL. When NULL, the function shows x-axis
labels for up to 100 rows, rotates them 45 degrees for 9 to 100 rows, and
hides them above 100 rows.}

\item{show_colnames}{logical,show colnames in plot or not, default T}

\item{cluster_rows}{logical, if rows (on the plot) should be clustered, default F}

\item{cluster_cols}{logical, if column (on the plot) should be clustered, default F}

\item{groupname}{name of group legend}

\item{expname}{name of exp legend}

\item{fill_mid}{use median value as geom_tile fill midpoint}

\item{rotate_rownames}{logical or NULL. When NULL, the function uses the
automatic rotation rule described in \code{show_rownames}.}
}
\value{
a ggplot object
}
\description{
draw heatmap plot with annotation by ggplot2
}
\examples{
\dontshow{if (requireNamespace("ggplot2", quietly = TRUE) && requireNamespace("pheatmap", quietly = TRUE) && requireNamespace("tidyr", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)) withAutoprint(\{ # examplesIf}
exp_dat = matrix(sample(100:1000,40),ncol = 4)
exp_dat[1:(nrow(exp_dat)/2),] =  exp_dat[1:(nrow(exp_dat)/2),]-1000
rownames(exp_dat) = paste0("sample",1:nrow(exp_dat))
colnames(exp_dat) = paste0("gene",1:ncol(exp_dat))
group = rep(c("A","B"),each = nrow(exp_dat)/2)
group = factor(group,levels = c("A","B"))
ggheat(exp_dat,group)
ggheat(exp_dat,group,cluster_rows = TRUE)
ggheat(exp_dat,group,cluster_rows = TRUE,show_rownames = FALSE,
       show_colnames = FALSE,groupname = "risk",expname = "expression")
\dontshow{\}) # examplesIf}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{trans_array}
\alias{trans_array}
\title{trans_array}
\usage{
trans_array(exp, ids, from = "probe_id", to = "symbol", average = FALSE)
}
\arguments{
\item{exp}{microarray or RNA-seq expression matrix}

\item{ids}{data.frame with original rownames and new rownames}

\item{from}{colname for original rownames}

\item{to}{colname for new rownames}

\item{average}{logical, if TRUE, average rows that map to the same target
symbol; if FALSE, keep the first matched row for each symbol.}
}
\value{
a transformed expression set with new rownames
}
\description{
transform rownames for microarray or rnaseq expression matrix
}
\examples{
exp = matrix(1:50,nrow = 10)
rownames(exp) = paste0("g",1:10)
ids = data.frame(probe_id = paste0("g",1:10),
                symbol = paste0("G",c(1:9,9)))
trans_array(exp,ids)
trans_array(exp,ids,average = TRUE)
}
\seealso{
\code{\link{trans_exp}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exp_hub1}
\alias{exp_hub1}
\title{exp_hub1}
\format{
An object of class \code{matrix} (inherits from \code{array}) with 8 rows and 350 columns.
}
\usage{
exp_hub1
}
\description{
Example TCGA+GTEx expression matrix.
}
\examples{
  exp_hub1[1:4,1:4]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{draw_boxplot}
\alias{draw_boxplot}
\title{draw boxplot for expression}
\usage{
draw_boxplot(
  exp,
  group_list,
  method = "kruskal.test",
  sort = TRUE,
  drop = FALSE,
  width = 0.5,
  pvalue_cutoff = 0.05,
  xlab = "Gene",
  ylab = "Expression",
  grouplab = "Group",
  p.label = FALSE,
  add_error_bar = FALSE,
  color = c("#2874C5", "#f87669", "#e6b707", "#868686", "#66C2A5", "#FC8D62", "#8DA0CB",
    "#E78AC3", "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3"),
  ...
)
}
\arguments{
\item{exp}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{method}{one of kruskal.test,aov,t.test and wilcox.test}

\item{sort}{whether the boxplot will be sorted}

\item{drop}{whether to discard insignificant values}

\item{width}{width of boxplot and error bar}

\item{pvalue_cutoff}{if drop = TRUE,genes with p-values below the threshold will be drawn}

\item{xlab}{title of the x axis}

\item{ylab}{title of the y axis}

\item{grouplab}{title of group legend}

\item{p.label}{whether to show the exact p value in the plot; if
`FALSE`, show significance stars}

\item{add_error_bar}{whether to add error bar}

\item{color}{color vector}

\item{...}{other parameters from stat_compare_means}
}
\value{
a boxplot according to \code{exp} and grouped by \code{group}.
}
\description{
draw boxplot for expression
}
\examples{
\dontshow{if (requireNamespace("tidyr", quietly = TRUE) && requireNamespace("ggpubr", quietly = TRUE)) withAutoprint(\{ # examplesIf}
draw_boxplot(t(iris[,1:4]),iris$Species)
exp <-  matrix(rnorm(60),nrow = 10)
colnames(exp) <- paste0("sample",1:6)
rownames(exp) <- paste0("gene",1:10)
exp[,4:6] = exp[,4:6] +10
exp[1:4,1:4]
group_list <- factor(rep(c("A","B"),each = 3))
draw_boxplot(exp,group_list)
draw_boxplot(exp,group_list,color = c("grey","red"))
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{draw_heatmap}};\code{\link{draw_volcano}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/11_surv_box_plot.R
\name{risk_plot}
\alias{risk_plot}
\title{risk_plot}
\usage{
risk_plot(
  exprSet_hub,
  meta,
  riskscore,
  cut.point = FALSE,
  color = c("#2fa1dd", "#f87669"),
  n_cutoff = 3
)
}
\arguments{
\item{exprSet_hub}{a tumor expression set for hubgenes}

\item{meta}{meta data corresponds to expression set}

\item{riskscore}{a numeric vector of riskscore}

\item{cut.point}{logical, whether to use \code{cut_point}; if FALSE, use
the median as the cutoff.}

\item{color}{color for boxplot}

\item{n_cutoff}{numeric cutoff used after scaling; values outside
\code{[-n_cutoff, n_cutoff]} are truncated before plotting.}
}
\value{
risk plot
}
\description{
draw risk plot
}
\examples{
\dontshow{if (requireNamespace("survival", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE) && requireNamespace("pheatmap", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)) withAutoprint(\{ # examplesIf}
risk_plot(exprSet_hub1,meta1,riskscore = rnorm(nrow(meta1)))
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{exp_boxplot}};\code{\link{box_surv}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{meta1}
\alias{meta1}
\title{meta1}
\format{
An object of class \code{data.frame} with 177 rows and 4 columns.
}
\usage{
meta1
}
\description{
Example clinical table matched to exprSet_hub1.
}
\examples{
  head(meta1)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/10_survival_cut_cox_km.R
\name{surv_KM}
\alias{surv_KM}
\title{surv_KM}
\usage{
surv_KM(
  exprSet_hub,
  meta,
  cut.point = FALSE,
  pvalue_cutoff = 0.05,
  min_gn = 0.1
)
}
\arguments{
\item{exprSet_hub}{a tumor expression set for hubgenes}

\item{meta}{meta data corresponds to expression set}

\item{cut.point}{logical, whether to use \code{cut_point}; if FALSE, use
the median as the cutoff.}

\item{pvalue_cutoff}{p value cut off ,0.05 by defult}

\item{min_gn}{Depending on the expression of a gene, there may be a large
difference in sample counts between the two groups. If the smaller group
contains less than 10 percent of all samples, the gene will be discarded.}
}
\value{
a vector with gene names and log_rank p value
}
\description{
calculate log_rank test p values for genes
}
\examples{
\dontshow{if (requireNamespace("survival", quietly = TRUE)) withAutoprint(\{ # examplesIf}
surv_KM(exprSet_hub1,meta1)
surv_KM(exprSet_hub1,meta1,pvalue_cutoff = 1)
surv_KM(exprSet_hub1,meta1,cut.point = TRUE)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{point_cut}};\code{\link{surv_cox}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7_quick_double_enrich.R
\name{double_enrich}
\alias{double_enrich}
\title{draw enrichment bar plots for both up and down genes}
\usage{
double_enrich(deg, n = 10, color = c("#2874C5", "#f87669"), species = "human")
}
\arguments{
\item{deg}{a data.frame contains at least two columns:"ENTREZID" and "change"}

\item{n}{how many terms will you perform for up and down genes respectively}

\item{color}{color for bar plot}

\item{species}{choose human, mouse, or rat; default is human}
}
\value{
a list with kegg and go bar plot according to up and down genes enrichment result.
}
\description{
draw enrichment bar plots for both up and down genes,for human only.
}
\examples{
\dontrun{
deg = data.frame(
  ENTREZID = genes[1:20],
  change = rep(c("up", "down"), 10)
)
double_enrich(deg)
}
}
\seealso{
\code{\link{quick_enrich}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{deseq_data}
\alias{deseq_data}
\title{deseq_data}
\format{
An object of class \code{data.frame} with 552 rows and 6 columns.
}
\usage{
deseq_data
}
\description{
Example DESeq2 differential analysis result.
}
\examples{
  head(deseq_data)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/9_cor_t_choose.R
\name{cor.one}
\alias{cor.one}
\title{cor.test for one variable with all variables}
\usage{
cor.one(
  x,
  var,
  drop.var = min(x[, var], na.rm = TRUE) - 0.001,
  drop.other = min(x[, -which(colnames(x) == var)], na.rm = TRUE) - 0.001,
  min.obs = 10
)
}
\arguments{
\item{x}{A numeric matrix or data.frame}

\item{var}{your chosen variable,only one.}

\item{drop.var}{drop values in var}

\item{drop.other}{drop values in other columns}

\item{min.obs}{minimum number of observations  after dropping}
}
\value{
A data.frame with cor.test p.value and estimate
}
\description{
cor.test for all variables(each two columns)
}
\examples{
x = iris[,-5]
cor.one(x,"Sepal.Width")
}
\seealso{
\code{\link{cor.full}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{get_gpl_txt}
\alias{get_gpl_txt}
\title{get gpl txt from GEO}
\usage{
get_gpl_txt(gpl, destdir = getwd(), download = FALSE)
}
\arguments{
\item{gpl}{gpl accession from GEO database}

\item{destdir}{The destination directory for data downloads}

\item{download}{download the txt file or not}
}
\value{
No return value. When `download = TRUE`, the GPL annotation file is
  saved in `destdir`.
}
\description{
get gpl annotation txt file from GEO database
}
\examples{
get_gpl_txt("GPL23270",destdir = tempdir())
}
\seealso{
\code{\link{geo_download}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{match_exp_cl}
\alias{match_exp_cl}
\title{match_exp_cl}
\usage{
match_exp_cl(exp, cl, id_column = "id", sample_centric = TRUE)
}
\arguments{
\item{exp}{TCGA  expression set}

\item{cl}{TCGA clinical data.frame}

\item{id_column}{which column contains patient IDs, either a column number or
column name}

\item{sample_centric}{logical, default TRUE; keep all samples from the same
patients. If FALSE, keep only one tumor sample for each patient.}
}
\value{
a transformed clinical data.frame with sample ids.
}
\description{
match exp and clinical data from TCGA
}
\examples{
a = match_exp_cl(exp_hub1,meta1[,2:4],"X_PATIENT")
exp_matched = a[[1]]
cl_matched = a[[2]]
b = match_exp_cl(exp_hub1,meta1[,2:4],"X_PATIENT",sample_centric = FALSE)
exp_matched = b[[1]]
cl_matched = b[[2]]
}
\seealso{
\code{\link{make_tcga_group}};\code{\link{sam_filter}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6_multi_deg_all.R
\name{get_cgs}
\alias{get_cgs}
\title{get_cgs}
\usage{
get_cgs(deg)
}
\arguments{
\item{deg}{a data.frame created by Differential analysis}
}
\value{
a list with upgenes,downgenes,diffgenes.
}
\description{
extract DEGs from deg data.frame
}
\examples{
\dontrun{
#two group
gse = "GSE42872"
geo = geo_download(gse,destdir=tempdir())
group_list = rep(c("A","B"),each = 3)
ids = AnnoProbe::idmap('GPL6244',destdir=tempdir())
deg = get_deg(geo$exp,group_list,ids)
cgs = get_cgs(deg)
#mutigroup
gse = "GSE474"
geo = geo_download(gse,destdir=tempdir())
geo$exp[1:4,1:4]
geo$exp=log2(geo$exp+1)
group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
find_anno(geo$gpl)
ids = AnnoProbe::idmap(geo$gpl,destdir = tempdir())
deg = multi_deg(geo$exp,group_list,ids,adjust = FALSE)
cgs = get_cgs(deg)
}
}
\seealso{
\code{\link{geo_download}};\code{\link{draw_volcano}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mRNA_anno}
\alias{mRNA_anno}
\title{mRNA_anno}
\format{
An object of class \code{data.frame} with 19797 rows and 2 columns.
}
\usage{
mRNA_anno
}
\description{
Compact mRNA annotation table for TCGA expression matrices.
}
\details{
Only the columns needed by the package are kept, so the bundled object stays
small and can be used offline without extra annotation files.
}
\examples{
  head(mRNA_anno)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{geo_parser}
\alias{geo_parser}
\title{geo_parser}
\usage{
geo_parser(gse, destdir = getwd())
}
\arguments{
\item{gse}{GEO accession number}

\item{destdir}{The destination directory for data downloads}
}
\value{
an ExpressionSet object list
}
\description{
Legacy helper for parsing GEO series matrix files.
Prefer [geo_download()] for normal use.
}
\examples{
\dontrun{
gse = "GSE42872"
a = try(geo_parser(gse,destdir=tempdir()), silent = TRUE)
}
}
\seealso{
\code{\link{find_anno}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{draw_KM}
\alias{draw_KM}
\title{draw_KM}
\usage{
draw_KM(
  meta,
  group_list,
  time_col = "time",
  event_col = "event",
  legend.title = "Group",
  legend.labs = NULL,
  color = c("#2874C5", "#f87669", "#e6b707", "#868686", "#92C5DE", "#F4A582", "#66C2A5",
    "#FC8D62", "#8DA0CB", "#E78AC3", "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3"),
  ...
)
}
\arguments{
\item{meta}{survival data with time and event column}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{time_col}{colname of time}

\item{event_col}{colname of event}

\item{legend.title}{legend title}

\item{legend.labs}{character vector specifying legend labels}

\item{color}{color vector}

\item{...}{other parameters from ggsurvplot}
}
\value{
a KM-plot
}
\description{
draw KM-plot with two or more group
}
\examples{
\dontshow{if (requireNamespace("survival", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE)) withAutoprint(\{ # examplesIf}
x = survival::lung
draw_KM(meta = x,
       group_list = x$sex,event_col = "status")
\dontshow{\}) # examplesIf}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{make_tcga_group}
\alias{make_tcga_group}
\title{make_tcga_group}
\usage{
make_tcga_group(exp)
}
\arguments{
\item{exp}{sample names or an expression matrix/data.frame with sample
names in its columns}
}
\value{
a group factor with normal and tumor, corresponding to the sample
  names
}
\description{
make tcga group for given tcga sample names or expression matrix
}
\examples{
k = make_tcga_group(exp_hub1);table(k)
k2 = make_tcga_group(colnames(exp_hub1));table(k2)
}
\seealso{
\code{\link{sam_filter}};\code{\link{match_exp_cl}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3_get_deg.R
\name{get_deg}
\alias{get_deg}
\title{get_deg}
\usage{
get_deg(
  exp,
  group_list,
  ids = NULL,
  logFC_cutoff = 1,
  pvalue_cutoff = 0.05,
  adjust = TRUE,
  entriz = TRUE,
  species = "human"
)
}
\arguments{
\item{exp}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{ids}{optional probe-to-symbol annotation table. If `NULL`, `exp` is
treated as a gene-level matrix and no ID conversion is performed.}

\item{logFC_cutoff}{Cutoff value of logFC,1 by default.}

\item{pvalue_cutoff}{Cutoff value of pvalue,0.05 by default.}

\item{adjust}{a logical value; if TRUE, use \code{padj} and plot
\code{-log10(padj)}; otherwise use \code{P.value} and plot
\code{-log10(P.value)}.}

\item{entriz}{whether to convert symbols to Entrez IDs}

\item{species}{choose human, mouse, or rat; default is human}
}
\value{
a deg data.frame
}
\description{
Do differential analysis according to an expression set and group
information.
}
\examples{
\dontrun{
if(requireNamespace("Biobase",quietly = TRUE)&
   requireNamespace("AnnoProbe",quietly = TRUE)){
  gse = "GSE42872"
  a = geo_download(gse,destdir=tempdir())
  find_anno(geo$gpl)
  ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
  Group = rep(c("control","treat"),each = 3)
  Group = factor(Group)
  deg = get_deg(geo$exp,Group,ids,entriz = FALSE)
  head(deg)
}else{
  if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
    warning("Package 'AnnoProbe' needed for this function to work.
         Please install it by install.packages('AnnoProbe')",call. = FALSE)
  }
  if(!requireNamespace("Biobase",quietly = TRUE)) {
    warning("Package 'Biobase' needed for this function to work.
         Please install it by BiocManager::install('Biobase')",call. = FALSE)
  }
}
}
}
\seealso{
\code{\link{multi_deg}};\code{\link{get_deg_all}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_dumd_all.R
\name{intersect_all}
\alias{intersect_all}
\title{intersect_all}
\usage{
intersect_all(...)
}
\arguments{
\item{...}{some vectors or a list with some vectors}
}
\value{
vector
}
\description{
calculate intersect  set for two or more elements
}
\examples{
x1 = letters[1:4]
x2 = letters[3:6]
x3 = letters[3:4]
re =intersect_all(x1,x2,x3)
re2 = intersect_all(list(x1,x2,x3))
re3 = union_all(x1,x2,x3)
}
\seealso{
\code{\link{union_all}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5_get_deg_all.R
\name{get_deg_all}
\alias{get_deg_all}
\title{get_deg_all}
\usage{
get_deg_all(
  exp,
  group_list,
  ids = NULL,
  symmetry = TRUE,
  my_genes = NULL,
  show_rownames = FALSE,
  cluster_cols = TRUE,
  color_volcano = c("#2874C5", "grey", "#f87669"),
  logFC_cutoff = 1,
  pvalue_cutoff = 0.05,
  adjust = TRUE,
  entriz = TRUE,
  n_cutoff = 2,
  annotation_legend = FALSE,
  lab = NA,
  change_col = "change",
  species = "human"
)
}
\arguments{
\item{exp}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{ids}{optional probe-to-symbol annotation table. If `NULL`, `exp` is
treated as a gene-level matrix and no ID conversion is performed.}

\item{symmetry}{a logical value ,would you like to get your plot symmetrical}

\item{my_genes}{genes for pheatmap}

\item{show_rownames}{logical,show rownames or not}

\item{cluster_cols}{boolean values determining if columns should be clustered or \code{hclust} object.}

\item{color_volcano}{color for volcano}

\item{logFC_cutoff}{Cutoff value of logFC,1 by default.}

\item{pvalue_cutoff}{Cutoff value of pvalue,0.05 by default.}

\item{adjust}{a logical value; if TRUE, use \code{padj} and plot
\code{-log10(padj)}; otherwise use \code{P.value} and plot
\code{-log10(P.value)}.}

\item{entriz}{logical, if TRUE, convert symbols to Entrez IDs.}

\item{n_cutoff}{numeric cutoff used after scaling; values outside
\code{[-n_cutoff, n_cutoff]} are truncated before plotting.}

\item{annotation_legend}{logical,show annotation legend or not}

\item{lab}{label for  x axis in volcano plot; when `NA`, pkg = 4 uses
`log2FoldChange`, and the other package codes keep their original labels}

\item{change_col}{column name for the up/down/status label. Default uses
`change` when it exists; otherwise the function falls back to the
existing automatic classification logic.}

\item{species}{choose human, mouse, or rat; default is human}
}
\value{
a list with a deg data.frame, volcano plot, PCA plot, heatmap, and
  a list with DEGs.
}
\description{
Do differential analysis according to an expression set and group
information.
}
\examples{
\dontrun{
if(requireNamespace("Biobase",quietly = TRUE)&
   requireNamespace("AnnoProbe",quietly = TRUE)){
  gse = "GSE42872"
  geo = geo_download(gse,destdir=tempdir())
  group_list = rep(c("A","B"),each = 3)
  group_list = factor(group_list)
  find_anno(geo$gpl)
  ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
  dcp = get_deg_all(geo$exp,group_list,ids,entriz = FALSE)
  head(dcp$deg)
  dcp$plots
}else{
  if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
    warning("Package 'AnnoProbe' needed for this function to work.
         Please install it by install.packages('AnnoProbe')",call. = FALSE)
  }
  if(!requireNamespace("Biobase",quietly = TRUE)) {
    warning("Package 'Biobase' needed for this function to work.
         Please install it by BiocManager::install('Biobase')",call. = FALSE)
  }
}
}
}
\seealso{
\code{\link{get_deg}};\code{\link{multi_deg_all}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_auto_geo_group.R
\name{print.geo_auto_group}
\alias{print.geo_auto_group}
\title{Print a GEO auto-group result}
\usage{
\method{print}{geo_auto_group}(x, ...)
}
\arguments{
\item{x}{An object returned by `auto_geo_group()`.}

\item{...}{Additional arguments passed to `print()`.}
}
\description{
Print a GEO auto-group result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{trans_exp}
\alias{trans_exp}
\title{trans_exp}
\usage{
trans_exp(exp, mrna_only = FALSE, lncrna_only = FALSE, gtex = FALSE)
}
\arguments{
\item{exp}{expression matrix with Ensembl IDs as rownames}

\item{mrna_only}{only keep mrna rows in result}

\item{lncrna_only}{only keep lncrna rows in result}

\item{gtex}{logical, whether to include GTEx data}
}
\value{
a transformed expression set with symbol
}
\description{
Transform rownames of an expression matrix from Ensembl IDs to gene
symbols.
}
\examples{
exp = matrix(rnorm(1000),ncol = 10)
rownames(exp) = sample(mRNA_anno$gene_id,100)
colnames(exp) = paste0("sample",1:10)
k  = trans_exp(exp)
}
\seealso{
\code{\link{trans_array}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{get_ids}
\alias{get_ids}
\title{Get probe-to-symbol ids for a GEO platform accession.}
\usage{
get_ids(gpl, type = NULL, install = FALSE, update = FALSE)
}
\arguments{
\item{gpl}{a GEO platform accession}

\item{type}{optional AnnoProbe `type` setting. If omitted, the function
tries candidate types automatically.}

\item{install}{whether to install and library the package}

\item{update}{whether to update the package}
}
\value{
a cleaned two-column data.frame with `probe_id` and `symbol`
}
\description{
Prefer the available Bioconductor annotation package when possible, and
fall back to `AnnoProbe::idmap()` when no Bioconductor annotation package
source exists. The function prints a short explanation plus the code it
used.
}
\examples{
\dontrun{
get_ids("GPL570")
}
}
\seealso{
\code{\link{find_anno}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{draw_tsne}
\alias{draw_tsne}
\title{draw_tsne}
\usage{
draw_tsne(
  exp,
  group_list,
  perplexity = 30,
  color = c("#2874C5", "#f87669", "#e6b707", "#868686", "#92C5DE", "#F4A582", "#66C2A5",
    "#FC8D62", "#8DA0CB", "#E78AC3", "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3"),
  color.label = "group",
  addEllipses = TRUE
)
}
\arguments{
\item{exp}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{perplexity}{numeric; perplexity parameter for Rtsne}

\item{color}{color vector}

\item{color.label}{color legend label}

\item{addEllipses}{logical,add ellipses or not}
}
\value{
a ggplot object
}
\description{
draw tsne plot with annotation by ggplot2
}
\examples{
\dontshow{if (requireNamespace("Rtsne", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)) withAutoprint(\{ # examplesIf}
exp <-  matrix(rnorm(10000),nrow = 50)
colnames(exp) <- paste0("sample",1:200)
rownames(exp) <- paste0("gene",1:50)
exp[1:4,1:4]
exp[,1:100] = exp[,1:100]+10
group_list <- factor(rep(c("A","B"),each = 100))
draw_tsne(exp,group_list)
\dontshow{\}) # examplesIf}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cod}
\alias{cod}
\title{cod}
\format{
An object of class \code{matrix} (inherits from \code{array}) with 100 rows and 512 columns.
}
\usage{
cod
}
\description{
Example TCGA expression matrix used in grouping and plotting demos.
}
\examples{
  cod
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7_quick_double_enrich.R
\name{quick_enrich}
\alias{quick_enrich}
\title{quick_enrich}
\usage{
quick_enrich(genes, kkgo_file = NULL, destdir = getwd(), species = "human")
}
\arguments{
\item{genes}{a gene symbol or Entrez ID vector}

\item{kkgo_file}{optional RDS cache filename for KEGG and GO results;
when `NULL`, the function recomputes on every call and does not write a cache}

\item{destdir}{destination directory used to save the cache file when
`kkgo_file` is provided}

\item{species}{choose human, mouse, or rat; default is human}
}
\value{
enrichment results and dotplots
}
\description{
Perform enrichment analysis for a gene list; human is the default species.
}
\examples{
\dontrun{
head(genes)
g = quick_enrich(genes,destdir = tempdir())
names(g)
g$kk
g$go
}
}
\seealso{
\code{\link{double_enrich}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{sam_filter}
\alias{sam_filter}
\title{sam_filter}
\usage{
sam_filter(exp)
}
\arguments{
\item{exp}{TCGA or TCGA+GTEx expression matrix from GDC or Xena}
}
\value{
a transformed expression set without duplicated samples
}
\description{
drop duplicated samples from the same patients
}
\examples{
exp_hub1[1:4,1:4]
dim(exp_hub1)
exp2 = sam_filter(exp_hub1)
dim(exp2)
g = make_tcga_group(exp_hub1);table(g)
table(!duplicated(substr(colnames(exp_hub1[,g=="tumor"]),1,12)))
}
\seealso{
\code{\link{make_tcga_group}};\code{\link{match_exp_cl}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exists_anno_list}
\alias{exists_anno_list}
\title{exists_anno_list}
\format{
An object of class \code{character} of length 175.
}
\usage{
exists_anno_list
}
\description{
GEO GPL platforms with direct AnnoProbe annotation support.
}
\examples{
  exists_anno_list
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{draw_pca}
\alias{draw_pca}
\title{draw PCA plots}
\usage{
draw_pca(
  exp,
  group_list,
  color = c("#2874C5", "#f87669", "#e6b707", "#868686", "#92C5DE", "#F4A582", "#66C2A5",
    "#FC8D62", "#8DA0CB", "#E78AC3", "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3"),
  addEllipses = TRUE,
  style = "default",
  color.label = "Group",
  title = "",
  ...
)
}
\arguments{
\item{exp}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{color}{color vector}

\item{addEllipses}{logical,add ellipses or not}

\item{style}{plot style,"default","ggplot2"and "3D"}

\item{color.label}{color legend label}

\item{title}{plot title}

\item{...}{other parameters from \code{fviz_pca_ind}}
}
\value{
a PCA plot according to \code{exp} and grouped by \code{group_list}.
}
\description{
do PCA analysis and warning a PCA plot
}
\examples{
\dontshow{if (requireNamespace("FactoMineR", quietly = TRUE) && requireNamespace("factoextra", quietly = TRUE)) withAutoprint(\{ # examplesIf}
draw_pca(t(iris[,1:4]),iris$Species)
#change color
draw_pca(t(iris[,1:4]),iris$Species,color = c("#E78AC3", "#A6D854", "#FFD92F"))
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("FactoMineR", quietly = TRUE) && requireNamespace("ggplot2", quietly = TRUE)) withAutoprint(\{ # examplesIf}
draw_pca(t(iris[,1:4]),iris$Species,style = "ggplot2")
\dontshow{\}) # examplesIf}
\dontshow{if (requireNamespace("FactoMineR", quietly = TRUE) && requireNamespace("scatterplot3d", quietly = TRUE)) withAutoprint(\{ # examplesIf}
draw_pca(t(iris[,1:4]),iris$Species,style = "3D")
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{draw_heatmap}};\code{\link{draw_volcano}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/9_cor_t_choose.R
\name{cor.full}
\alias{cor.full}
\title{cor.test for all variables}
\usage{
cor.full(x, drop = min(x, na.rm = TRUE) - 0.001, min.obs = 10)
}
\arguments{
\item{x}{A numeric matrix or data.frame}

\item{drop}{drop values}

\item{min.obs}{minimum number of observations  after dropping}
}
\value{
a data.frame with cor.test p.value and estimate
}
\description{
cor.test for all variables(each two columns)
}
\examples{
x = iris[,-5]
cor.full(x)
}
\seealso{
\code{\link{cor.one}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{get_count_txt}
\alias{get_count_txt}
\title{get count from GEO}
\usage{
get_count_txt(gse, destdir = getwd(), download = FALSE)
}
\arguments{
\item{gse}{GEO accession number or a path to a cached `.rds` or `.RData`
file containing an `ExpressionSet` or a list of `ExpressionSet` objects}

\item{destdir}{The destination directory for data downloads}

\item{download}{download the txt file or not}
}
\value{
No return value. When `download = TRUE`, the count file is saved in
  `destdir`.
}
\description{
get RNA-seq count file from GEO database
}
\examples{
get_count_txt("GSE162550",destdir = tempdir())
}
\seealso{
\code{\link{geo_download}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6_multi_deg_all.R
\name{draw_heatmap2}
\alias{draw_heatmap2}
\title{draw heatmap plots}
\usage{
draw_heatmap2(exp, group_list, deg, my_genes = NULL, heat_union = TRUE, ...)
}
\arguments{
\item{exp}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{deg}{a data.frame created by Differential analysis}

\item{my_genes}{genes for pheatmap}

\item{heat_union}{logical ,use union or intersect DEGs for heatmap}

\item{...}{other parameters from draw_heatmap}
}
\value{
a heatmap plot according to \code{exp} and grouped by
  \code{group_list}.
}
\description{
Print heatmap plots for an expression matrix grouped by \code{group_list}.
}
\examples{
\dontrun{
if(requireNamespace("Biobase",quietly = TRUE)&
   requireNamespace("AnnoProbe",quietly = TRUE)){
  gse = "GSE474"
  geo = geo_download(gse,destdir=tempdir())
  geo$exp[1:4,1:4]
  geo$exp=log2(geo$exp+1)
  group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
  ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
  group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
  find_anno(geo$gpl)
  ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
  deg = multi_deg(geo$exp,group_list,ids,adjust = FALSE,entriz = FALSE)
  draw_heatmap2(geo$exp,group_list,deg)
}else{
  if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
    warning("Package 'AnnoProbe' needed for this function to work.
         Please install it by install.packages('AnnoProbe')",call. = FALSE)
  }
  if(!requireNamespace("Biobase",quietly = TRUE)) {
    warning("Package 'Biobase' needed for this function to work.
         Please install it by BiocManager::install('Biobase')",call. = FALSE)
  }
}
}
}
\seealso{
\code{\link{draw_pca}};\code{\link{draw_volcano}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{deg}
\alias{deg}
\title{deg}
\format{
An object of class \code{data.frame} with 18591 rows and 10 columns.
}
\usage{
deg
}
\description{
Example limma differential analysis result for GSE42872.
}
\examples{
  head(deg)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{find_anno}
\alias{find_anno}
\title{Find an annotation table for a GEO platform accession.}
\usage{
find_anno(gpl, install = FALSE, update = FALSE)
}
\arguments{
\item{gpl}{a GEO platform accession}

\item{install}{whether to install and library the package}

\item{update}{whether to update the package}
}
\value{
No return value. The function prints available annotation resources
  and optionally installs a Bioconductor annotation package.
}
\description{
The function prints a short explanation plus the code used to obtain
annotations. It only tests AnnoProbe `type` values when no Bioconductor
annotation package source exists for the GPL accession.
}
\examples{
\dontrun{
try(find_anno("GPL570"), silent = TRUE)
}
}
\seealso{
\code{\link{geo_download}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/13_network_edge_node.R
\name{edges_to_nodes}
\alias{edges_to_nodes}
\title{edges_to_nodes}
\usage{
edges_to_nodes(edges)
}
\arguments{
\item{edges}{data.frame}
}
\value{
nodes data.frame
}
\description{
get nodes from edges
}
\examples{
df = data.frame(a = c("gene1","gene2","gene3"),
b = c("d,f,a,b",
"c,e,g",
"a,b,d"))
edges = interaction_to_edges(df)
nodes = edges_to_nodes(edges)
}
\seealso{
\code{\link{interaction_to_edges}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/13_network_edge_node.R
\name{interaction_to_edges}
\alias{interaction_to_edges}
\title{interaction_to_edges}
\usage{
interaction_to_edges(df, a = 1, b = 2, sep = ",")
}
\arguments{
\item{df}{interactions data.frame}

\item{a}{column to replicate}

\item{b}{column to split}

\item{sep}{a character string to separate b column}
}
\value{
a new data.frame with two columns, one interaction per row
}
\description{
Split interactions by `sep` and return an edges data.frame.
}
\examples{
df = data.frame(a = c("gene1","gene2","gene3"),
b = c("d,f,a,b",
"c,e,g",
"a,b,d"))
interaction_to_edges(df)
}
\seealso{
\code{\link{edges_to_nodes}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/15_auto_geo_group.R
\name{auto_geo_group}
\alias{auto_geo_group}
\title{auto_geo_group}
\usage{
auto_geo_group(
  x,
  group_col = "title",
  choice = NULL,
  min_group_size = 2L,
  max_groups = 20L,
  min_coverage = 0.8,
  search_other_cols = TRUE,
  stopwords = .tinyarray_geo_stopwords(),
  levels = NULL,
  ref = NULL,
  verbose = TRUE
)
}
\arguments{
\item{x}{A GEO result list with a `pd` data frame, or a phenotype data frame.}

\item{group_col}{Column name to try first. The default is `title`.}

\item{choice}{Candidate index to select. Use `0` to cancel. If `NULL`, the
function returns the candidate table without selecting.}

\item{min_group_size}{Minimum number of samples required for a split label.}

\item{max_groups}{Maximum number of group labels allowed in one column.}

\item{min_coverage}{Minimum fraction of samples that must be assigned a label.}

\item{search_other_cols}{Whether to search other columns when `group_col` and
`title` fail.}

\item{stopwords}{Character vector of generic words to ignore during tokenization.}

\item{levels}{Optional character vector giving the desired factor level order.}

\item{ref}{Optional reference level to place first.}

\item{verbose}{Whether to print messages and candidate tables.}
}
\value{
A list with `group_list`, `source_col`, `selected_index`,
  `candidate_table`, `candidates`, and `pdata`.
}
\description{
Infer sample groups from GEO phenotype metadata.
}
\details{
The function first tries the user-specified `group_col`, then `title`. If no
stable grouping is found, it scans the remaining columns and returns a table
of candidate groupings for manual selection. Both binary and multigroup
factors are supported. Rare groups are left as `NA` rather than being
relabeled as another group.

Labels that differ only in capitalization are merged, retaining their most
common spelling. Shared affixes and trailing sample identifiers are removed
when they can be identified consistently.
}
\examples{
pd <- data.frame(
  title = c("Control sample", "Control sample", "Treat sample", "Treat sample"),
  stringsAsFactors = FALSE
)
rownames(pd) <- paste0("s", 1:4)
res <- auto_geo_group(
  pd,
  choice = 1,
  levels = c("Control", "Treat"),
  ref = "Control",
  verbose = FALSE
)
res$source_col
levels(res$group_list)
table(res$group_list)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{corscatterplot}
\alias{corscatterplot}
\title{corscatterplot}
\usage{
corscatterplot(
  dat,
  x,
  y,
  color_cor = "blue",
  fill_cor = "lightgray",
  fill_x = "#ff820e",
  fill_y = "#0000fe",
  type = "density",
  ...
)
}
\arguments{
\item{dat}{plot data}

\item{x}{x}

\item{y}{y}

\item{color_cor}{color for cor reg.line}

\item{fill_cor}{fill for cor reg.line}

\item{fill_x}{fill for top density plot}

\item{fill_y}{fill for right density plot}

\item{type}{whether to use a density plot or a histogram plot for the side panel.}

\item{...}{other parameters for ggscatter}
}
\value{
a ggplot object
}
\description{
draw cor scatter plot with density plot by ggplot2
}
\examples{
\dontshow{if (requireNamespace("ggpubr", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)) withAutoprint(\{ # examplesIf}
corscatterplot(iris,"Sepal.Length","Sepal.Width")
\dontshow{\}) # examplesIf}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{genes}
\alias{genes}
\title{genes}
\format{
An object of class \code{character} of length 511.
}
\usage{
genes
}
\description{
Small example vector of Entrez IDs.
}
\examples{
  genes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_geo_id.R
\name{geo_download}
\alias{geo_download}
\title{geo_download}
\usage{
geo_download(
  gse,
  by_annopbrobe = TRUE,
  simpd = TRUE,
  colon_remove = TRUE,
  download_timeout = getOption("timeout"),
  destdir = getwd(),
  n = 1
)
}
\arguments{
\item{gse}{GEO accession number or a path to a cached `.rds` or `.RData`
file containing an `ExpressionSet` or a list of `ExpressionSet` objects}

\item{by_annopbrobe}{whether to try the AnnoProbe cache first and fall back
to GEOquery::getGEO()}

\item{simpd}{whether to simplify phenotype data by dropping columns with a
single unique value}

\item{colon_remove}{whether to drop raw `characteristics_ch1*` columns
while keeping the derived `:ch1` phenotype column names intact}

\item{download_timeout}{timeout in seconds used for GEOquery fallback and
for mirror stall detection during the resumable download}

\item{destdir}{The destination directory for data downloads}

\item{n}{For data with more than one ExpressionSet, specify which one to analyze}
}
\value{
a list with `exp`, `pd`, `gpl`, and `group_candidates`, or `NULL`
  when the data cannot be read or downloaded
}
\description{
Download GEO expression data and return the expression matrix, phenotype
data, and platform information.
}
\details{
Cached `eSet` objects are stored as `.rds` files in `destdir`; legacy
`.Rdata` cache files are still readable, and incomplete cache files will be
kept so the download can resume automatically.
}
\examples{
\dontrun{
if(requireNamespace("Biobase",quietly = TRUE)){
  gse = "GSE42872"
  a = geo_download(gse,destdir=tempdir())
}
}
}
\seealso{
\code{\link{find_anno}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{lnc_anno}
\alias{lnc_anno}
\title{lnc_anno}
\format{
An object of class \code{data.frame} with 14852 rows and 2 columns.
}
\usage{
lnc_anno
}
\description{
Compact lncRNA annotation table for TCGA expression matrices.
}
\details{
Only the columns needed by the package are kept, so the bundled object stays
small and can be used offline without extra annotation files.
}
\examples{
  head(lnc_anno)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{draw_volcano}
\alias{draw_volcano}
\title{draw a volcano plot}
\usage{
draw_volcano(
  deg,
  lab = NA,
  xlab.package = TRUE,
  pvalue_cutoff = 0.05,
  logFC_cutoff = 1,
  pkg = 1,
  adjust = TRUE,
  change_col = "change",
  symmetry = FALSE,
  color = c("#2874C5", "grey", "#f87669")
)
}
\arguments{
\item{deg}{a data.frame created by Differential analysis}

\item{lab}{label for  x axis in volcano plot; when `NA`, pkg = 4 uses
`log2FoldChange`, and the other package codes keep their original labels}

\item{xlab.package}{whether to use the package name as the x axis name}

\item{pvalue_cutoff}{Cutoff value of pvalue,0.05 by default.}

\item{logFC_cutoff}{Cutoff value of logFC,1 by default.}

\item{pkg}{a integer ,means which Differential analysis packages you used,we support three packages by now, 1,2,3,4 respectively means "DESeq2","edgeR","limma(voom)","limma"; pkg = 4 uses `log2FoldChange` as the default x-axis label.}

\item{adjust}{a logical value; if TRUE, use \code{padj} and plot
\code{-log10(padj)}; otherwise use \code{P.value} and plot
\code{-log10(P.value)}.}

\item{change_col}{column name for the up/down/status label. Default uses
`change` when it exists; otherwise the function falls back to the
existing automatic classification logic.}

\item{symmetry}{a logical value ,would you like to get your plot symmetrical}

\item{color}{color vector}
}
\value{
a volcano plot according to \code{logFC} and either
  \code{-log10(P.value)} or \code{-log10(padj)}, depending on \code{adjust}.
}
\description{
warning a volcano plot for Differential analysis result in data.frame format.
}
\examples{
\dontshow{if (requireNamespace("ggplot2", quietly = TRUE)) withAutoprint(\{ # examplesIf}
head(deseq_data)
draw_volcano(deseq_data)
draw_volcano(deseq_data,pvalue_cutoff = 0.01,logFC_cutoff = 2)
draw_volcano(deseq_data,color = c("darkgreen", "darkgrey", "#B2182B"))
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{draw_heatmap}};\code{\link{draw_pca}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{pkg_all}
\alias{pkg_all}
\title{pkg_all}
\format{
An object of class \code{data.frame} with 82 rows and 3 columns.
}
\usage{
pkg_all
}
\description{
Lookup table for GEO platform annotation packages.
}
\examples{
  head(pkg_all)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exprSet_hub1}
\alias{exprSet_hub1}
\title{exprSet_hub1}
\format{
An object of class \code{matrix} (inherits from \code{array}) with 8 rows and 177 columns.
}
\usage{
exprSet_hub1
}
\description{
Example TCGA CPM matrix with tumor samples only.
}
\examples{
  exprSet_hub1[1:4,1:4]
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/9_cor_t_choose.R
\name{t_choose}
\alias{t_choose}
\title{t_choose}
\usage{
t_choose(
  genes,
  exp,
  group_list,
  up_only = FALSE,
  down_only = FALSE,
  pvalue_cutoff = 0.05
)
}
\arguments{
\item{genes}{a vector with some genes}

\item{exp}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{up_only}{keep up genes in the result only}

\item{down_only}{keep down genes in the result only}

\item{pvalue_cutoff}{p value cut off ,0.05 by defult}
}
\value{
a vector with differential expressed genes
}
\description{
choose differential expressed genes by simple t.test
}
\examples{
exp = matrix(rnorm(1000),ncol = 10)
rownames(exp) = sample(mRNA_anno$gene_id,100)
colnames(exp) = c(paste0("TCGA",1:5),paste0("GTEX",1:5))
exp2  = trans_exp(exp)
exp2[,1:5] = exp2[,1:5]+10
group_list = rep(c("A","B"),each = 5)
genes = sample(rownames(exp2),3)
t_choose(genes,exp2,group_list)
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8_tcga_group_trans_exp.R
\name{trans_entrezexp}
\alias{trans_entrezexp}
\alias{trans_entrez_exp}
\title{trans_entrezexp}
\usage{
trans_entrezexp(entrezexp, species = "human", average = FALSE)

trans_entrez_exp(entrezexp, species = "human", average = FALSE)
}
\arguments{
\item{entrezexp}{expression matrix with Entrez IDs as rownames}

\item{species}{choose human, mouse, or rat; default is human}

\item{average}{logical, if TRUE, average rows that map to the same target
symbol; if FALSE, keep the first matched row for each symbol.}
}
\value{
a transformed expression set with symbol
}
\description{
Transform rownames of an expression set from Entrez IDs to gene symbols
using \code{clusterProfiler::bitr()}.
}
\examples{
\dontshow{if (requireNamespace("clusterProfiler", quietly = TRUE) && requireNamespace("org.Hs.eg.db", quietly = TRUE)) withAutoprint(\{ # examplesIf}
exp = matrix(rnorm(200),ncol = 10)
rownames(exp) = c("79691", "56271", "8662", "10394", "55630", "159162", "23541",
                  "79723", "54413", "22927", "92342", "23787", "5550", "8924",
                  "55274", "866", "8844", "353299", "587", "1473")
colnames(exp) = paste0("s",1:10)
exp2 = trans_entrezexp(exp)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{trans_exp}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{draw_venn}
\alias{draw_venn}
\title{draw a venn plot}
\usage{
draw_venn(
  x,
  main,
  color = c("#2874C5", "#f87669", "#e6b707", "#868686", "#66C2A5", "#FC8D62", "#8DA0CB",
    "#E78AC3", "#A6D854", "#FFD92F", "#E5C494", "#B3B3B3"),
  imagetype = "png",
  filename = NULL,
  lwd = 1,
  lty = 1,
  col = color[1:length(x)],
  fill = color[1:length(x)],
  cat.col = color[1:length(x)],
  cat.cex = 1,
  cat.dist = -0.15,
  rotation.degree = 0,
  main.cex = 1,
  cex = 1,
  alpha = 0.1,
  reverse = TRUE,
  ...
)
}
\arguments{
\item{x}{a list for plot}

\item{main}{Character giving the main title of the diagram}

\item{color}{color vector}

\item{imagetype}{Specification of the image format (e.g. tiff, png or svg)}

\item{filename}{Filename for image output, or if NULL returns the grid object itself}

\item{lwd}{width of the circle's circumference}

\item{lty}{dash pattern of the circle's circumference}

\item{col}{Colour of the circle's circumference}

\item{fill}{Colour of the circle's area}

\item{cat.col}{Colour of the category name}

\item{cat.cex}{size of the category name}

\item{cat.dist}{The distance (in npc units) of the category name from the edge of the circle (can be negative)}

\item{rotation.degree}{Number of degrees to rotate the entire diagram}

\item{main.cex}{Number giving the cex (font size) of the main title}

\item{cex}{size of the area label}

\item{alpha}{Alpha transparency of the circle's area}

\item{reverse}{logical,reflect the three-set Venn diagram along its central
vertical axis of symmetry. Use in combination with rotation
to generate all possible set orders}

\item{...}{other parameters from venn.diagram}
}
\value{
a venn plot according to \code{x}, \code{y}, and \code{z} using the
  list names as labels.
}
\description{
warning a venn plot for deg result created by three packages
}
\examples{
\dontshow{if (requireNamespace("VennDiagram",quietly = TRUE) && requireNamespace("ggplotify",quietly = TRUE) && requireNamespace("cowplot",quietly = TRUE)) withAutoprint(\{ # examplesIf}
x = list(Deseq2=sample(1:100,30),edgeR = sample(1:100,30),limma = sample(1:100,30))
draw_venn(x,"test")
draw_venn(x,"test",color = c("darkgreen", "darkblue", "#B2182B"))
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{draw_pca}};\code{\link{draw_volcano}};\code{\link{draw_heatmap}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/11_surv_box_plot.R
\name{exp_surv}
\alias{exp_surv}
\title{exp_surv}
\usage{
exp_surv(exprSet_hub, meta, cut.point = FALSE, color = c("#2874C5", "#f87669"))
}
\arguments{
\item{exprSet_hub}{a tumor expression set for hubgenes}

\item{meta}{meta data corresponds to expression set}

\item{cut.point}{logical, whether to use \code{cut_point}; if FALSE, use
the median as the cutoff.}

\item{color}{color for boxplot}
}
\value{
survival plots list for all genes
}
\description{
draw surv plot for a hub gene expression matrix for tumor samples
}
\examples{
\dontshow{if (requireNamespace("survival", quietly = TRUE) && requireNamespace("survminer", quietly = TRUE) && requireNamespace("patchwork", quietly = TRUE)) withAutoprint(\{ # examplesIf}
tmp = exp_surv(exprSet_hub1,meta1)
patchwork::wrap_plots(tmp)+patchwork::plot_layout(guides = "collect")
tmp2 = exp_surv(exprSet_hub1,meta1,cut.point = TRUE)
patchwork::wrap_plots(tmp2)+patchwork::plot_layout(guides = "collect")
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{exp_boxplot}};\code{\link{box_surv}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6_multi_deg_all.R
\name{draw_volcano2}
\alias{draw_volcano2}
\title{draw_volcano2}
\usage{
draw_volcano2(deg, pkg = 4, lab, change_col = "change", ...)
}
\arguments{
\item{deg}{a data.frame created by Differential analysis}

\item{pkg}{a integer ,means which Differential analysis packages you used,we support three packages by now, 1,2,3,4 respectively means "DESeq2","edgeR","limma(voom)","limma"; pkg = 4 uses `log2FoldChange` as the default x-axis label.}

\item{lab}{label for  x axis in volcano plot; when `NA`, pkg = 4 uses
`log2FoldChange`, and the other package codes keep their original labels}

\item{change_col}{column name for the up/down/status label. Default uses
`change` when it exists; otherwise the function falls back to the
existing automatic classification logic.}

\item{...}{other parameters from draw_volcano}
}
\value{
one or more volcano plot
}
\description{
Print one or more volcano plots for differential analysis results in a
data.frame.
}
\examples{
\dontrun{
if(requireNamespace("Biobase",quietly = TRUE)&
   requireNamespace("AnnoProbe",quietly = TRUE)){
#two group
gse = "GSE42872"
geo = geo_download(gse,destdir=tempdir())
group_list = rep(c("A","B"),each = 3)
ids = AnnoProbe::idmap('GPL6244',destdir = tempdir())
deg = get_deg(geo$exp,group_list,ids)
draw_volcano2(deg)
#multigroup
gse = "GSE474"
geo = geo_download(gse,destdir=tempdir())
geo$exp[1:4,1:4]
geo$exp=log2(geo$exp+1)
group_list=ifelse(stringr::str_detect(geo$pd$title,"MObese"),"MObese",
ifelse(stringr::str_detect(geo$pd$title,"NonObese"),"NonObese","Obese"))
group_list=factor(group_list,levels = c("NonObese","Obese","MObese"))
find_anno(geo$gpl)
ids <- AnnoProbe::idmap(geo$gpl,destdir = tempdir())
deg = multi_deg(geo$exp,group_list,ids,adjust = FALSE,entriz = FALSE)
draw_volcano2(deg)
draw_volcano2(deg,color = c("darkgreen","grey","darkred"))
}else{
  if(!requireNamespace("AnnoProbe",quietly = TRUE)) {
    warning("Package 'AnnoProbe' needed for this function to work.
         Please install it by install.packages('AnnoProbe')",call. = FALSE)
  }
  if(!requireNamespace("Biobase",quietly = TRUE)) {
    warning("Package 'Biobase' needed for this function to work.
         Please install it by BiocManager::install('Biobase')",call. = FALSE)
  }
}
}
}
\seealso{
\code{\link{geo_download}};\code{\link{draw_volcano}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{draw_heatmap}
\alias{draw_heatmap}
\title{draw a heatmap plot}
\usage{
draw_heatmap(
  n,
  group_list,
  scale_before = FALSE,
  n_cutoff = 3,
  legend = FALSE,
  show_rownames = FALSE,
  annotation_legend = FALSE,
  split_column = FALSE,
  show_column_title = FALSE,
  color = (grDevices::colorRampPalette(c("#2fa1dd", "white", "#f87669")))(100),
  color_an = c("#2fa1dd", "#f87669", "#e6b707", "#868686", "#92C5DE", "#F4A582",
    "#66C2A5", "#FC8D62", "#8DA0CB", "#E78AC3", "#A6D854", "#FFD92F", "#E5C494",
    "#B3B3B3"),
  scale = TRUE,
  main = NA,
  ...
)
}
\arguments{
\item{n}{A numeric matrix}

\item{group_list}{A character or factor vector with one label per sample;
character input will be converted to factor and unused levels will be
dropped.}

\item{scale_before}{deprecated parameter}

\item{n_cutoff}{numeric cutoff used after scaling; values outside
\code{[-n_cutoff, n_cutoff]} are truncated before plotting.}

\item{legend}{logical,show legend or not}

\item{show_rownames}{logical,show rownames or not}

\item{annotation_legend}{logical,show annotation legend or not}

\item{split_column}{split column by group_list}

\item{show_column_title}{show column title or not}

\item{color}{color for heatmap}

\item{color_an}{color for column annotation}

\item{scale}{logical,scale the matrix or not}

\item{main}{the title of the plot}

\item{...}{other parameters from pheatmap}
}
\value{
a heatmap plot according to \code{n} and grouped by
  \code{group_list}.
}
\description{
draw a heatmap plot for an expression matrix grouped by \code{group_list};
if \code{scale = TRUE}, the matrix is scaled before plotting.
}
\examples{
\dontshow{if (requireNamespace("ggplotify", quietly = TRUE) && requireNamespace("pheatmap", quietly = TRUE)) withAutoprint(\{ # examplesIf}
#example data
exp = matrix(abs(rnorm(60,sd = 16)),nrow = 10)
exp[,4:6] <- exp[,4:6]+20
colnames(exp) <- paste0("sample",1:6)
rownames(exp) <- paste0("gene",1:10)
exp[1:4,1:4]
group_list = factor(rep(c("A","B"),each = 3))
draw_heatmap(exp,group_list)
#use iris
n = t(iris[,1:4]);colnames(n) = 1:150
group_list = iris$Species
draw_heatmap(n,group_list)
draw_heatmap(n,group_list,color = colorRampPalette(c("green","black","red"))(100),
             color_an = c("red","blue","pink") )
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{draw_pca}};\code{\link{draw_volcano}};\code{\link{draw_venn}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/14_dumd_all.R
\name{dumd}
\alias{dumd}
\title{count unique values in every columns for data.frame}
\usage{
dumd(x)
}
\arguments{
\item{x}{A data.frame.}
}
\value{
The simple data.frame of columns unique values count in \code{x}
}
\description{
in geo analysis,this function can help you simplify pdata, delete columns with unique values,which can't be used as group vector
}
\examples{
dumd(iris)
data(ToothGrowth)
x = ToothGrowth
dumd(ToothGrowth)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/12_cor_and_hyper.R
\name{plcortest}
\alias{plcortest}
\title{plcortest}
\usage{
plcortest(lnc_exp, mRNA_exp, cor_cutoff = 0)
}
\arguments{
\item{lnc_exp}{lncRNA expression set}

\item{mRNA_exp}{mRNA expression set which nrow equal to lncRNA_exp}

\item{cor_cutoff}{cor estimate cut_off, default 0}
}
\value{
a list with cor.test result,names are lncRNAs, element are mRNAs
}
\description{
make cor.test for given lncRNA and mRNA
}
\examples{
# to update
}
\seealso{
\code{\link{hypertest}}
}
\author{
Xiaojie Sun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_plots.R
\name{corheatmap}
\alias{corheatmap}
\title{corheatmap}
\usage{
corheatmap(exp, x, y, color = c("#2fa1dd", "white", "#f87669"))
}
\arguments{
\item{exp}{A numeric matrix}

\item{x}{genes or cells from exp}

\item{y}{genes or cells from exp}

\item{color}{color for heatmap}
}
\value{
a ggplot object
}
\description{
draw cor heatmap
}
\examples{
\dontshow{if (requireNamespace("Hmisc", quietly = TRUE) && requireNamespace("pheatmap", quietly = TRUE)) withAutoprint(\{ # examplesIf}
x = rownames(exprSet_hub1)[1:3]
y = rownames(exprSet_hub1)[4:7]
corheatmap(exprSet_hub1,x,y)
\dontshow{\}) # examplesIf}
}
\author{
Xiaojie Sun
}
